@@ -1,19 +1,17 @@
-/**
-* Copyright 2005-2007 ECMWF
-* 
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*
- * C Implementation: print_data
- *
- * Description: prints all the data contained in a grib file
- *
- * Author: Enrico Fucile <enrico.fucile@ecmwf.int>
+ * C Implementation: set_data
  *
+ * Description: set the data contained in a grib file
  *
  */
 #include <stdio.h>
@@ -31,7 +29,7 @@ int main(int argc, char** argv) {
   double *values = NULL;
   size_t values_len= 0;
 
-  char* filename ;
+  const char* filename = "regular_ll_pl_grib1";
   grib_handle *h = NULL;
   double d,e;
   long count;
@@ -59,7 +57,7 @@ int main(int argc, char** argv) {
 
   GRIB_CHECK(grib_set_long(h,"bitsPerValue",16),0);
 
-  /* get data values*/
+  /* set data values*/
   GRIB_CHECK(grib_set_double_array(h,"values",values,values_len),0);
 
   grib_write_message(h,argv[1],"w");
