@@ -1,9 +1,11 @@
-! Copyright 2005-2007 ECMWF
+! Copyright 2005-2013 ECMWF.
+!
+! This software is licensed under the terms of the Apache Licence Version 2.0
+! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 ! 
-! Licensed under the GNU Lesser General Public License which
-! incorporates the terms and conditions of version 3 of the GNU
-! General Public License.
-! See LICENSE and gpl-3.0.txt for details.
+! In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+! virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+!
 !
 !
 !  Description: how to create a multi field message in memory and write
@@ -40,6 +42,9 @@ program multi_write
 !  write messages to a file
   call grib_multi_write(multi_gribid,outfile)
 
+  call grib_release(in_gribid)
+  call grib_release(multi_gribid)
+
   call grib_close_file(infile)
   call grib_close_file(outfile)
 
