@@ -15,8 +15,8 @@ def example():
     f.close()
 
     keys = [
-        'numberOfPointsAlongAParallel',
-        'numberOfPointsAlongAMeridian',
+        'Ni',
+        'Nj',
         'latitudeOfFirstGridPointInDegrees',
         'longitudeOfFirstGridPointInDegrees',
         'latitudeOfLastGridPointInDegrees',
