@@ -10,8 +10,8 @@ def example():
     f = open(INPUT)
 
     keys = [
-        'numberOfPointsAlongAParallel',
-        'numberOfPointsAlongAMeridian',
+        'Ni',
+        'Nj',
         'latitudeOfFirstGridPointInDegrees',
         'longitudeOfFirstGridPointInDegrees',
         'latitudeOfLastGridPointInDegrees',
@@ -23,8 +23,11 @@ def example():
         if gid is None: break
 
         for key in keys:
+            if not grib_is_defined(gid,key): raise Exception("Key was not defined")
             print '%s=%s' % (key,grib_get(gid,key))
 
+        if grib_is_defined(gid,"A_very_silly_girl"): raise Exception("Key was defined")
+
         print 'There are %d values, average is %f, min is %f, max is %f' % (
                   grib_get_size(gid,'values'),
                   grib_get(gid,'average'),
