@@ -1,4 +1,4 @@
-public :: grib_get, grib_set, grib_get_data, grib_is_missing
+public :: grib_get, grib_set, grib_get_data, grib_is_missing, grib_is_defined
 public :: grib_open_file, grib_close_file,grib_read_bytes,grib_write_bytes
 public :: grib_multi_support_on, grib_multi_support_off
 public :: grib_keys_iterator_new, &
@@ -12,7 +12,7 @@ public :: grib_keys_iterator_get_name, &
           grib_keys_iterator_rewind
 public :: grib_new_from_message, grib_new_from_template, &
           grib_new_from_samples, grib_new_from_file, &
-		  grib_read_from_file
+		  grib_read_from_file,grib_headers_only_new_from_file
 public :: grib_release
 public :: grib_dump
 public :: grib_get_error_string
@@ -22,4 +22,5 @@ public :: grib_write, grib_multi_append
 public :: grib_check
 public :: grib_clone, grib_copy_namespace
 public :: grib_index_get,grib_index_select,&
-          grib_index_create,grib_index_get_size,grib_index_release
+          grib_index_create,grib_index_get_size,grib_index_release,&
+		  grib_util_sections_copy
