@@ -73,10 +73,7 @@
   !> @param message     array containing the coded message
   !> @param status      GRIB_SUCCESS if OK, integer value on error
   interface grib_new_from_message
-      module procedure grib_new_from_message_int2
       module procedure grib_new_from_message_int4
-      module procedure grib_new_from_message_real4
-      module procedure grib_new_from_message_real8
       module procedure grib_new_from_message_char
   end interface grib_new_from_message
 
@@ -92,10 +89,7 @@
   !> @param nbytes      number of bytes to be read
   !> @param status      GRIB_SUCCESS if OK, integer value on error
   interface grib_read_from_file
-      module procedure grib_read_from_file_int2
       module procedure grib_read_from_file_int4
-      module procedure grib_read_from_file_real4
-      module procedure grib_read_from_file_real8
       module procedure grib_read_from_file_char
   end interface grib_read_from_file
 
@@ -111,11 +105,10 @@
   !> @param nbytes      number of bytes to be read
   !> @param status      GRIB_SUCCESS if OK, integer value on error
   interface grib_read_bytes
-      module procedure grib_read_bytes_int2
       module procedure grib_read_bytes_int4
-      module procedure grib_read_bytes_real4
-      module procedure grib_read_bytes_real8
       module procedure grib_read_bytes_char
+      module procedure grib_read_bytes_real8
+      module procedure grib_read_bytes_real4
   end interface grib_read_bytes
 
   !> Write nbytes bytes from the buffer in a file opened with grib_open_file.
@@ -130,11 +123,10 @@
   !> @param nbytes      number of bytes to be written
   !> @param status      GRIB_SUCCESS if OK, integer value on error
   interface grib_write_bytes
-      module procedure grib_write_bytes_int2
       module procedure grib_write_bytes_int4
-      module procedure grib_write_bytes_real4
-      module procedure grib_write_bytes_real8
       module procedure grib_write_bytes_char
+      module procedure grib_write_bytes_real8
+      module procedure grib_write_bytes_real4
   end interface grib_write_bytes
 
 
