@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 #include "grib_fortran_prototypes.h"
@@ -24,6 +25,12 @@
 
 #include <ctype.h>
 
+/* Have file ids distinct from grib ids, in order to be
+*  protected against user errors where a file id is given
+*  instead of a grib id or viceversa
+*/
+#define MIN_FILE_ID 50000
+
 #if GRIB_PTHREADS
 static pthread_once_t once  = PTHREAD_ONCE_INIT;
 static pthread_mutex_t handle_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -145,7 +152,7 @@ static int push_file(FILE* f,char* buffer){
   l_grib_file* current  = file_set;
   l_grib_file* previous = file_set;
   l_grib_file* new      = NULL;
-  int myindex = 1;
+  int myindex = MIN_FILE_ID;
 
   if(!file_set){
     file_set = malloc(sizeof(l_grib_file));
@@ -184,6 +191,7 @@ static void _push_handle(grib_handle *h,int *gid){
   l_grib_handle* new= NULL;
   int myindex= 1;
 
+  /*
   if (*gid > 0 ) {
     while(current) {
       if(current->id == *gid) break;
@@ -195,8 +203,7 @@ static void _push_handle(grib_handle *h,int *gid){
       return;
     }
   }
-
-  current= handle_set;
+  */
 
   if(!handle_set){
     handle_set = malloc(sizeof(l_grib_handle));
@@ -207,6 +214,8 @@ static void _push_handle(grib_handle *h,int *gid){
     return;
   }
 
+  current= handle_set;
+
   while(current){
     if(current->id < 0){
       current->id = -(current->id);
@@ -237,6 +246,7 @@ static void _push_index(grib_index *h,int *gid){
   l_grib_index* new= NULL;
   int myindex= 1;
 
+/*
   if (*gid > 0 ) {
     while(current) {
       if(current->id == *gid) break;
@@ -248,8 +258,7 @@ static void _push_index(grib_index *h,int *gid){
       return;
     }
   }
-
-  current= index_set;
+  */
 
   if(!index_set){
     index_set = malloc(sizeof(l_grib_index));
@@ -260,6 +269,8 @@ static void _push_index(grib_index *h,int *gid){
     return;
   }
 
+  current= index_set;
+
   while(current){
     if(current->id < 0){
       current->id = -(current->id);
@@ -290,6 +301,7 @@ static void _push_multi_handle(grib_multi_handle *h,int *gid){
   l_grib_multi_handle* new= NULL;
   int myindex= 1;
 
+/*
   if (*gid > 0 ) {
     while(current) {
       if(current->id == *gid) break;
@@ -301,8 +313,7 @@ static void _push_multi_handle(grib_multi_handle *h,int *gid){
       return;
     }
   }
-
-  current= multi_handle_set;
+  */
 
   if(!multi_handle_set){
     multi_handle_set = malloc(sizeof(l_grib_multi_handle));
@@ -313,6 +324,8 @@ static void _push_multi_handle(grib_multi_handle *h,int *gid){
     return;
   }
 
+  current= multi_handle_set;
+
   while(current){
     if(current->id < 0){
       current->id = -(current->id);
@@ -515,6 +528,9 @@ static grib_multi_handle* get_multi_handle(int multi_handle_id){
 
 static FILE* get_file(int file_id){
   l_grib_file* current  = file_set;
+
+  if ( file_id < MIN_FILE_ID ) return NULL;
+
   while(current){
     if(current->id == file_id) return current->f;
     current = current->next;
@@ -594,7 +610,10 @@ static int _clear_index(int index_id){
   while(current){
     if(current->id == index_id){
       current->id = -(current->id);
-      /*if (current->h) return grib_index_delete(current->h);*/
+      if (current->h) {
+	  	grib_index_delete(current->h);
+		return GRIB_SUCCESS;
+	  }
     }
     current = current->next;
   }
@@ -841,16 +860,22 @@ int grib_f_close_file__(int* fid){
 int grib_f_close_file(int* fid){
   return grib_f_close_file_(fid);
 }
+static int file_count=0;
 
 void grib_f_write_on_fail(int* gid) {
 	grib_context* c=grib_context_get_default();
 	if (c->write_on_fail) {
 		char filename[100]={0,};
 		grib_handle* h=NULL;
-		clock_t t;
-		t=times(NULL);
-		t=t<0?-t:t;
-		sprintf(filename,"%ld_error.grib",(long)t);
+		pid_t pid=getpid();
+
+		GRIB_PTHREAD_ONCE(&once,&init)
+		GRIB_MUTEX_LOCK(&handle_mutex)
+		file_count++;
+		GRIB_MUTEX_UNLOCK(&handle_mutex)
+
+		sprintf(filename,"%ld_%d_error.grib",(long)pid,file_count);
+
 		h=get_handle(*gid);
 		if (h) grib_write_message(h,filename,"w");
 	}
@@ -1109,7 +1134,7 @@ int grib_f_keys_iterator_get_name_(int* iterid,char* name,int len) {
 
   fort_char_clean(name,len);
 
-  sprintf(buf,grib_keys_iterator_get_name(kiter));
+  sprintf(buf, "%s", grib_keys_iterator_get_name(kiter));
   lsize=strlen(buf);
   if (len < lsize) return GRIB_ARRAY_TOO_SMALL;
 
@@ -1403,14 +1428,35 @@ int grib_f_index_new_from_file_(char* file ,char* keys ,int* gid, int lfile, int
   return GRIB_INVALID_FILE;
 }
 
-int grib_f_index_new_from_file__(char* file ,                                char* keys ,int* gid, int lfile, int lkeys){
+int grib_f_index_new_from_file__(char* file, char* keys, int* gid, int lfile, int lkeys){
   return grib_f_index_new_from_file_(file ,keys ,gid, lfile, lkeys);
 }
 
-int grib_f_index_new_from_file(char* file ,                                char* keys ,int* gid, int lfile, int lkeys){
+int grib_f_index_new_from_file(char* file, char* keys, int* gid, int lfile, int lkeys){
   return grib_f_index_new_from_file_(file ,keys ,gid, lfile, lkeys);
 }
 
+int grib_f_index_add_file_(int* iid, char* file, int lfile) {
+  grib_index *i = get_index(*iid);
+  int err = GRIB_SUCCESS;
+  char buf[1024];
+
+  if (!i) {
+    return GRIB_INVALID_INDEX;
+  } else {
+    err = grib_index_add_file(i,cast_char(buf,file,lfile));
+    return err;
+  }
+}
+
+int grib_f_index_add_file__(int* iid, char* file, int lfile) {
+  return grib_f_index_add_file_(iid,file,lfile);
+}
+
+int grib_f_index_add_file(int* iid, char* file, int lfile) {
+  return grib_f_index_add_file_(iid,file,lfile);
+}
+
 int grib_f_index_read_(char* file, int* gid, int lfile) {
   int err = 0;
   char fname[1024]={0,};
@@ -1940,6 +1986,21 @@ int grib_f_is_missing(int* gid, char* key,int* isMissing,int len){
 	return grib_f_is_missing_(gid,key,isMissing,len);
 }
 
+int grib_f_is_defined_(int* gid, char* key,int* isDefined,int len){
+	grib_handle *h = get_handle(*gid);
+	char buf[1024];
+	if(!h)  return GRIB_INVALID_GRIB;
+
+	*isDefined=grib_is_defined(h, cast_char(buf,key,len));
+	return GRIB_SUCCESS;
+}
+int grib_f_is_defined__(int* gid, char* key,int* isDefined,int len){
+	return grib_f_is_defined_(gid,key,isDefined,len);
+}
+int grib_f_is_defined(int* gid, char* key,int* isDefined,int len){
+	return grib_f_is_defined_(gid,key,isDefined,len);
+}
+
 int grib_f_set_real4_(int* gid, char* key, float* val, int len){
 
   grib_handle *h = get_handle(*gid);
