@@ -109,6 +109,9 @@ int grib_f_new_from_index(int *iid, int *gid);
 int grib_f_index_new_from_file_(char *file, char *keys, int *gid, int lfile, int lkeys);
 int grib_f_index_new_from_file__(char *file, char *keys, int *gid, int lfile, int lkeys);
 int grib_f_index_new_from_file(char *file, char *keys, int *gid, int lfile, int lkeys);
+int grib_f_index_add_file_(int* iid, char* file, int lfile);
+int grib_f_index_add_file__(int* iid, char* file, int lfile);
+int grib_f_index_add_file(int* iid, char* file, int lfile);
 int grib_f_index_read_(char *file, int *gid, int lfile);
 int grib_f_index_read__(char *file, int *gid, int lfile);
 int grib_f_index_read(char *file, int *gid, int lfile);
@@ -185,6 +188,7 @@ int grib_f_set_missing_(int *gid, char *key, int len);
 int grib_f_set_missing__(int *gid, char *key, int len);
 int grib_f_set_missing(int *gid, char *key, int len);
 int grib_f_is_missing_(int *gid, char *key, int *isMissing, int len);
+int grib_f_is_defined_(int *gid, char *key, int *isDefined, int len);
 int grib_f_is_missing__(int *gid, char *key, int *isMissing, int len);
 int grib_f_is_missing(int *gid, char *key, int *isMissing, int len);
 int grib_f_set_real4_(int *gid, char *key, float *val, int len);
