@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 #include "assert.h"
@@ -568,7 +569,10 @@ static int _clear_index(int index_id){
   while(current){
     if(current->id == index_id){
       current->id = -(current->id);
-      /*if (current->h) return grib_index_delete(current->h);*/
+      if (current->h) {
+        grib_index_delete(current->h);
+        return GRIB_SUCCESS;
+      }
     }
     current = current->next;
   }
@@ -988,6 +992,46 @@ int grib_c_count_in_file(FILE* f,int* n) {
   return err;
 }
 
+int grib_c_new_gts_from_file(FILE* f, int* gid){
+  grib_handle *h = NULL;
+  int err = 0;
+
+  if(f){
+    h = grib_gts_handle_new_from_file(0,f,&err);
+
+    if(h){
+      push_handle(h,gid);
+      return GRIB_SUCCESS;
+    } else {
+      *gid=-1;
+      return GRIB_END_OF_FILE;
+    }
+  }
+
+  *gid=-1;
+  return GRIB_INVALID_FILE;
+}
+
+int grib_c_new_bufr_from_file(FILE* f, int* gid){
+  grib_handle *h = NULL;
+  int err = 0;
+
+  if(f){
+    h = grib_bufr_handle_new_from_file(0,f,&err);
+
+    if(h){
+      push_handle(h,gid);
+      return GRIB_SUCCESS;
+    } else {
+      *gid=-1;
+      return GRIB_END_OF_FILE;
+    }
+  }
+
+  *gid=-1;
+  return GRIB_INVALID_FILE;
+}
+
 int grib_c_new_from_file(FILE* f, int* gid, int headers_only){
   grib_handle *h = NULL;
   int err = 0;
@@ -1050,6 +1094,18 @@ int grib_c_index_new_from_file(char* file ,char* keys ,int* gid){
   return GRIB_INVALID_FILE;
 }
 
+int grib_c_index_add_file(int* iid, char* file) {
+  grib_index *i = get_index(*iid);
+  int err = GRIB_SUCCESS;
+
+  if (!i) {
+    return GRIB_INVALID_INDEX;
+  } else {
+    err = grib_index_add_file(i,file);
+    return err;
+  }
+}
+
 int grib_c_index_release(int* hid){
   return clear_index(*hid);
 }
@@ -1113,6 +1169,16 @@ int grib_c_get_size_int(int* gid, char* key, int* val){
   }
 }
 
+int grib_c_get_string_length(int* gid, char* key, size_t* val){
+  grib_handle *h = get_handle(*gid);
+
+  if(!h)
+    return GRIB_INVALID_GRIB;
+  else
+    return grib_get_string_length(h, key, val);
+
+}
+
 int grib_c_get_size_long(int* gid, char* key, long* val){
   grib_handle *h = get_handle(*gid);
   int err = GRIB_SUCCESS;
@@ -1251,7 +1317,7 @@ int grib_c_index_get_string(int* gid, char* key, char* val, int *eachsize,int* s
 	int j;
     if (*eachsize < l ) {
 	  grib_context_free(h->context,bufval);
-	  printf("eachsize=%d strlen(bufval[i])=%d\n",*eachsize,strlen(bufval[i]));
+	  printf("eachsize=%d strlen(bufval[i])=%d\n",*eachsize,(unsigned int)strlen(bufval[i]));
 	  return GRIB_ARRAY_TOO_SMALL;
 	}
 	memcpy(p,bufval[i],l);
@@ -1369,6 +1435,29 @@ int grib_c_set_missing(int* gid, char* key){
   return grib_set_missing(h, key);
 }
 
+/*
+ * Submits all keys with the given values in one go.
+ * 'keyvals' is a string of the form:
+ *    key1=val1,key2=val2,key3=val3
+ */
+int grib_c_set_key_vals(int* gid, char* keyvals)
+{
+	grib_handle* h = get_handle(*gid);
+	if (!h) return GRIB_INVALID_GRIB;
+	{
+		int err = GRIB_SUCCESS;
+		grib_values values[1024]={{0,},};
+		int count = 1000; /* max. num key/val pairs */
+		if ((err = parse_keyval_string(NULL, keyvals, 1, GRIB_TYPE_UNDEFINED, values, &count)) != GRIB_SUCCESS) {
+			return err;
+		}
+		if ((err = grib_set_values(h,values,count)) != GRIB_SUCCESS) {
+			return err;
+		}
+	}
+	return GRIB_SUCCESS;
+}
+
 int grib_c_is_missing(int* gid, char* key,int* isMissing){
   int err=0;
 	grib_handle *h = get_handle(*gid);
@@ -1378,6 +1467,14 @@ int grib_c_is_missing(int* gid, char* key,int* isMissing){
 	return err;
 }
 
+int grib_c_is_defined(int* gid, char* key,int* isDefined){
+	grib_handle *h = get_handle(*gid);
+	if(!h) return GRIB_INVALID_GRIB;
+
+	*isDefined=grib_is_defined(h, key);
+	return GRIB_SUCCESS;
+}
+
 int grib_c_set_real4(int* gid, char* key, float* val){
 
   grib_handle *h = get_handle(*gid);
@@ -1547,7 +1644,7 @@ int grib_c_set_double(int *gid, char *key, double *val) {
 
   if(!h) return GRIB_INVALID_GRIB;
 
-  return grib_get_double(h, key, val);
+  return grib_set_double(h, key, *val);
 }
 
 int grib_c_get_real8_element(int* gid, char* key,int* index, double* val){
@@ -1574,7 +1671,7 @@ int grib_c_find_nearest_four_single(int* gid,int* is_lsm,
     double* values,double* distances,
     int* indexes) {
     grib_nearest* nearest=NULL;
-    int err=0;
+    int err=0, result=0;
     unsigned long flags=0;
     size_t len=4;
     grib_handle *h = get_handle(*gid);
@@ -1584,8 +1681,10 @@ int grib_c_find_nearest_four_single(int* gid,int* is_lsm,
     nearest=grib_nearest_new(h,&err);
     if (err!=GRIB_SUCCESS) return err;
 
-    return grib_nearest_find(nearest,h,*inlat,*inlon,
-    flags,outlats,outlons,values,distances,indexes,&len);
+	result = grib_nearest_find(nearest,h,*inlat,*inlon,
+		flags,outlats,outlons,values,distances,indexes,&len);
+	grib_nearest_delete(nearest);
+	return result;
 }
 
 int grib_c_find_nearest_single(int* gid,int* is_lsm,
@@ -1841,23 +1940,29 @@ int grib_c_index_read(char* file, int* gid) {
 
 void no_fail_on_wrong_length(int flag) {
   grib_context *c = grib_context_get_default();
+  int value;
   assert(c != NULL);
-  int value = (flag != 0) ? 1 : 0;
+  value = (flag != 0) ? 1 : 0;
   c->no_fail_on_wrong_length = value;
 }
 
-void grib_c_gts_header_on() {
+void grib_c_gts_header_on(void) {
   grib_context *c = grib_context_get_default();
   assert(c != NULL);
   grib_gts_header_on(c);
 }
 
-void grib_c_gts_header_off() {
+void grib_c_gts_header_off(void) {
   grib_context *c = grib_context_get_default();
   assert(c != NULL);
   grib_gts_header_off(c);
 }
 
-long grib_c_get_api_version() {
+long grib_c_get_api_version(void) {
   return grib_get_api_version();
 }
+
+int grib_c_get_message(int *gid, const void **msg, size_t *size) {
+  grib_handle *h = get_handle(*gid);
+  return grib_get_message(h,msg,size);
+}
