@@ -31,7 +31,9 @@ NumPy support can be disabled by using the '--disable-numpy' flag.
 import gribapi_swig as _internal
 import types
 import sys
+import os
 from array import array
+from functools import wraps
 #import inspect
 
 KEYTYPES = {
@@ -40,11 +42,18 @@ KEYTYPES = {
     3:str,
 }
 
+# GRIB-51 Skip function arguments type checking if the
+# environment variable is defined
+no_type_checks = os.environ.get('GRIB_API_PYTHON_NO_TYPE_CHECKS') is not None
+
 # function arguments type checking decorator
 # got inspired from http://code.activestate.com/recipes/454322-type-checking-decorator/
 # modified to support multiple allowed types and all types in the same decorator call
 def require(**_params_):
     def check_types(_func_, _params_ = _params_):
+        if no_type_checks:
+            return _func_
+        @wraps(_func_)
         def modified(*args, **kw):
             arg_names = _func_.func_code.co_varnames
             #argnames, varargs, kwargs, defaults = inspect.getargspec(_func_)
@@ -64,8 +73,10 @@ class GribInternalError(Exception):
     @brief Wrap errors coming from the C API in a Python exception object.
     """
     def __init__(self, value):
+        # Call the base class constructor with the parameters it needs
+        Exception.__init__(self, value)
         if type(value) is int:
-            err,self.msg = _internal.grib_c_get_error_string(value,1024)
+            err, self.msg = _internal.grib_c_get_error_string(value, 1024)
             assert(err == 0)
         else:
             self.msg = value
@@ -78,7 +89,23 @@ class Bunch(dict):
     """
     def __init__(self, **kw):
         dict.__init__(self, kw)
-        self.__dict__ = self
+        self.__dict__.update(kw)
+
+    def __setitem__(self, key, value):
+        dict.__setitem__(self, key, value)
+        self.__dict__[key] = value
+
+    def __setattr__(self, key, value):
+        dict.__setitem__(self, key, value)
+        self.__dict__[key] = value
+
+    def __delitem__(self, key):
+        dict.__delitem__(self, key)
+        del self.__dict__[key]
+
+    def __delattr__(self, key):
+        dict.__delitem__(self, key)
+        del self.__dict__[key]
 
     def __str__(self):
         state = ["%s=%r" % (attribute, value)
@@ -87,6 +114,10 @@ class Bunch(dict):
         return '\n'.join(state)
 
 def with_numpy():
+    """
+    @brief Is numpy enabled?
+    @return 0->disabled, 1->enabled
+    """
     numpy = 0
     try:
         _internal.with_numpy()
@@ -108,8 +139,54 @@ def GRIB_CHECK(errid):
     if errid:
         raise GribInternalError(errid)
 
-@require(file=file)
-def grib_new_from_file(file,headers_only = False):
+@require(fileobj=file)
+def gts_new_from_file(fileobj):
+    """
+    @brief Load in memory a GTS message from a file.
+    
+    The message can be accessed through its id and it will be available\n
+    until @ref grib_release is called.\n
+
+    \b Examples: \ref get.py "get.py"
+    
+    @param fileobj        python file object
+    @return id of the GTS loaded in memory
+    @exception GribInternalError 
+    """
+    err, gribid = _internal.grib_c_new_gts_from_file(fileobj, 0)
+    if err:
+        if err == _internal.GRIB_END_OF_FILE:
+            return None
+        else:
+            GRIB_CHECK(err)
+    else:
+        return gribid
+
+@require(fileobj=file)
+def bufr_new_from_file(fileobj):
+    """
+    @brief Load in memory a BUFR message from a file.
+    
+    The message can be accessed through its id and it will be available\n
+    until @ref grib_release is called.\n
+
+    \b Examples: \ref get.py "get.py"
+    
+    @param fileobj        python file object
+    @return id of the BUFR loaded in memory
+    @exception GribInternalError 
+    """
+    err, gribid = _internal.grib_c_new_bufr_from_file(fileobj,0)
+    if err:
+        if err == _internal.GRIB_END_OF_FILE:
+            return None
+        else:
+            GRIB_CHECK(err)
+    else:
+        return gribid
+
+@require(fileobj=file)
+def grib_new_from_file(fileobj, headers_only = False):
     """
     @brief Load in memory a grib message from a file.
     
@@ -125,12 +202,12 @@ def grib_new_from_file(file,headers_only = False):
     
     \b Examples: \ref get.py "get.py"
     
-    @param file           python file object
+    @param fileobj        python file object
     @param headers_only   whether or not to load the message with the headers only                            
     @return id of the grib loaded in memory
     @exception GribInternalError 
     """
-    err,gribid = _internal.grib_c_new_from_file(file,0,headers_only)
+    err, gribid = _internal.grib_c_new_from_file(fileobj,0,headers_only)
     if err:
         if err == _internal.GRIB_END_OF_FILE:
             return None
@@ -139,20 +216,20 @@ def grib_new_from_file(file,headers_only = False):
     else:
         return gribid
 
-@require(file=file)
-def grib_count_in_file(file):
+@require(fileobj=file)
+def grib_count_in_file(fileobj):
     """
     @brief Count the messages in a file.
     
     \b Examples: \ref count_messages.py "count_messages.py"
     
-    @param file    python file object
+    @param fileobj    python file object
     @return number of messages in the file
     @exception GribInternalError 
     """
-    err,n = _internal.grib_c_count_in_file(file)
+    err, num = _internal.grib_c_count_in_file(fileobj)
     GRIB_CHECK(err)
-    return n
+    return num
 
 def grib_multi_support_on():
     """
@@ -183,7 +260,7 @@ def grib_release(gribid):
     GRIB_CHECK(_internal.grib_c_release(gribid))
 
 @require(gribid=int,key=str)
-def grib_get_string(gribid,key):
+def grib_get_string(gribid, key):
     """
     @brief Get the string value of a key from a grib message.
     
@@ -192,7 +269,8 @@ def grib_get_string(gribid,key):
     @return string value of key
     @exception GribInternalError 
     """
-    err,value = _internal.grib_c_get_string(gribid,key,1024)
+    length = grib_get_string_length(gribid, key)
+    err,value = _internal.grib_c_get_string(gribid, key, length)
     GRIB_CHECK(err)
 
     stpos = value.find('\0')
@@ -202,7 +280,7 @@ def grib_get_string(gribid,key):
     return value
 
 @require(gribid=int,key=str,value=str)
-def grib_set_string(gribid,key,value):
+def grib_set_string(gribid, key, value):
     """
     @brief Set the value for a string key in a grib message.
     
@@ -229,34 +307,34 @@ def grib_gribex_mode_off():
     """
     _internal.grib_c_gribex_mode_off()
 
-@require(gribid=int,file=file)
-def grib_write(gribid,file):
+@require(gribid=int,fileobj=file)
+def grib_write(gribid,fileobj):
     """
     @brief Write a message to a file.
 
     \b Examples: \ref set.py "set.py"
     
-    @param gribid      id of the grib loaded in memory
-    @param file       python file object
+    @param gribid     id of the grib loaded in memory
+    @param fileobj    python file object
     @exception GribInternalError 
     """
-    GRIB_CHECK(_internal.grib_c_write(gribid,file))
+    GRIB_CHECK(_internal.grib_c_write(gribid,fileobj))
 
-@require(multigribid=int,file=file)
-def grib_multi_write(multigribid,file):
+@require(multigribid=int,fileobj=file)
+def grib_multi_write(multigribid,fileobj):
     """
     @brief Write a multi field message to a file.
 
     \b Examples: \ref multi_write.py "multi_write.py"
     
     @param multigribid      id of the multi field grib loaded in memory
-    @param file             python file object
+    @param fileobj          python file object
     @exception GribInternalError 
     """
-    GRIB_CHECK(_internal.grib_c_multi_write(multigribid,file))
+    GRIB_CHECK(_internal.grib_c_multi_write(multigribid,fileobj))
 
-@require(ingribid=int,startsection=int,multigrib=int)
-def grib_multi_append(ingribid,startsection,multigrib):
+@require(ingribid=int,startsection=int,multigribid=int)
+def grib_multi_append(ingribid, startsection, multigribid):
     """
     @brief Append a single field grib message to a multi field grib message.
 
@@ -271,7 +349,7 @@ def grib_multi_append(ingribid,startsection,multigrib):
     @param multigribid   id of the output multi filed grib
     @exception GribInternalError 
     """
-    GRIB_CHECK(_internal.grib_c_multi_append(ingribid,startsection,multigrib))
+    GRIB_CHECK(_internal.grib_c_multi_append(ingribid, startsection, multigribid))
 
 @require(gribid=int,key=str)
 def grib_get_size(gribid,key):
@@ -288,6 +366,19 @@ def grib_get_size(gribid,key):
     GRIB_CHECK(err)
     return result
 
+@require(gribid=int,key=str)
+def grib_get_string_length(gribid,key):
+    """
+    @brief Get the length of the string version of a key.
+
+    @param gribid      id of the grib loaded in memory
+    @param key         name of the key
+    @exception GribInternalError 
+    """
+    err,result = _internal.grib_c_get_string_length(gribid,key)
+    GRIB_CHECK(err)
+    return result
+
 @require(iterid=int)
 def grib_skip_computed(iterid):
     """
@@ -628,10 +719,14 @@ def grib_set_double_array(gribid,key,inarray):
         a = _internal.new_doubleArray(nval)
         s = _internal.intp()
         s.assign(nval)
+
         for i in range(nval):
             _internal.doubleArray_setitem(a,i,inarray[i])
+
         GRIB_CHECK(_internal.grib_c_set_real8_array(gribid,key,a,s))
 
+        _internal.delete_doubleArray(a)
+
 @require(gribid=int,key=str)
 def grib_get_double_array(gribid,key):
     """
@@ -655,14 +750,19 @@ def grib_get_double_array(gribid,key):
         a = _internal.new_doubleArray(nval)
         s = _internal.intp()
         s.assign(nval)
+
         GRIB_CHECK(_internal.grib_c_get_real8_array(gribid,key,a,s))
+
         result = array("d")
         for i in range(nval):
             result.append(_internal.doubleArray_getitem(a,i))
+
+        _internal.delete_doubleArray(a)
+
         return result
 
 @require(gribid=int,key=str)
-def grib_set_long_array(gribid,key,value):
+def grib_set_long_array(gribid, key, inarray):
     """
     @brief Set the value of the key to an integer array.
 
@@ -674,20 +774,24 @@ def grib_set_long_array(gribid,key,value):
     
     @param gribid      id of the grib loaded in memory
     @param key         key name
-    @param value       tuple,list,python array,numpy array
+    @param inarray     tuple,list,python array,numpy array
     @exception GribInternalError 
     """
     if with_numpy():
-        GRIB_CHECK(_internal.grib_set_long_ndarray(gribid,key,value))
+        GRIB_CHECK(_internal.grib_set_long_ndarray(gribid, key, inarray))
     else:
         nval = len(inarray)
         a = _internal.new_longArray(nval)
         s = _internal.intp()
         s.assign(nval)
+
         for i in range(nval):
             _internal.longArray_setitem(a,i,inarray[i])
+
         GRIB_CHECK(_internal.grib_c_set_long_array(gribid,key,a,s))
 
+        _internal.delete_longArray(a)
+
 @require(gribid=int,key=str)
 def grib_get_long_array(gribid,key):
     """
@@ -711,10 +815,15 @@ def grib_get_long_array(gribid,key):
         a = _internal.new_longArray(nval)
         s = _internal.intp()
         s.assign(nval)
+
         GRIB_CHECK(_internal.grib_c_get_long_array(gribid,key,a,s))
+
         result = array("l")
         for i in range(nval):
             result.append(_internal.longArray_getitem(a,i))
+
+        _internal.delete_longArray(a)
+
         return result
 
 def grib_multi_new():
@@ -743,7 +852,7 @@ def grib_multi_release(gribid):
     GRIB_CHECK(_internal.grib_c_multi_release(gribid))
 
 @require(gribid_src=int,namespace=str,gribid_dest=int)
-def grib_copy_namespace(gribid_src,namespace,gribid_dest):
+def grib_copy_namespace(gribid_src, namespace, gribid_dest):
     """
     @brief Copy the value of all the keys belonging to a namespace from the source message
     to the destination message.
@@ -753,25 +862,39 @@ def grib_copy_namespace(gribid_src,namespace,gribid_dest):
     @param namespace      namespace to be copied
     @exception GribInternalError 
     """
-    GRIB_CHECK(_internal.grib_c_copy_namespace(gribid_src,name,gribid_dest))
+    GRIB_CHECK(_internal.grib_c_copy_namespace(gribid_src, namespace, gribid_dest))
 
-@require(file=str,keys=(tuple,list))
-def grib_index_new_from_file(file,keys):
+@require(filename=str,keys=(tuple,list))
+def grib_index_new_from_file(filename, keys):
     """
     @brief Create a new index from a file.
 
     \b Examples: \ref index.py "index.py"
 
-    @param file the file to index on
-    @param keys sequence of keys to index on
+    @param filename    path of the file to index on
+    @param keys        sequence of keys to index on
     @return index id
     @exception GribInternalError 
     """
     ckeys = ",".join(keys)
-    err,iid = _internal.grib_c_index_new_from_file(file,ckeys)
+    err,iid = _internal.grib_c_index_new_from_file(filename,ckeys)
     GRIB_CHECK(err)
     return iid
 
+@require(indexid=int,filename=str)
+def grib_index_add_file(indexid, filename):
+    """
+    @brief Add a file to an index.
+
+    \b Examples: \ref index.py "index.py"
+
+    @param indexid      id of the index to add the file to
+    @param filename     path of the file to be added to index
+    @exception GribInternalError 
+    """
+    err = _internal.grib_c_index_add_file(indexid, filename)
+    GRIB_CHECK(err)
+
 @require(indexid=int)
 def grib_index_release(indexid):
     """
@@ -819,10 +942,15 @@ def grib_index_get_long(indexid,key):
     a = _internal.new_longArray(nval)
     s = _internal.intp()
     s.assign(nval)
+
     GRIB_CHECK(_internal.grib_c_index_get_long(indexid,key,a,s))
+
     result = []
     for i in range(nval):
         result.append(_internal.longArray_getitem(a,i))
+
+    _internal.delete_longArray(a)
+
     return tuple(result)
 
 def grib_index_get_string(indexid,key):
@@ -874,10 +1002,15 @@ def grib_index_get_double(iid,key):
     a = _internal.new_doubleArray(nval)
     s = _internal.intp()
     s.assign(nval)
-    GRIB_CHECK(_internal.grib_c_index_get_double(iid,key,a,s))
+
+    GRIB_CHECK(_internal.grib_c_index_get_real8(iid,key,a,s))
+
     result = []
     for i in range(nval):
         result.append(_internal.doubleArray_getitem(a,i))
+
+    _internal.delete_doubleArray(a)
+
     return tuple(result)
 
 @require(indexid=int,key=str,val=int)
@@ -910,7 +1043,7 @@ def grib_index_select_double(iid,key,val):
     @param value     value of the key to select
     @exception GribInternalError 
     """
-    GRIB_CHECK(_internal.grib_c_index_select_double(iid,key,val))
+    GRIB_CHECK(_internal.grib_c_index_select_real8(iid,key,val))
 
 @require(indexid=int,key=str,val=str)
 def grib_index_select_string(indexid,key,val):
@@ -1021,6 +1154,9 @@ def grib_get_double_elements(gribid,key,indexes):
         for i in range(psize.value()):
             result.append(_internal.doubleArray_getitem(pval,i))
 
+        _internal.delete_intArray(pidx)
+        _internal.delete_doubleArray(pval)
+
         return result
 
 def grib_get_elements(gribid,key,indexes):
@@ -1059,10 +1195,53 @@ def grib_set_missing(gribid,key):
     """
     GRIB_CHECK(_internal.grib_c_set_missing(gribid,key))
 
+@require(gribid=int)
+def grib_set_key_vals(gribid, key_vals):
+    """
+    Set the values for several keys at once in a grib message.
+    
+    @param gribid      id of the grib loaded in memory
+    @param key_vals    can be a string, list/tuple or dictionary.
+                       If a string, format must be "key1=val1,key2=val2"
+                       If a list, it must contain strings of the form "key1=val1"
+    @exception         GribInternalError
+    """
+    if (len(key_vals) == 0):
+        raise GribInternalError("Empty key/values argument")
+    keyValsString = ""
+    if isinstance(key_vals, str):
+        # Plain string. We need to do a DEEP copy so as not to change the original
+        keyValsString = ''.join(key_vals)
+    elif isinstance(key_vals, list) or isinstance(key_vals, tuple):
+        # A list of key=val strings
+        for kv in key_vals:
+            if not isinstance(kv, str):
+                raise TypeError("Invalid list/tuple element type '%s'"%kv)
+            if not '=' in str(kv):
+                raise GribInternalError("Invalid list/tuple element format '%s'"%kv)
+            if len(keyValsString) > 0:
+                keyValsString += ','
+            keyValsString += kv
+    elif isinstance(key_vals, dict):
+        # A dictionary mapping keys to values
+        for key in key_vals.iterkeys():
+            if len(keyValsString) > 0:
+                keyValsString += ','
+            keyValsString += key+'=' + str(key_vals[key])
+    else:
+        raise TypeError("Invalid argument type")
+
+    GRIB_CHECK(_internal.grib_c_set_key_vals(gribid,keyValsString))
+
 @require(gribid=int,key=str)
 def grib_is_missing(gribid,key):
     """
     @brief Check if the value of a key is MISSING.
+
+    The value of a key is considered as MISSING when all the bits assigned to it
+    are set to 1. This is different from the actual key missing from the grib message.
+    The value of a key MISSING has a special significance and that can be read about
+    in the WMO documentation.
     
     @param gribid      id of the grib loaded in memory
     @param key         key name
@@ -1073,6 +1252,19 @@ def grib_is_missing(gribid,key):
     GRIB_CHECK(err)
     return value
 
+@require(gribid=int,key=str)
+def grib_is_defined(gribid,key):
+    """
+    @brief Check if a key is defined (exists)
+    @param gribid      id of the grib loaded in memory
+    @param key         key name
+    @return            0->not defined, 1->defined
+    @exception         GribInternalError 
+    """
+    err,value = _internal.grib_c_is_defined(gribid,key)
+    GRIB_CHECK(err)
+    return value
+
 @require(gribid=int,inlat=(int,float),inlon=(int,float))
 def grib_find_nearest(gribid,inlat,inlon,is_lsm = False,npoints = 1):
     """
@@ -1113,6 +1305,12 @@ def grib_find_nearest(gribid,inlat,inlon,is_lsm = False,npoints = 1):
                 index = _internal.intArray_getitem(pidx,i), \
             ))
 
+        _internal.delete_doubleArray(poutlat)
+        _internal.delete_doubleArray(poutlon)
+        _internal.delete_doubleArray(pvalues)
+        _internal.delete_doubleArray(pdist)
+        _internal.delete_intArray(pidx)
+
         return tuple(result)
     else:
         raise ValueError("Invalid value for npoints. Expecting 1 or 4.")
@@ -1137,7 +1335,7 @@ def grib_get_native_type(gribid,key):
         return None
 
 @require(gribid=int,key=str)
-def grib_get(gribid,key,type=None):
+def grib_get(gribid,key, ktype=None):
     """
     @brief Get the value of a key in a grib message.
 
@@ -1153,24 +1351,25 @@ def grib_get(gribid,key,type=None):
     
     @param gribid      id of the grib loaded in memory
     @param key         key name
+    @param ktype       the type we want the output in (int, float or str), native type if not specified
     @return scalar value of key as int, float or str
     @exception GribInternalError 
     """
-    if type is None:
-        type = grib_get_native_type(gribid,key)
+    if ktype is None:
+        ktype = grib_get_native_type(gribid,key)
 
     result = None
-    if type is int:
+    if ktype is int:
         result = grib_get_long(gribid,key)
-    elif type is float:
+    elif ktype is float:
         result = grib_get_double(gribid,key)
-    elif type is str:
+    elif ktype is str:
         result = grib_get_string(gribid,key)
 
     return result
 
 @require(gribid=int,key=str)
-def grib_get_array(gribid,key,type=None):
+def grib_get_array(gribid,key, ktype=None):
     """
     @brief Get the contents of an array key. 
     
@@ -1181,17 +1380,17 @@ def grib_get_array(gribid,key,type=None):
 
     @param gribid      id of the grib loaded in memory
     @param key         the key to get the value for
-    @param type        the type we want the output in (can be int or float), native type if not specified
+    @param ktype       the type we want the output in (can be int or float), native type if not specified
     @return numpy.ndarray or array
     @exception GribInternalError 
     """
-    if type is None:
-        type = grib_get_native_type(gribid,key)
+    if ktype is None:
+        ktype = grib_get_native_type(gribid,key)
 
     result = None
-    if type is int:
+    if ktype is int:
         result = grib_get_long_array(gribid,key)
-    elif type is float:
+    elif ktype is float:
         result = grib_get_double_array(gribid,key)
 
     return result
@@ -1277,21 +1476,21 @@ def grib_set_array(gribid,key,value):
     @exception GribInternalError 
 
     """
-    v = None
+    val0 = None
     try:
-        v = value[0]
+        val0 = value[0]
     except TypeError:
         pass
 
-    if isinstance(v,int):
-        grib_set_long_array(gribid,key,value)
-    elif isinstance(v,float):
-        grib_set_double_array(gribid,key,value)
+    if isinstance(val0, int):
+        grib_set_long_array(gribid, key, value)
+    elif isinstance(val0, float):
+        grib_set_double_array(gribid, key, value)
     else:
         raise GribInternalError("Invalid type of value when setting key '%s'." % key)
 
 @require(indexid=int,key=str)
-def grib_index_get(indexid,key,type=str):
+def grib_index_get(indexid,key, ktype=str):
     """
     @brief Get the distinct values of an index key. The key must belong to the index.
     
@@ -1306,15 +1505,15 @@ def grib_index_get(indexid,key,type=str):
     # so right now the default is str. The user can overwrite
     # the type but there is no way right now to do it automatically.
 
-    #if type is None:
-    #    type = grib_get_native_type(indexid,key)
+    #if ktype is None:
+    #    ktype = grib_get_native_type(indexid,key)
 
     result = None
-    if type is int:
+    if ktype is int:
         result = grib_index_get_long(indexid,key)
-    elif type is float:
+    elif ktype is float:
         result = grib_index_get_double(indexid,key)
-    elif type is str:
+    elif ktype is str:
         result = grib_index_get_string(indexid,key)
 
     return result
@@ -1340,8 +1539,8 @@ def grib_index_select(indexid,key,value):
     else:
         raise GribInternalError("Invalid type of value when setting key '%s'." % key)
 
-@require(indexid=int,file=str)
-def grib_index_write(indexid,file):
+@require(indexid=int,filename=str)
+def grib_index_write(indexid, filename):
     """
     @brief Write an index to a file for later reuse. 
     
@@ -1349,24 +1548,24 @@ def grib_index_write(indexid,file):
 
     \b Examples: \ref index.py "index.py"
 
-    @param indexid   id of the index
-    @param file      file to save the index to
+    @param indexid        id of the index
+    @param filename       path of file to save the index to
     @exception GribInternalError 
     """
-    GRIB_CHECK(_internal.grib_c_index_write(indexid,file))
+    GRIB_CHECK(_internal.grib_c_index_write(indexid, filename))
 
-@require(file=str)
-def grib_index_read(file):
+@require(filename=str)
+def grib_index_read(filename):
     """
     @brief Loads an index previously saved with \ref grib_index_write to a file.
 
     \b Examples: \ref index.py "index.py"
 
-    @param file    file to load the index from
+    @param filename    path of file to load the index from
     @return id of the loaded index
     @exception GribInternalError 
     """
-    err,indexid = _internal.grib_c_index_read(file)
+    err,indexid = _internal.grib_c_index_read(filename)
     GRIB_CHECK(err)
     return indexid
 
@@ -1407,3 +1606,40 @@ def grib_get_api_version():
     major = v
 
     return "%d.%d.%d" % (major,minor,revision)
+
+__version__ = grib_get_api_version()
+
+@require(gribid=int)
+def grib_get_message(gribid):
+    """
+    @brief Get the binary message.
+
+    Returns the binary string message associated with the grib identified by gribid.
+
+    @see grib_new_from_message
+    
+    @param gribid      id of the grib loaded in memory
+    @return binary string message associated with gribid
+    @exception GribInternalError 
+    """
+    error,message = _internal.grib_c_get_message(gribid)
+    GRIB_CHECK(error)
+    return message
+    
+@require(message=str)
+def grib_new_from_message(message):
+    """
+    @brief Create a grib handle from a message in memory.
+
+    Create a new grib message from the input binary string and return its grib id.
+
+    @see grib_get_message
+
+    @param message binary string message
+    @return gribid of the newly created grib message
+    @exception GribInternalError 
+    """
+    error,gribid = _internal.grib_c_new_from_message(0,message,len(message))
+    GRIB_CHECK(error)
+    return gribid
+
