@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /***************************************************************************
@@ -14,6 +15,21 @@
 
 #include "grib_api_internal.h"
 
+#if GRIB_PTHREADS
+static pthread_once_t once  = PTHREAD_ONCE_INIT;
+static pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
+
+static void init_mutex() {
+    pthread_mutexattr_t attr;
+    pthread_mutexattr_init(&attr);
+    pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init(&mutex1,&attr);
+    pthread_mutexattr_destroy(&attr);
+
+}
+#endif
+
+
 static void init(grib_action_class *c)
 {
     if(c && !c->inited)
@@ -79,8 +95,14 @@ int grib_create_accessor(grib_section* p, grib_action* a,  grib_loader* h)
     init(c);
     while(c)
     {
-        if(c->create_accessor)
-            return c->create_accessor(p, a, h);
+        if(c->create_accessor) {
+			int ret;
+			GRIB_PTHREAD_ONCE(&once,&init_mutex);
+			GRIB_MUTEX_LOCK(&mutex1);
+            ret=c->create_accessor(p, a, h);
+			GRIB_MUTEX_UNLOCK(&mutex1);
+            return ret;
+		}
         c = c->super ? *(c->super) : NULL;
     }
     fprintf(stderr,"Cannot create accessor %s %s\n",a->name,a->cclass->name);
