@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /***************************************************************************
@@ -88,8 +89,6 @@ grib_action* grib_action_create_alias(grib_context* context, const char* name, c
 
 	act->context = context;
 
-
-
 	act->op           = NULL;
 	act->name         = grib_context_strdup_persistent(context, name);
 	if(name_space)
@@ -99,36 +98,34 @@ grib_action* grib_action_create_alias(grib_context* context, const char* name, c
 	act->flags        = flags;
 	a                 = (grib_action_alias*)act;
 	a->target         = arg1 ? grib_context_strdup_persistent(context, arg1) : NULL;
-	
 
 	return act;
 }
 
 static void compile(grib_action* act, grib_compiler *compiler)
 {
-    grib_action_alias* a  = (grib_action_alias*)act;
-    fprintf(compiler->out,"%s = grib_action_create_alias(ctx,",compiler->var);
-    fprintf(compiler->out,"\"%s\",",act->name);
-    if(a->target) {
-    fprintf(compiler->out,"\"%s\",",a->target);
-    }
-    else
-    {
-    fprintf(compiler->out,"NULL,");
-    }
-    if(act->name_space) {
-        fprintf(compiler->out,"\"%s\",",act->name_space);
-    }
-    else
-    {
-        fprintf(compiler->out,"NULL,");
-    }    
-    grib_compile_flags(compiler, act->flags);
-    fprintf(compiler->out,");");
-    fprintf(compiler->out,"\n");
+	grib_action_alias* a  = (grib_action_alias*)act;
+	fprintf(compiler->out,"%s = grib_action_create_alias(ctx,",compiler->var);
+	fprintf(compiler->out,"\"%s\",",act->name);
+	if(a->target) {
+		fprintf(compiler->out,"\"%s\",",a->target);
+	}
+	else
+	{
+		fprintf(compiler->out,"NULL,");
+	}
+	if(act->name_space) {
+		fprintf(compiler->out,"\"%s\",",act->name_space);
+	}
+	else
+	{
+		fprintf(compiler->out,"NULL,");
+	}
+	grib_compile_flags(compiler, act->flags);
+	fprintf(compiler->out,");");
+	fprintf(compiler->out,"\n");
 }
 
-
 static int same(const char* a,const char* b) 
 {
 	if(a == b) return 1;
@@ -136,10 +133,9 @@ static int same(const char* a,const char* b)
 	return 0;
 }
 
-
 static int create_accessor( grib_section* p, grib_action* act,grib_loader *h)
 {
-	int i,id;
+	int i,j,id;
 	grib_action_alias* self = (grib_action_alias*)act;
 	grib_accessor *x=NULL ;
 	grib_accessor *y=NULL ;
@@ -149,14 +145,14 @@ static int create_accessor( grib_section* p, grib_action* act,grib_loader *h)
 		x = grib_find_accessor_fast(p->h,self->target);
 		if(x == NULL)
 		{
-			grib_context_log(p->h->context,GRIB_LOG_ERROR,"alias %s: cannot find %s",
-							 act->name,self->target);
+			grib_context_log(p->h->context,GRIB_LOG_WARNING,"alias %s: cannot find %s",
+					act->name,self->target);
 			return GRIB_SUCCESS;
 		}
 
 		if (x->name_space==NULL) 
 			x->name_space=act->name_space;
-		
+
 		i = 0;
 		while(i < MAX_ACCESSOR_NAMES) {
 			if(x->all_names[i] != NULL && !strcmp(x->all_names[i],act->name) ) {
@@ -179,7 +175,7 @@ static int create_accessor( grib_section* p, grib_action* act,grib_loader *h)
 			i++;
 		}
 		grib_context_log(p->h->context,GRIB_LOG_FATAL,
-						 "unable to alias %s : increase MAX_ACCESSOR_NAMES",act->name);
+				"unable to alias %s : increase MAX_ACCESSOR_NAMES",act->name);
 
 		return GRIB_INTERNAL_ERROR;
 	}
@@ -197,7 +193,7 @@ static int create_accessor( grib_section* p, grib_action* act,grib_loader *h)
 			{
 
 				grib_context_log ( p->h->context,GRIB_LOG_DEBUG,"alias %s.%s already defined for %s. Deleting old alias",
-								   act->name_space,act->name,y->name );
+						act->name_space,act->name,y->name );
 				/* printf("[%s %s]\n",y->all_names[i], y->all_name_spaces[i]); */
 
 				while ( i < MAX_ACCESSOR_NAMES-1 )
@@ -219,12 +215,12 @@ static int create_accessor( grib_section* p, grib_action* act,grib_loader *h)
 			return GRIB_SUCCESS;
 	}
 
-    if (!self->target) return GRIB_SUCCESS;
-    
+	if (!self->target) return GRIB_SUCCESS;
+
 	x = grib_find_accessor_fast(p->h,self->target);
 	if(x == NULL)
 	{
-		grib_context_log(p->h->context,GRIB_LOG_ERROR,"alias %s: cannot find %s",
+		grib_context_log(p->h->context,GRIB_LOG_WARNING,"alias %s: cannot find %s",
 				act->name,self->target);
 		return GRIB_SUCCESS;
 	}
@@ -238,13 +234,23 @@ static int create_accessor( grib_section* p, grib_action* act,grib_loader *h)
 		}
 	}
 
-
 	i = 0;
 	while(i < MAX_ACCESSOR_NAMES) {
 		if(x->all_names[i] == NULL)
 		{
-			x->all_names[i]       =  act->name;
-			x->all_name_spaces[i] =  act->name_space;
+			/* Only add entries if not already there */
+			int found = 0;
+			for(j=0; j<i && !found; ++j) {
+				int nameSame      = same(x->all_names[j], act->name);
+				int namespaceSame = same(x->all_name_spaces[j], act->name_space);
+				if (nameSame && namespaceSame) {
+					found = 1;
+				}
+			}
+			if (!found) {  /* Not there. So add them */
+				x->all_names[i]       =  act->name;
+				x->all_name_spaces[i] =  act->name_space;
+			}
 			return GRIB_SUCCESS;
 		}
 		i++;
