@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
- * 
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /***************************************************************************
@@ -20,7 +21,7 @@
    CLASS      = action
    SUPER      = action_class_section
    IMPLEMENTS = create_accessor
-   IMPLEMENTS = dump
+   IMPLEMENTS = dump;compile
    IMPLEMENTS = destroy
    IMPLEMENTS = reparse
    MEMBERS    = grib_expression *expression
@@ -41,6 +42,7 @@ or edit "action.class" and rerun ./make_class.pl
 
 static void init_class      (grib_action_class*);
 static void dump            (grib_action* d, FILE*,int);
+static void compile         (grib_action* a, grib_compiler* compiler);
 static void destroy         (grib_context*,grib_action*);
 static int create_accessor(grib_section*,grib_action*,grib_loader*);
 static grib_action* reparse(grib_action* a,grib_accessor* acc,int *doit);
@@ -73,6 +75,7 @@ static grib_action_class _grib_action_class_list = {
     0,                            /* notify_change */
     &reparse,                            /* reparse */
     0,                            /* execute */
+    &compile,                            /* compile */
 };
 
 grib_action_class* grib_action_class_list = &_grib_action_class_list;
@@ -94,42 +97,6 @@ static void dump( grib_action* act, FILE* f, int lvl)
 	grib_dump_action_branch(f,a->block_list,lvl+1);
 }
 
-#if 0
-static int create_accessor_buffer(grib_section* p, grib_action* act)
-{
-	grib_action_list* a = ( grib_action_list*)act;
-
-	grib_accessor* ga = NULL;
-	grib_section*  gs = NULL;
-	grib_action*  la = NULL;
-	grib_action*  next = NULL;
-	int ret = 0;
-	long val = 0;
-	grib_expression_evaluate_long(p->h,a->expression,&val);
-
-	grib_context_log(p->h->context, GRIB_LOG_DEBUG, " List %s creating %d values \n", act->name, val );
-
-	ga = grib_accessor_factory(p, act,0,NULL);
-	if(!ga)  return GRIB_BUFFER_TOO_SMALL;
-	/* gs = grib_get_sub_section(ga); */
-	gs = ga->sub_section;
-
-	grib_push_accessor(ga,p->block);
-
-	la = a->block_list;
-
-	while(val--){
-		next = la;
-		while(next){
-			ret =   grib_create_accessor(gs, next,NULL);
-			if(ret != GRIB_SUCCESS) return ret;
-			next= next->next;
-		}
-	}
-	return GRIB_SUCCESS;
-
-}
-#endif
 
 static int create_accessor(grib_section* p, grib_action* act,grib_loader* h)
 {
@@ -151,7 +118,6 @@ static int create_accessor(grib_section* p, grib_action* act,grib_loader* h)
 
 	ga = grib_accessor_factory(p, act,0,NULL);
 	if(!ga)  return GRIB_BUFFER_TOO_SMALL;
-	/* gs = grib_get_sub_section(ga); */
 	gs = ga->sub_section;
 	ga->loop = val;
 
@@ -194,6 +160,22 @@ grib_action* grib_action_create_list( grib_context* context, const char* name, g
 	return act;
 }
 
+static void compile(grib_action* act, grib_compiler *compiler)
+{
+    grib_action_list* a  = (grib_action_list*)act;
+    char b[80];
+
+    if(a->block_list)
+        grib_compile_action_branch(a->block_list, compiler,b); 
+    else
+        strcpy(b,"NULL");
+
+    fprintf(compiler->out,"%s = grib_action_create_list(ctx,",compiler->var);
+    fprintf(compiler->out,"\"%s\",", act->name);
+    grib_compile_expression(a->expression, compiler);
+    fprintf(compiler->out,",%s);\n",b);
+}
+
 static grib_action* reparse(grib_action* a,grib_accessor* acc,int *doit)
 {
 	grib_action_list* self = ( grib_action_list*)a;
