@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-* 
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /***************************************************************************
  *   Jean Baptiste Filippi - 01.11.2005                                                           *
@@ -74,6 +75,7 @@ static grib_action_class _grib_action_class_trigger = {
     0,                            /* notify_change */
     &reparse,                            /* reparse */
     0,                            /* execute */
+    0,                            /* compile */
 };
 
 grib_action_class* grib_action_class_trigger = &_grib_action_class_trigger;
@@ -83,6 +85,7 @@ static void init_class(grib_action_class* c)
 	c->xref	=	(*(c->super))->xref;
 	c->notify_change	=	(*(c->super))->notify_change;
 	c->execute	=	(*(c->super))->execute;
+	c->compile	=	(*(c->super))->compile;
 }
 /* END_CLASS_IMP */
 
@@ -118,40 +121,6 @@ static void dump( grib_action* act, FILE* f, int lvl)
 
 }
 
-#if 0
-
-static int create_accessor_buffer(grib_section* p, grib_action* act)
-{
-	int ret = GRIB_SUCCESS;
-	grib_action_trigger* a = ( grib_action_trigger*)act;
-	grib_action* next = NULL;
-	grib_accessor* as = NULL;
-	grib_section*         gs = NULL; 
-
-	as = grib_accessor_factory(p, act,0,NULL);
-
-	if(!as)return GRIB_INTERNAL_ERROR;
-
-	/* gs = grib_get_sub_section(as); */
-	gs = as->sub_section;
-
-	grib_push_accessor(as,p->block);
-
-	gs->branch = 0; /* Force a reparse each time */
-
-	grib_dependency_observe_arguments(as,a->trigger_on);
-
-
-	next = a->block;
-	while(next){
-		ret = grib_create_accessor_buffer(gs, next);
-		if(ret != GRIB_SUCCESS) return ret;
-		next= next->next;
-	}
-
-	return GRIB_SUCCESS;
-}
-#endif
 
 static int  create_accessor(grib_section* p, grib_action* act, grib_loader *h )
 {
@@ -166,7 +135,6 @@ static int  create_accessor(grib_section* p, grib_action* act, grib_loader *h )
 
 	if(!as) return GRIB_INTERNAL_ERROR;
 
-	/* gs = grib_get_sub_section(as); */
 	gs = as->sub_section;
 	gs->branch = 0; /* Force a reparse each time */
 
