@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /***************************************************************************
  * Enrico Fucile                                                                         *
@@ -17,7 +18,7 @@
    START_CLASS_DEF
    CLASS      = action
    IMPLEMENTS = create_accessor
-   IMPLEMENTS = dump;xref
+   IMPLEMENTS = dump;xref;compile
    IMPLEMENTS = destroy;notify_change
    MEMBERS    = grib_expression *expression
    MEMBERS    = grib_action     *block_true
@@ -40,6 +41,7 @@ or edit "action.class" and rerun ./make_class.pl
 static void init_class      (grib_action_class*);
 static void dump            (grib_action* d, FILE*,int);
 static void xref            (grib_action* d, FILE* f,const char* path);
+static void compile         (grib_action* a, grib_compiler* compiler);
 static void destroy         (grib_context*,grib_action*);
 static int create_accessor(grib_section*,grib_action*,grib_loader*);
 static int notify_change(grib_action* a, grib_accessor* observer,grib_accessor* observed);
@@ -72,6 +74,7 @@ static grib_action_class _grib_action_class_when = {
     &notify_change,                            /* notify_change */
     0,                            /* reparse */
     0,                            /* execute */
+    &compile,                            /* compile */
 };
 
 grib_action_class* grib_action_class_when = &_grib_action_class_when;
@@ -108,6 +111,27 @@ grib_action* grib_action_create_when( grib_context* context,
   return act;
 }
 
+static void compile(grib_action* act, grib_compiler *compiler)
+{
+    grib_action_when* a  = (grib_action_when*)act;
+    char t[80];
+    char f[80];
+
+    if(a->block_true)
+        grib_compile_action_branch(a->block_true, compiler,t); 
+    else
+        strcpy(t,"NULL");
+
+    if(a->block_false)
+        grib_compile_action_branch(a->block_false, compiler,f);
+    else
+        strcpy(f,"NULL");
+
+    fprintf(compiler->out,"%s = grib_action_create_when(ctx,",compiler->var);
+    grib_compile_expression(a->expression, compiler);
+    fprintf(compiler->out,",%s,%s);\n", t,f);
+}
+
 
 static int create_accessor(grib_section* p, grib_action* act,grib_loader *h)
 {
