@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 #include "grib_api_internal.h"
@@ -15,6 +16,20 @@
 /* This file is generated my ./make_class.pl */
 #include "grib_accessor_class.h"
 
+#if GRIB_PTHREADS
+static pthread_once_t once  = PTHREAD_ONCE_INIT;
+static pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
+
+static void init() {
+    pthread_mutexattr_t attr;
+    pthread_mutexattr_init(&attr);
+    pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init(&mutex1,&attr);
+    pthread_mutexattr_destroy(&attr);
+
+}
+#endif
+
 struct table_entry
 {
   char                  *type;
@@ -40,6 +55,8 @@ grib_section*  grib_create_root_section(const grib_context *context, grib_handle
   char* fpath=0;
   grib_section*   s   = (grib_section*) grib_context_malloc_clear(context,sizeof(grib_section));
 
+  GRIB_PTHREAD_ONCE(&once,&init);
+  GRIB_MUTEX_LOCK(&mutex1);
   if(h->context->grib_reader == NULL) {
 	if ((fpath=grib_context_full_path(h->context,"boot.def"))==NULL) {
       grib_context_log(h->context,GRIB_LOG_FATAL,
@@ -47,6 +64,7 @@ grib_section*  grib_create_root_section(const grib_context *context, grib_handle
     }
     grib_parse_file(h->context,fpath);
   }
+  GRIB_MUTEX_UNLOCK(&mutex1);
 
   s->h = h;
   s->aclength = NULL;
@@ -83,8 +101,8 @@ grib_accessor* grib_accessor_factory(grib_section* p, grib_action* creator,
 	grib_accessor* a=NULL;
 	size_t size=0;
 
-	/* c = get_class(p->h->context,creator->op); */
-	c=*((grib_accessor_classes_hash(creator->op,strlen(creator->op)))->cclass);
+	c = get_class(p->h->context,creator->op);
+	/* c=*((grib_accessor_classes_hash(creator->op,strlen(creator->op)))->cclass); */
 
 	a = (grib_accessor*) grib_context_malloc_clear(p->h->context,c->size);
 
@@ -341,7 +359,8 @@ void grib_update_paddings(grib_section* s)
   grib_accessor* last = NULL;
   grib_accessor* changed;
 
-  while((changed = find_paddings(s)) != NULL)
+  /* while((changed = find_paddings(s)) != NULL) */
+  while((changed = find_paddings(s->h->root)) != NULL)
   {
     Assert(changed != last);
     grib_resize(changed,grib_preferred_size(changed,0));
