@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 /*
@@ -17,6 +18,7 @@
 
    IMPLEMENTS = next_offset
    IMPLEMENTS = unpack_double;unpack_double_element
+   IMPLEMENTS = unpack_long
    IMPLEMENTS = unpack_string
    IMPLEMENTS = init;dump;update_size
    MEMBERS=const char* tableReference
@@ -38,6 +40,7 @@ or edit "accessor.class" and rerun ./make_class.pl
 */
 
 static int unpack_double(grib_accessor*, double* val,size_t *len);
+static int unpack_long(grib_accessor*, long* val,size_t *len);
 static int unpack_string (grib_accessor*, char*, size_t *len);
 static long next_offset(grib_accessor*);
 static void dump(grib_accessor*, grib_dumper*);
@@ -70,6 +73,7 @@ static grib_accessor_class _grib_accessor_class_bitmap = {
     0,                    /* free mem                       */
     &dump,                       /* describes himself         */
     &next_offset,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -78,7 +82,7 @@ static grib_accessor_class _grib_accessor_class_bitmap = {
     0,               /* grib_pack procedures long      */
     0,               /* grib_pack procedures long      */
     0,                  /* grib_pack procedures long      */
-    0,                /* grib_unpack procedures long    */
+    &unpack_long,                /* grib_unpack procedures long    */
     0,                /* grib_pack procedures double    */
     &unpack_double,              /* grib_unpack procedures double  */
     0,                /* grib_pack procedures string    */
@@ -104,6 +108,7 @@ grib_accessor_class* grib_accessor_class_bitmap = &_grib_accessor_class_bitmap;
 
 static void init_class(grib_accessor_class* c)
 {
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -112,7 +117,6 @@ static void init_class(grib_accessor_class* c)
 	c->pack_missing	=	(*(c->super))->pack_missing;
 	c->is_missing	=	(*(c->super))->is_missing;
 	c->pack_long	=	(*(c->super))->pack_long;
-	c->unpack_long	=	(*(c->super))->unpack_long;
 	c->pack_double	=	(*(c->super))->pack_double;
 	c->pack_string	=	(*(c->super))->pack_string;
 	c->pack_bytes	=	(*(c->super))->pack_bytes;
@@ -200,6 +204,30 @@ static void dump(grib_accessor* a, grib_dumper* dumper)
 }
 
 
+static int unpack_long(grib_accessor* a, long* val, size_t *len)
+{
+  long pos = a->offset*8;
+  size_t tlen;
+  long i;
+
+  tlen = grib_value_count(a);
+
+  if(*len < tlen)
+  {
+    grib_context_log(a->parent->h->context, GRIB_LOG_ERROR, "Wrong size for %s it contains %d values ", a->name , tlen );
+    *len = 0;
+    return GRIB_ARRAY_TOO_SMALL;
+  }
+
+  for(i=0;i<tlen;i++)
+  {
+    val[i] = (long)grib_decode_unsigned_long(a->parent->h->buffer->data, &pos,1);
+  }
+  *len = tlen;
+  return GRIB_SUCCESS;
+
+}
+
 static int unpack_double   (grib_accessor* a, double* val, size_t *len)
 {
   long pos = a->offset*8;
