@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*****************************************
  *  Enrico Fucile
@@ -78,6 +79,7 @@ static grib_accessor_class _grib_accessor_class_codetable = {
     &destroy,                    /* free mem                       */
     &dump,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -102,6 +104,7 @@ static grib_accessor_class _grib_accessor_class_codetable = {
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
     0,     /* unpack only ith value          */
+    0,     /* unpack a subarray         */
     0,             		/* clear          */
 };
 
@@ -112,6 +115,7 @@ grib_accessor_class* grib_accessor_class_codetable = &_grib_accessor_class_codet
 static void init_class(grib_accessor_class* c)
 {
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
 	c->sub_section	=	(*(c->super))->sub_section;
@@ -130,6 +134,7 @@ static void init_class(grib_accessor_class* c)
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
+	c->unpack_double_subarray	=	(*(c->super))->unpack_double_subarray;
 	c->clear	=	(*(c->super))->clear;
 }
 
@@ -137,7 +142,6 @@ static void init_class(grib_accessor_class* c)
 
 static int grib_load_codetable(grib_context* c,const char* filename,
            const char* recomposed_name,size_t size,grib_codetable* t); 
-
 static void init(grib_accessor* a, const long len, grib_arguments* params) {
   int n=0;
   grib_accessor_codetable* self  = (grib_accessor_codetable*)a;
@@ -152,9 +156,8 @@ static void init(grib_accessor* a, const long len, grib_arguments* params) {
 
   if (a->flags & GRIB_ACCESSOR_FLAG_TRANSIENT) {
     a->length = 0;
-	if (a->vvalue != NULL)
-		grib_context_free_persistent(a->parent->h->context, a->vvalue);
-    a->vvalue = grib_context_malloc_clear_persistent(a->parent->h->context,sizeof(grib_virtual_value));
+	if (!a->vvalue) 
+		a->vvalue = grib_context_malloc_clear(a->parent->h->context,sizeof(grib_virtual_value));
     a->vvalue->type=grib_accessor_get_native_type(a);
     a->vvalue->length=len;
     if (act->default_value!=NULL) {
@@ -202,10 +205,12 @@ static grib_codetable* load_table(grib_accessor_codetable* self)
   grib_codetable* t = NULL;
   grib_codetable* next=NULL ;
   grib_accessor* a=(grib_accessor*)self;
-  char filename[1024]={0,};
+  char *filename=0;
+  char name[1024]={0,};
   char recomposed[1024]={0,};
   char localRecomposed[1024]={0,};
-  char localFilename[1024]={0,};
+  char *localFilename=0;
+  char localName[1024]={0,};
   char masterDir[1024]={0,};
   char localDir[1024]={0,};
   size_t len=1024;
@@ -218,25 +223,25 @@ static grib_codetable* load_table(grib_accessor_codetable* self)
     grib_get_string(h,self->localDir,localDir,&len);
 
   if (*masterDir!=0) {
-    sprintf(filename,"%s/%s",masterDir,self->tablename);
-    grib_recompose_name(h, NULL,filename, recomposed,0);
-    grib_context_full_path(c,recomposed,filename);
+    sprintf(name,"%s/%s",masterDir,self->tablename);
+    grib_recompose_name(h, NULL,name, recomposed,0);
+    filename=grib_context_full_path(c,recomposed);
   } else {
     grib_recompose_name(h, NULL,self->tablename, recomposed,0);
-    grib_context_full_path(c,recomposed,filename);
+    filename=grib_context_full_path(c,recomposed);
   }
 
   if (*localDir!=0) {
-    sprintf(localFilename,"%s/%s",localDir,self->tablename);
-    grib_recompose_name(h, NULL,localFilename, localRecomposed,0);
-    grib_context_full_path(c,localRecomposed,localFilename);
+    sprintf(localName,"%s/%s",localDir,self->tablename);
+    grib_recompose_name(h, NULL,localName, localRecomposed,0);
+    localFilename=grib_context_full_path(c,localRecomposed);
   }
   
   next=c->codetable;
   while(next) {
-    if(strcmp(filename,next->filename[0]) == 0 &&
-       ((*localFilename==0 && next->filename[1]==NULL) ||
-           ((*localFilename!=0 && next->filename[1]!=NULL)
+    if((filename && next->filename[0] && strcmp(filename,next->filename[0]) == 0) &&
+       ((localFilename==0 && next->filename[1]==NULL) ||
+           ((localFilename!=0 && next->filename[1]!=NULL)
            && strcmp(localFilename,next->filename[1]) ==0)) )
       return next;
     next = next->next;
@@ -253,9 +258,9 @@ static grib_codetable* load_table(grib_accessor_codetable* self)
   t = (grib_codetable*)grib_context_malloc_clear_persistent(c,sizeof(grib_codetable) +
       (size-1)*sizeof(code_table_entry));
 
-  if (*filename!=0) grib_load_codetable(c,filename,recomposed,size,t);
+  if (filename!=0) grib_load_codetable(c,filename,recomposed,size,t);
 
-  if (*localFilename!=0) grib_load_codetable(c,localFilename,localRecomposed,size,t);
+  if (localFilename!=0) grib_load_codetable(c,localFilename,localRecomposed,size,t);
 
   if (t->filename[0]==NULL && t->filename[1]==NULL) {
     grib_context_free_persistent(c,t);
@@ -270,6 +275,7 @@ static int grib_load_codetable(grib_context* c,const char* filename,
            const char* recomposed_name,size_t size,grib_codetable* t) {
   char line[1024];
   FILE *f = NULL;
+  int lineNumber = 0;
 
   grib_context_log(c,GRIB_LOG_DEBUG,"Loading code table form %s",filename);
 
@@ -299,6 +305,7 @@ static int grib_load_codetable(grib_context* c,const char* filename,
     char* r = title;
     char* units=0;
     char unknown[]="unknown";
+	++lineNumber;
 
     line[strlen(line)-1] = 0;
 
@@ -311,6 +318,11 @@ static int grib_load_codetable(grib_context* c,const char* filename,
 
     if( *p =='\0' ) continue;
 
+	if (!isdigit(*p))
+	{
+	   grib_context_log(c,GRIB_LOG_ERROR, "Invalid entry in file %s: line %d", filename, lineNumber);
+	   continue; /* skip this line */
+	}
     Assert(isdigit(*p));
 
     while(*p != '\0')
@@ -386,7 +398,6 @@ void grib_codetable_delete(grib_context* c) {
   {
     grib_codetable* s = t->next;
     int i;
-    /* printf("freeing %s\n",t->filename); */
 
     for(i = 0; i < t->size; i++)
     {
@@ -528,6 +539,10 @@ static int pack_string(grib_accessor* a, const char* buffer, size_t *len)
   long i;
   size_t size = 1;
 
+  typedef int (*cmpproc)(const char*, const char*);
+
+  cmpproc cmp = a->flags | GRIB_ACCESSOR_FLAG_LOWERCASE ? strcasecmp : strcmp;
+
   if(!self->table) self->table = load_table(self);
   table=self->table;
 
@@ -541,7 +556,7 @@ static int pack_string(grib_accessor* a, const char* buffer, size_t *len)
 
   for(i = 0 ; i < table->size; i++)
     if(table->entries[i].abbreviation)
-		  if(strcmp(table->entries[i].abbreviation,buffer) == 0)
+		  if(cmp(table->entries[i].abbreviation,buffer) == 0)
         return grib_pack_long(a,&i,&size);
 
   if (a->flags & GRIB_ACCESSOR_FLAG_NO_FAIL) {
@@ -600,7 +615,7 @@ static int pack_expression(grib_accessor* a, grib_expression *e){
     cval = grib_expression_evaluate_string(a->parent->h,e,tmp,&len,&ret);
     if (ret!=GRIB_SUCCESS) {
       grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"grib_accessor_codetable.pack_expression: unable to evaluate string %s to be set in %s\n",grib_expression_get_name(e),a->name);
-      exit(ret);
+	  return ret;
     }
     len = strlen(cval) + 1;
     ret = grib_pack_string(a,cval,&len);
