@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 
@@ -73,6 +74,7 @@ static grib_accessor_class _grib_accessor_class_data_apply_bitmap = {
     0,                    /* free mem                       */
     &dump,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -108,6 +110,7 @@ grib_accessor_class* grib_accessor_class_data_apply_bitmap = &_grib_accessor_cla
 static void init_class(grib_accessor_class* c)
 {
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
 	c->sub_section	=	(*(c->super))->sub_section;
@@ -187,32 +190,35 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
 
   double missing_value = 0;
 
-    if(!grib_find_accessor(a->parent->h,self->bitmap))
+  if(!grib_find_accessor(a->parent->h,self->bitmap))
       return grib_get_double_array_internal(a->parent->h,self->coded_values,val,len);
 
   if((err = grib_get_size(a->parent->h,self->coded_values,&coded_n_vals)) != GRIB_SUCCESS)
     return err;
 
+  if((err = grib_get_double_internal(a->parent->h,self->missing_value,&missing_value))
+      != GRIB_SUCCESS)  return err;
+
   if(*len < n_vals)
   {
     *len = n_vals;
     return GRIB_ARRAY_TOO_SMALL;
   }
 
-  if((err = grib_get_double_internal(a->parent->h,self->missing_value,&missing_value))
-      != GRIB_SUCCESS)  return err;
+  if (coded_n_vals==0) {
+	  for(i=0;i < n_vals;i++)
+		  val[i] = missing_value;
+
+	  *len=n_vals;
+	  return GRIB_SUCCESS;
+  }
 
   if((err = grib_get_double_array_internal(a->parent->h,self->bitmap,val,&n_vals))
       != GRIB_SUCCESS)
     return err;
 
-
-  if(coded_n_vals > 0){
-    coded_vals = grib_context_malloc(a->parent->h->context,coded_n_vals*sizeof(double));
-
-    if(coded_vals == NULL)
-      return GRIB_OUT_OF_MEMORY;
-  }
+  coded_vals = grib_context_malloc(a->parent->h->context,coded_n_vals*sizeof(double));
+  if(coded_vals == NULL) return GRIB_OUT_OF_MEMORY;
 
   if((err = grib_get_double_array_internal(a->parent->h,self->coded_values,coded_vals,&coded_n_vals))
       != GRIB_SUCCESS)
@@ -331,16 +337,12 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
     } 
   }
 
-  if (j) {
-    err = grib_set_double_array_internal(a->parent->h,self->coded_values,coded_vals,j);
-  } else {
-    j=*len;
-    err = grib_set_double_array_internal(a->parent->h,self->coded_values,coded_vals,j);
+  err = grib_set_double_array_internal(a->parent->h,self->coded_values,coded_vals,j);
+  if (j==0) {
     if (self->number_of_values)
       err=grib_set_long_internal(a->parent->h,self->number_of_values,0);
-    if (self->binary_scale_factor) {
+    if (self->binary_scale_factor) 
       err=grib_set_long_internal(a->parent->h,self->binary_scale_factor,0);
-    }
   }
 
   grib_context_free(a->parent->h->context,coded_vals);
