@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 #include <math.h>
@@ -60,6 +61,7 @@ typedef struct grib_accessor_data_complex_packing {
 	const char* offsetsection;
 	int dirty;
 /* Members defined in data_simple_packing */
+	int edition;
 	const char*  units_factor;
 	const char*  units_bias;
 	const char*  changing_precision;
@@ -94,6 +96,7 @@ static grib_accessor_class _grib_accessor_class_data_complex_packing = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -130,6 +133,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
 	c->get_native_type	=	(*(c->super))->get_native_type;
@@ -246,6 +250,7 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   long   pen_m= 0;
 
   double operat= 0;
+  int bytes;
 
   decode_float_proc decode_float = NULL;
 
@@ -297,12 +302,15 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   switch (ieee_floats) {
     case 0:
       decode_float=grib_long_to_ibm;
+	  bytes=4;
       break;
     case 1:
       decode_float=grib_long_to_ieee;
+	  bytes=4;
       break;
     case 2:
       decode_float=grib_long_to_ieee64;
+	  bytes=8;
       break;
     default:
       return GRIB_NOT_IMPLEMENTED;
@@ -321,6 +329,16 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   hres = buf;
   lres = buf;
 
+  if (pen_j == sub_j) {
+	n_vals = (pen_j+1)*(pen_j+2);
+	d = grib_power(-decimal_scale_factor,10) ;
+  	grib_ieee_decode_array(a->parent->h->context,buf,n_vals,bytes,val);
+	if (d) {
+		for (i=0;i<n_vals;i++) val[i]*=d;
+	}
+	return 0;
+  }
+
   packed_offset = grib_byte_offset(a) +  4*(sub_k+1)*(sub_k+2);
 
   lpos = 8*(packed_offset-offsetdata);
@@ -424,6 +442,10 @@ static double calculate_pfactor(grib_context *ctx,const double* spectralField, l
   double weightedSumOverX = 0.0, weightedSumOverY = 0.0, sumOfWeights = 0.0, x, y;
   double numerator = 0.0, denominator = 0.0, slope;
 
+  /* Catch corner case. See GRIB-172 */
+  if (ismax-ismin <= 1) {
+    return 1; /* any value will do! we cannot do linear fit on a single point! */
+  }
   /*
   // Setup the weights
    */
@@ -564,6 +586,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len) {
   double max = 0;
   double current_val = 0;
   short mixmax_unset = 0;
+  int bytes;
 
   encode_float_proc encode_float = NULL;
 
@@ -607,12 +630,15 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len) {
   switch (ieee_floats) {
     case 0:
       encode_float =grib_ibm_to_long;
+	  bytes=4;
       break;
     case 1:
       encode_float =grib_ieee_to_long;
+	  bytes=4;
       break;
     case 2:
       encode_float =grib_ieee64_to_long;
+	  bytes=8;
       break;
     default:
       return GRIB_NOT_IMPLEMENTED;
@@ -621,15 +647,31 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len) {
   Assert (sub_j == sub_k); Assert( sub_j == sub_m);
   Assert (pen_j == pen_k); Assert( pen_j == pen_m);
 
-  Assert ( sub_j < pen_j );
-
   n_vals = (pen_j+1)*(pen_j+2);
+  d = grib_power(decimal_scale_factor,10) ;
 
   if(*len != n_vals){
     grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"COMPLEX_PACKING : wrong number of values, expected %d - got %d",n_vals,*len);
     return GRIB_INTERNAL_ERROR;
   }
 
+  if (pen_j == sub_j) {
+	double* values;
+	if (d) {
+		values=grib_context_malloc_clear(a->parent->h->context,sizeof(double)*n_vals);
+		for (i=0;i<n_vals;i++) values[i]=val[i]*d;
+	} else {
+		values=(double*)val;
+	}
+	buflen=n_vals*bytes;
+	buf  = grib_context_malloc_clear(a->parent->h->context,buflen);
+  	grib_ieee_encode_array(a->parent->h->context,values,n_vals,bytes,buf);
+	if (d) grib_context_free(a->parent->h->context,values);
+	grib_buffer_replace(a, buf, buflen,1,1);
+	grib_context_free(a->parent->h->context,buf);
+	return 0;
+  }
+
   if(!laplacianOperatorIsSet) {
     laplacianOperator = calculate_pfactor(a->parent->h->context,val,pen_j,sub_j);
     if((ret = grib_set_double_internal(a->parent->h,self->laplacianOperator,laplacianOperator))
@@ -662,7 +704,6 @@ printf("PACKING LAPLACE set=%ld value=%.20f\n",laplacianOperatorIsSet,laplacianO
     scals[i] = ((double)pow(i*(i+1),laplacianOperator));
 
 
-  d = grib_power(decimal_scale_factor,10) ;
   i=0;
 
   mmax = 0;
@@ -764,19 +805,39 @@ printf("PACKING LAPLACE set=%ld value=%.20f\n",laplacianOperatorIsSet,laplacianO
     grib_encode_double_array_complex((maxv-hcount)*2,&(val[i]),bits_per_value,reference_value,&(scals[lup]),d,s,lres,&lpos);
     i+=(maxv-hcount)*2;
 #else
-    for(lcount=hcount; lcount < maxv ; lcount++)
-    {
-      current_val = (((((val[i++]*d) * scals[lup])-reference_value)*s)+0.5);
-      if(current_val < 0)
-        grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"COMPLEX_PACKING : negative coput before packing (%g)", current_val);
-      grib_encode_unsigned_long(lres, current_val, &lpos, bits_per_value);
+	if (bits_per_value % 8) {
+		for(lcount=hcount; lcount < maxv ; lcount++)
+		{
+		  current_val = (((((val[i++]*d) * scals[lup])-reference_value)*s)+0.5);
+		  if(current_val < 0)
+			grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
+					"COMPLEX_PACKING : negative coput before packing (%g)", current_val);
+		  grib_encode_unsigned_longb(lres, current_val, &lpos, bits_per_value);
 
-      current_val = (((((val[i++]*d) * scals[lup])-reference_value)*s)+0.5);
-      if(current_val < 0)
-        grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"COMPLEX_PACKING : negative coput before packing (%g)", current_val);
-      grib_encode_unsigned_long(lres, current_val, &lpos, bits_per_value);
-      lup++;
-    }
+		  current_val = (((((val[i++]*d) * scals[lup])-reference_value)*s)+0.5);
+		  if(current_val < 0)
+			grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
+					"COMPLEX_PACKING : negative coput before packing (%g)", current_val);
+		  grib_encode_unsigned_longb(lres, current_val, &lpos, bits_per_value);
+		  lup++;
+		}
+	} else {
+		for(lcount=hcount; lcount < maxv ; lcount++)
+		{
+		  current_val = (((((val[i++]*d) * scals[lup])-reference_value)*s)+0.5);
+		  if(current_val < 0)
+			grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
+				"COMPLEX_PACKING : negative coput before packing (%g)", current_val);
+		  grib_encode_unsigned_long(lres, current_val, &lpos, bits_per_value);
+
+		  current_val = (((((val[i++]*d) * scals[lup])-reference_value)*s)+0.5);
+		  if(current_val < 0)
+			grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
+				"COMPLEX_PACKING : negative coput before packing (%g)", current_val);
+		  grib_encode_unsigned_long(lres, current_val, &lpos, bits_per_value);
+		  lup++;
+		}
+	}
 #endif
 
     maxv--;
@@ -786,7 +847,8 @@ printf("PACKING LAPLACE set=%ld value=%.20f\n",laplacianOperatorIsSet,laplacianO
 
   if(((hpos/8) != hsize) &&((lpos/8) != lsize))
   {
-    grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"COMPLEX_PACKING : Mismatch in packing between high resolution and low resolution part");
+    grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
+				"COMPLEX_PACKING : Mismatch in packing between high resolution and low resolution part");
     grib_context_free(a->parent->h->context,buf);
     grib_context_free(a->parent->h->context,scals);
     return GRIB_INTERNAL_ERROR;
@@ -794,8 +856,6 @@ printf("PACKING LAPLACE set=%ld value=%.20f\n",laplacianOperatorIsSet,laplacianO
 
   buflen = ((hpos + lpos)/8);
 
-
-
   if((ret = grib_set_double_internal(a->parent->h,self->reference_value, reference_value)) != GRIB_SUCCESS)
     return ret;
   {
