@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 #include <math.h>
@@ -51,6 +52,7 @@ typedef struct grib_accessor_data_g1complex_packing {
 	const char* offsetsection;
 	int dirty;
 /* Members defined in data_simple_packing */
+	int edition;
 	const char*  units_factor;
 	const char*  units_bias;
 	const char*  changing_precision;
@@ -91,6 +93,7 @@ static grib_accessor_class _grib_accessor_class_data_g1complex_packing = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -127,6 +130,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -164,6 +168,7 @@ static void init(grib_accessor* a,const long v, grib_arguments* args)
   self->packingType  = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->ieee_packing = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->precision = grib_arguments_get_name(a->parent->h,args,self->carg++);
+  self->edition=1;
   a->flags |= GRIB_ACCESSOR_FLAG_DATA;
   }
 
@@ -173,7 +178,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
   grib_accessor_data_g1complex_packing* self =
       (grib_accessor_data_g1complex_packing*)a;
   int ret = GRIB_SUCCESS;
-  long   subsetLen = 0;
+  long seclen=0;
   long   sub_j= 0;
   long   sub_k= 0;
   long   sub_m= 0;
@@ -236,9 +241,15 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 #endif
     ret = grib_get_long_internal(a->parent->h,self->bits_per_value,&bits_per_value);
     if(ret != GRIB_SUCCESS) return ret;
-    buflen = grib_byte_count(a);
 
-    half_byte = (buflen*8)-((*len-subsetLen)*bits_per_value);
+    ret = grib_get_long_internal(a->parent->h,self->seclen,&seclen);
+    if(ret != GRIB_SUCCESS) return ret;
+
+    buflen = 32*(sub_k+1)*(sub_k+2)+(*len-(sub_k+1)*(sub_k+2))*bits_per_value+18*8;
+    half_byte = seclen*8-buflen;
+	if (a->parent->h->context->debug==-1) {
+		printf("GRIB_API DEBUG: half_byte=%ld\n",half_byte);
+	}
 
     ret = grib_set_long_internal(a->parent->h,self->half_byte, half_byte);
     if(ret != GRIB_SUCCESS) return ret;
