@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-* 
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 
@@ -49,6 +50,7 @@ typedef struct grib_accessor_data_g2simple_packing {
 	const char* offsetsection;
 	int dirty;
 /* Members defined in data_simple_packing */
+	int edition;
 	const char*  units_factor;
 	const char*  units_bias;
 	const char*  changing_precision;
@@ -73,6 +75,7 @@ static grib_accessor_class _grib_accessor_class_data_g2simple_packing = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -109,6 +112,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
 	c->get_native_type	=	(*(c->super))->get_native_type;
@@ -138,7 +142,9 @@ static void init_class(grib_accessor_class* c)
 
 static void init(grib_accessor* a,const long v, grib_arguments* args)
 {
+  grib_accessor_data_g2simple_packing *self =(grib_accessor_data_g2simple_packing*)a;
   a->flags |= GRIB_ACCESSOR_FLAG_DATA;
+  self->edition=2;
 }
 
 static long value_count(grib_accessor* a)
