@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 #include "grib_api_internal.h"
@@ -60,6 +61,7 @@ typedef struct grib_accessor_data_jpeg2000_packing {
 	const char* offsetsection;
 	int dirty;
 /* Members defined in data_simple_packing */
+	int edition;
 	const char*  units_factor;
 	const char*  units_bias;
 	const char*  changing_precision;
@@ -93,6 +95,7 @@ static grib_accessor_class _grib_accessor_class_data_jpeg2000_packing = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -129,6 +132,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
 	c->get_native_type	=	(*(c->super))->get_native_type;
@@ -172,6 +176,7 @@ static void init(grib_accessor* a,const long v, grib_arguments* args)
 	self->list_defining_points   = grib_arguments_get_name(a->parent->h,args,self->carg++);
 	self->number_of_data_points  = grib_arguments_get_name(a->parent->h,args,self->carg++);
 	self->scanning_mode      = grib_arguments_get_name(a->parent->h,args,self->carg++);
+	self->edition=2;
 	a->flags |= GRIB_ACCESSOR_FLAG_DATA;
 
 #if HAVE_LIBJASPER
@@ -323,7 +328,7 @@ static int pack_double(grib_accessor* a, const double* cval, size_t *len)
 	long scanning_mode;
 	long list_defining_points;
 	long number_of_data_points;
-    int ret =0;
+   int ret =0;
 	j2k_encode_helper helper;
 	double units_factor=1.0;
 	double units_bias=0.0;
@@ -346,17 +351,16 @@ static int pack_double(grib_accessor* a, const double* cval, size_t *len)
 			  grib_set_double_internal(a->parent->h,self->units_bias,0.0);
 	}
 
-
-    if (units_factor != 1.0) {
+   if (units_factor != 1.0) {
         if (units_bias != 0.0)
             for (i=0;i<n_vals;i++) val[i]=val[i]*units_factor+units_bias;
         else
             for (i=0;i<n_vals;i++) val[i]*=units_factor;
-    } else if (units_bias != 0.0)
+   } else if (units_bias != 0.0)
         for (i=0;i<n_vals;i++) val[i]+=units_bias;
 
-    ret = super->pack_double(a,val,len);
-    switch (ret) {
+   ret = super->pack_double(a,val,len);
+   switch (ret) {
       case GRIB_CONSTANT_FIELD:
         grib_buffer_replace(a, NULL, 0,1,1);
         return GRIB_SUCCESS;
@@ -366,7 +370,7 @@ static int pack_double(grib_accessor* a, const double* cval, size_t *len)
       default:
         grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"unable to compute packing parameters\n");
         return ret;
-    }
+   }
 
   if((ret = grib_get_double_internal(a->parent->h,self->reference_value, &reference_value))
       != GRIB_SUCCESS)
@@ -384,9 +388,8 @@ static int pack_double(grib_accessor* a, const double* cval, size_t *len)
      != GRIB_SUCCESS)
     return ret;
 
-
-    decimal = grib_power(decimal_scale_factor,10) ;
-    divisor = grib_power(-binary_scale_factor,2);
+   decimal = grib_power(decimal_scale_factor,10) ;
+   divisor = grib_power(-binary_scale_factor,2);
 
 	simple_packing_size = (((bits_per_value*n_vals)+7)/8)*sizeof(unsigned char);
 	buf  = grib_context_malloc_clear(a->parent->h->context,simple_packing_size+EXTRA_BUFFER_SIZE);
@@ -469,6 +472,7 @@ static int pack_double(grib_accessor* a, const double* cval, size_t *len)
 	helper.reference_value = reference_value;
 	helper.divisor         = divisor;
 	helper.decimal         = decimal;
+	helper.jpeg_length     = 0;
 
     switch (self->jpeg_lib) {
 		case OPENJPEG_LIB:
@@ -492,10 +496,10 @@ static int pack_double(grib_accessor* a, const double* cval, size_t *len)
 		if(f) {
 			if(fwrite(helper.jpeg_buffer,helper.jpeg_length,1,f) != 1)
 				perror(self->dump_jpg);
-			fclose(f);
+			if (fclose(f) != 0)
+				perror(self->dump_jpg);
 		}
 		else perror(self->dump_jpg);
-
 	}
 
 	grib_buffer_replace(a, helper.jpeg_buffer, helper.jpeg_length, 1, 1);
