@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /*********************************************
@@ -20,7 +21,7 @@
    SUPER      = grib_accessor_class_abstract_long_vector
    IMPLEMENTS = pack_string;unpack_string;value_count
    IMPLEMENTS = pack_long;unpack_long;dump
-   IMPLEMENTS = get_native_type
+   IMPLEMENTS = get_native_type;string_length
    IMPLEMENTS = init; destroy
    MEMBERS    = const char* p1
    MEMBERS    = const char* p2
@@ -48,6 +49,7 @@ static int pack_long(grib_accessor*, const long* val,size_t *len);
 static int pack_string(grib_accessor*, const char*, size_t *len);
 static int unpack_long(grib_accessor*, long* val,size_t *len);
 static int unpack_string (grib_accessor*, char*, size_t *len);
+static size_t string_length(grib_accessor*);
 static long value_count(grib_accessor*);
 static void destroy(grib_context*,grib_accessor*);
 static void dump(grib_accessor*, grib_dumper*);
@@ -84,6 +86,7 @@ static grib_accessor_class _grib_accessor_class_g1step_range = {
     &destroy,                    /* free mem                       */
     &dump,                       /* describes himself         */
     0,                /* get length of section     */
+    &string_length,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -171,6 +174,7 @@ static void dump(grib_accessor* a, grib_dumper* dumper)
 
 }
 
+static int error_on_units=1;
 
 static int u2s1[] =  {
   60,      /* (0) minutes */
@@ -260,14 +264,6 @@ int grib_g1_step_get_steps(grib_accessor* a,long* start,long* end) {
 	else if (!strcmp(stepType,"instant")) *start = *end = p1;
     else if (!strcmp(stepType,"accum") && timeRangeIndicator==0 ) {*start =0; *end = p1;}
 
-	if(*start > *end) {
-		/* Special case for Aladin, where P1 is missing in accumulation */
-		if(!(*start == 255 && *end == 0 && timeRangeIndicator==2)) 
-		{
-			*end = *start;
-		}
-	}
-
 	if (u2s1[unit] == u2s[step_unit] || (*start==0 && *end==0) ) return 0;
 
 	newstart = (*start) * u2s1[unit];
@@ -286,20 +282,7 @@ int grib_g1_step_get_steps(grib_accessor* a,long* start,long* end) {
 	}
 
 	if (newstart % u2sf_step_unit != 0 || newend % u2sf_step_unit != 0) {
-		size_t step_unit_string_len=10;
-		char step_unit_string[10];
-		if (self->step_unit != NULL)
-			grib_get_string(a->parent->h,self->step_unit,step_unit_string,&step_unit_string_len);
-		else
-			sprintf(step_unit_string,"h");
-
-		err = grib_set_long_internal(a->parent->h,self->step_unit,unit);
-		
-		grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
-				"unable to represent the step in %s",
-				step_unit_string);
 		return GRIB_DECODING_ERROR;
-	
 	} else {
 		*start = newstart/u2sf_step_unit;
 		*end   = newend/u2sf_step_unit;
@@ -314,12 +297,29 @@ static int unpack_string(grib_accessor* a, char* val, size_t *len) {
 	size_t size=0;
 	long start=0,end=0;
 	long timeRangeIndicator=0;
+	long unit;
 	int err=0;
     char stepType[20]={0,};
     size_t stepTypeLen=20;
 
-	if ((err=grib_g1_step_get_steps(a,&start,&end))!=GRIB_SUCCESS)
+	if ((err=grib_g1_step_get_steps(a,&start,&end))!=GRIB_SUCCESS) {
+		size_t step_unit_string_len=10;
+		char step_unit_string[10];
+
+		if (self->step_unit != NULL)
+			grib_get_string(a->parent->h,self->step_unit,step_unit_string,&step_unit_string_len);
+		else
+			sprintf(step_unit_string,"h");
+
+		if (error_on_units) {
+			grib_get_long_internal(a->parent->h,self->unit,&unit);
+			grib_set_long_internal(a->parent->h,self->step_unit,unit);
+			grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
+				"unable to represent the step in %s",
+				step_unit_string);
+		}
 		return err;
+	}
 
 	err = grib_get_long_internal(a->parent->h,self->timeRangeIndicator,&timeRangeIndicator);
 	if(err)  return err;
@@ -335,10 +335,36 @@ static int unpack_string(grib_accessor* a, char* val, size_t *len) {
 		start += 24;
 	}
 
-    if (!strcmp(stepType,"instant") )
-		sprintf(buf,"%ld",end);
-	else
-		sprintf(buf,"%ld-%ld",start,end);
+	if(strcmp(stepType,"instant") == 0) {
+		sprintf(buf,"%ld",start);
+	}
+	else if( (strcmp(stepType,"avgfc") == 0) ||
+			(strcmp(stepType,"avgua") == 0) ||
+			(strcmp(stepType,"avgia") == 0) ) 
+	{
+		sprintf(buf,"%ld",start);
+	}
+	else if(
+		(strcmp(stepType,"accum") == 0) || 
+		(strcmp(stepType,"avg") == 0) || 
+		(strcmp(stepType,"min") == 0) || 
+		(strcmp(stepType,"max") == 0) || 
+		(strcmp(stepType,"rms") == 0) || 
+		(strcmp(stepType,"diff") == 0)
+		)
+	{
+		if(start == end) {
+			sprintf(buf,"%ld",end);
+		}
+		else
+		{
+			sprintf(buf,"%ld-%ld",start, end);
+		}
+	}
+	else {
+		grib_context_log(a->parent->h->context,GRIB_LOG_ERROR, "Unknown stepType=[%s] timeRangeIndicator=[%ld]",stepType,timeRangeIndicator);
+		return GRIB_NOT_IMPLEMENTED;
+	}
 
 	size=strlen(buf)+1;
 
@@ -552,12 +578,12 @@ static int pack_string(grib_accessor* a, const char* val, size_t *len){
 
 static long value_count(grib_accessor* a)
 {
-	char result[1024]  ;
-	size_t s = sizeof(result);
+	return 1;
+}
 
-	unpack_string(a,result,&s);
-
-	return s;
+static size_t string_length(grib_accessor* a)
+{
+	return 255;
 }
 
 static int pack_long(grib_accessor* a, const long* val, size_t *len)
@@ -568,6 +594,18 @@ static int pack_long(grib_accessor* a, const long* val, size_t *len)
 	char* p=sval;
 	size_t svallen=100;
 	grib_accessor_g1step_range* self = (grib_accessor_g1step_range*)a;
+	char stepType[20]={0,};
+	size_t stepTypeLen=20;
+	long step_unit=0;
+	int err=0;
+
+	if (self->stepType) {
+		err = grib_get_string_internal(a->parent->h,self->stepType,stepType,&stepTypeLen);
+		if(err)  return err;
+	} else sprintf(stepType,"unknown");
+
+	if(self->step_unit!=NULL && (err = grib_get_long_internal(a->parent->h,self->step_unit,&step_unit)))
+	        return err;
 
 	switch (self->pack_index) {
 		case  -1 :
@@ -576,22 +614,36 @@ static int pack_long(grib_accessor* a, const long* val, size_t *len)
 			return pack_string( a,buff,&bufflen);
 		case  0 :
 			self->pack_index=-1;
+			error_on_units=0;
 			unpack_string(a,sval,&svallen);
+			error_on_units=1;
 			while (*p != '-' && *p != '\0' ) p++;
-			if (*p=='-') 
+			if (*p=='-') {
 				sprintf(buff,"%ld-%s",*val,++p);
-			else
-				sprintf(buff,"%ld",*val);
+			} else {
+				if (strcmp(stepType,"instant") && strcmp(stepType,"avgd")) {
+					sprintf(buff,"%ld-%s",*val,sval);
+				} else {
+					sprintf(buff,"%ld",*val);
+				}
+			}
 			return pack_string( a,buff,&bufflen);
 		case  1 :
 			self->pack_index=-1;
+			error_on_units=0;
 			unpack_string(a,sval,&svallen);
+			error_on_units=1;
 			while (*p != '-' && *p != '\0' ) p++;
 			if (*p=='-') {
 				*p='\0';
 				sprintf(buff,"%s-%ld",sval,*val);
-			} else
-				sprintf(buff,"%ld",*val);
+			} else {
+				if (strcmp(stepType,"instant") && strcmp(stepType,"avgd")) {
+					sprintf(buff,"%s-%ld",sval,*val);
+				} else {
+					sprintf(buff,"%ld",*val);
+				}
+			}
 			return pack_string( a,buff,&bufflen);
 		default :
 			Assert(self->pack_index<2);
