@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 
@@ -65,6 +66,7 @@ static grib_accessor_class _grib_accessor_class_g2level = {
     0,                    /* free mem                       */
     &dump,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -100,6 +102,7 @@ grib_accessor_class* grib_accessor_class_g2level = &_grib_accessor_class_g2level
 static void init_class(grib_accessor_class* c)
 {
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -147,69 +150,72 @@ static void dump(grib_accessor* a, grib_dumper* dumper)
 
 static int unpack_long(grib_accessor* a, long* val, size_t *len)
 {
-  int ret=0;
-  grib_accessor_g2level* self = (grib_accessor_g2level*)a;
+	int ret=0;
+	grib_accessor_g2level* self = (grib_accessor_g2level*)a;
 
-  long type_first   = 0;
-  long scale_first  = 0;
-  long value_first  = 0;
-  char pressure_units[10]={0,};
-  size_t pressure_units_len=10;
+	long type_first   = 0;
+	long scale_first  = 0;
+	long value_first  = 0;
+	char pressure_units[10]={0,};
+	size_t pressure_units_len=10;
 
-  double v;
+	double v;
 
-  if((ret = grib_get_long_internal(a->parent->h, self->type_first,&type_first))
-      !=GRIB_SUCCESS) return ret;
-  if((ret = grib_get_long_internal(a->parent->h, self->scale_first,&scale_first))
-      !=GRIB_SUCCESS) return ret;
-  if((ret = grib_get_long_internal(a->parent->h, self->value_first,&value_first))
-      !=GRIB_SUCCESS) return ret;
-  if((ret = grib_get_string_internal(a->parent->h, self->pressure_units,pressure_units,&pressure_units_len))
-      !=GRIB_SUCCESS) return ret;
+	if((ret = grib_get_long_internal(a->parent->h, self->type_first,&type_first))
+			!=GRIB_SUCCESS) return ret;
+	if((ret = grib_get_long_internal(a->parent->h, self->scale_first,&scale_first))
+			!=GRIB_SUCCESS) return ret;
+	if((ret = grib_get_long_internal(a->parent->h, self->value_first,&value_first))
+			!=GRIB_SUCCESS) return ret;
+	if((ret = grib_get_string_internal(a->parent->h, self->pressure_units,pressure_units,&pressure_units_len))
+			!=GRIB_SUCCESS) return ret;
 
-  if (value_first == GRIB_MISSING_LONG) {
-    *val=0;
-    return GRIB_SUCCESS;
-  }
-  /* value = value_first * 10 ^ -scale_first */
+	if (value_first == GRIB_MISSING_LONG) {
+		*val=0;
+		return GRIB_SUCCESS;
+	}
+	/* value = value_first * 10 ^ -scale_first */
 
-/*
-fprintf(stderr,"LEVEL %s %s %ld %ld\n",self->scale_first,
-self->value_first,scale_first,value_first);
-*/
+	if(*len < 1)
+		return GRIB_WRONG_ARRAY_SIZE;
 
-  if(*len < 1)
-    return GRIB_WRONG_ARRAY_SIZE;
+	v = value_first;
 
-  v = value_first;
+	if(scale_first != GRIB_MISSING_LONG)
+	{
+		while(scale_first<0 && v!=0)  { v *= 10.0; scale_first ++; }
+		while(scale_first>0 && v!=0)  { v /= 10.0; scale_first --; }
+	}
 
-  if(scale_first != GRIB_MISSING_LONG)
-  {
-    while(scale_first<0)   { v *= 10.0; scale_first ++; }
-    while(scale_first>0)   { v /= 10.0; scale_first --; }
-  }
+	switch(type_first)
+	{
+	case 100: /* Isobaric surface (Pa) */
+		if (!strcmp(pressure_units,"hPa"))
+		{
+			long x= v/100.0;  /* 1 hPa = 100 Pa */
+			if (scale_first == 0 && x==0)
+			{
+				/* Switch to Pa instead of hPa as the value is less than a hectoPascal */
+				char pa[]="Pa";
+				size_t lpa=strlen(pa);
+				if((ret = grib_set_string_internal(a->parent->h, self->pressure_units,pa,&lpa)) !=GRIB_SUCCESS)
+					return ret;
+			}
+			else
+			{
+				v=x;
+			}
+		}
+		break;
 
-  switch(type_first)
-  {
-    case 100: /* Pa */
-      if (!strcmp(pressure_units,"hPa")) 
-        v /= 100.0;
-      break;
+	case 109: /* Potential vorticity surface */
+		v  *= 1e6;
+		break;
+	}
 
-    case 109: /* pv */
-      v  *= 1e6;
-      break;
-  }
+	val[0] =  (long)(v+0.5);
 
-
-  val[0] =  (long)(v+0.5);
-/*
-  fprintf(stderr,"LEVEL %ld %g\n",val[0],v);
-  Assert(val[0] == v);
-
-fprintf(stderr,"LEVEL %ld\n",val[0]); */
-
-  return GRIB_SUCCESS;
+	return GRIB_SUCCESS;
 }
 
 /* TODO: Support double */
