@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*********************************************
  *   Enrico Fucile
@@ -20,7 +21,7 @@
    SUPER      = grib_accessor_class_gen
    IMPLEMENTS = pack_string;unpack_string;value_count
    IMPLEMENTS = pack_long;unpack_long;dump
-   IMPLEMENTS = get_native_type
+   IMPLEMENTS = get_native_type;string_length
    IMPLEMENTS = init
    MEMBERS    = const char* startStep
    MEMBERS    = const char* endStep
@@ -43,6 +44,7 @@ static int pack_long(grib_accessor*, const long* val,size_t *len);
 static int pack_string(grib_accessor*, const char*, size_t *len);
 static int unpack_long(grib_accessor*, long* val,size_t *len);
 static int unpack_string (grib_accessor*, char*, size_t *len);
+static size_t string_length(grib_accessor*);
 static long value_count(grib_accessor*);
 static void dump(grib_accessor*, grib_dumper*);
 static void init(grib_accessor*,const long, grib_arguments* );
@@ -69,6 +71,7 @@ static grib_accessor_class _grib_accessor_class_g2step_range = {
     0,                    /* free mem                       */
     &dump,                       /* describes himself         */
     0,                /* get length of section     */
+    &string_length,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -162,7 +165,15 @@ static int unpack_string(grib_accessor* a, char* val, size_t *len) {
   } else {
     ret = grib_get_long_internal(h,self->endStep,&end);
     if (ret) return ret;
-    sprintf(buf,"%ld-%ld",start,end);
+
+	if(start == end)
+	{
+		sprintf(buf,"%ld",end);
+	}
+	else
+	{
+		sprintf(buf,"%ld-%ld",start,end);
+	}
   }
 
   size=strlen(buf)+1;
@@ -200,12 +211,12 @@ static int pack_string(grib_accessor* a, const char* val, size_t *len){
 
 static long value_count(grib_accessor* a)
 {
-  char result[1024]  ;
-  size_t s = sizeof(result);
+	return 1;
+}
 
-  unpack_string(a,result,&s);
-
-  return s;
+static size_t string_length(grib_accessor* a)
+{
+  return 255;
 }
 
 static int pack_long(grib_accessor* a, const long* val, size_t *len)
