@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /***************************************************************************
  *   Jean Baptiste Filippi - 01.11.2005                                                           *
@@ -23,7 +24,7 @@
    IMPLEMENTS = unpack_string;pack_string
    IMPLEMENTS = unpack_bytes;pack_bytes
    IMPLEMENTS = unpack_double_subarray 
-   IMPLEMENTS = init;dump;destroy
+   IMPLEMENTS = init;dump;destroy;string_length
    IMPLEMENTS = get_native_type;sub_section
    IMPLEMENTS = next_offset;value_count;byte_offset;byte_count
    IMPLEMENTS = notify_change;pack_expression
@@ -55,6 +56,7 @@ static int unpack_bytes (grib_accessor*,unsigned char*, size_t *len);
 static int unpack_double(grib_accessor*, double* val,size_t *len);
 static int unpack_long(grib_accessor*, long* val,size_t *len);
 static int unpack_string (grib_accessor*, char*, size_t *len);
+static size_t string_length(grib_accessor*);
 static long byte_count(grib_accessor*);
 static long byte_offset(grib_accessor*);
 static long next_offset(grib_accessor*);
@@ -89,6 +91,7 @@ static grib_accessor_class _grib_accessor_class_gen = {
     &destroy,                    /* free mem                       */
     &dump,                       /* describes himself         */
     &next_offset,                /* get length of section     */
+    &string_length,              /* get length of string      */
     &value_count,                /* get number of values      */
     &byte_count,                 /* get number of bytes      */
     &byte_offset,                /* get offset to bytes           */
@@ -195,6 +198,11 @@ static long value_count(grib_accessor* a)
   return 1;
 }
 
+static size_t string_length(grib_accessor* a)
+{
+	return 1024;
+}
+
 static long byte_count(grib_accessor* a)
 {
   return a->length;
@@ -365,7 +373,7 @@ static int pack_expression(grib_accessor* a, grib_expression *e){
         grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"unable to set %s as string",a->name);
         return ret;
       }
-      len = strlen(cval)+1;
+      len = strlen(cval);
       return grib_pack_string(a,cval,&len);
       break;
   }
