@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-* 
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 /* 
@@ -15,7 +16,7 @@
    CLASS      = accessor
    SUPER      = grib_accessor_class_ascii
    IMPLEMENTS = pack_string;unpack_string;value_count
-   IMPLEMENTS = init
+   IMPLEMENTS = init;string_length
    MEMBERS =  int gts_offset
    MEMBERS =  int gts_length
    END_CLASS_DEF
@@ -34,6 +35,7 @@ or edit "accessor.class" and rerun ./make_class.pl
 
 static int pack_string(grib_accessor*, const char*, size_t *len);
 static int unpack_string (grib_accessor*, char*, size_t *len);
+static size_t string_length(grib_accessor*);
 static long value_count(grib_accessor*);
 static void init(grib_accessor*,const long, grib_arguments* );
 static void init_class(grib_accessor_class*);
@@ -60,6 +62,7 @@ static grib_accessor_class _grib_accessor_class_gts_header = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    &string_length,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -84,6 +87,7 @@ static grib_accessor_class _grib_accessor_class_gts_header = {
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
     0,     /* unpack only ith value          */
+    0,     /* unpack a subarray         */
     0,             		/* clear          */
 };
 
@@ -116,6 +120,7 @@ static void init_class(grib_accessor_class* c)
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
+	c->unpack_double_subarray	=	(*(c->super))->unpack_double_subarray;
 	c->clear	=	(*(c->super))->clear;
 }
 
@@ -161,13 +166,13 @@ static int    unpack_string(grib_accessor* a, char* val, size_t *len)
 	return GRIB_SUCCESS;
 }
 
-
 static long value_count(grib_accessor* a)
 {
-	char result[1024]  ;
-	size_t s = sizeof(result);
+	return 1;
+}
 
-	unpack_string(a,result,&s);
-
-    return s;
+static size_t string_length(grib_accessor* a)
+{
+	grib_handle* h=a->parent->h;
+	return h->gts_header_len;
 }
