@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /**************************************
  *  Enrico Fucile
@@ -64,6 +65,7 @@ static grib_accessor_class _grib_accessor_class_ifs_param = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -100,6 +102,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -146,7 +149,9 @@ static int    unpack_long   (grib_accessor* a, long* val, size_t *len)
   if((ret = grib_get_long_internal(a->parent->h, self->paramId,&paramId)) != GRIB_SUCCESS)
     return ret;
 
-  if (paramId > 200000 && paramId < 200999 ) 
+  if (paramId > 129000 && paramId < 129999 ) 
+  	*val = paramId - 129000;
+  else if (paramId > 200000 && paramId < 200999 ) 
   	*val = paramId - 200000;
   else if (paramId > 211000 && paramId < 211999 )
 	  *val = paramId - 1000;
@@ -184,6 +189,20 @@ static int pack_long(grib_accessor* a, const long* val, size_t *len)
 			break;
   	}
   }
+  if (type == 50 || type == 52 ) {
+	if (paramId>1000) {
+		table=paramId/1000;
+		param=paramId-table*1000;
+		paramId=param;
+	}
+	switch (table) {
+		case 128:
+			paramId+=129000;
+		    break;
+	  	default:
+			break;
+	}
+  }
 
   return grib_set_long_internal(a->parent->h,self->paramId,paramId);
 }
