@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 
@@ -16,6 +17,7 @@
    CLASS      = accessor
    SUPER      = grib_accessor_class_ascii
    IMPLEMENTS = unpack_long;pack_string;
+   IMPLEMENTS = pack_long
    IMPLEMENTS = init
    END_CLASS_DEF
 
@@ -31,6 +33,7 @@ or edit "accessor.class" and rerun ./make_class.pl
 
 */
 
+static int pack_long(grib_accessor*, const long* val,size_t *len);
 static int pack_string(grib_accessor*, const char*, size_t *len);
 static int unpack_long(grib_accessor*, long* val,size_t *len);
 static void init(grib_accessor*,const long, grib_arguments* );
@@ -56,6 +59,7 @@ static grib_accessor_class _grib_accessor_class_ksec1expver = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -63,7 +67,7 @@ static grib_accessor_class _grib_accessor_class_ksec1expver = {
     0,                /* get sub_section                */
     0,               /* grib_pack procedures long      */
     0,               /* grib_pack procedures long      */
-    0,                  /* grib_pack procedures long      */
+    &pack_long,                  /* grib_pack procedures long      */
     &unpack_long,                /* grib_unpack procedures long    */
     0,                /* grib_pack procedures double    */
     0,              /* grib_unpack procedures double  */
@@ -92,6 +96,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -99,7 +104,6 @@ static void init_class(grib_accessor_class* c)
 	c->sub_section	=	(*(c->super))->sub_section;
 	c->pack_missing	=	(*(c->super))->pack_missing;
 	c->is_missing	=	(*(c->super))->is_missing;
-	c->pack_long	=	(*(c->super))->pack_long;
 	c->pack_double	=	(*(c->super))->pack_double;
 	c->unpack_double	=	(*(c->super))->unpack_double;
 	c->unpack_string	=	(*(c->super))->unpack_string;
@@ -172,6 +176,10 @@ static int unpack_long(grib_accessor* a, long* val, size_t *len)
 static int pack_string(grib_accessor* a, const char* val, size_t *len)
 {
   int i = 0;
+  if (len[0] != 4 ) {
+  	grib_context_log(a->parent->h->context, GRIB_LOG_ERROR, "Wrong length for %s. It has to be 4",a->name);
+	return GRIB_INVALID_KEY_VALUE;
+  }
   if(len[0] > (a->length)+1)
   {
     grib_context_log(a->parent->h->context, GRIB_LOG_ERROR, "pack_string: Wrong size (%d) for %s it contains %d values ", len[0], a->name , a->length+1 );
@@ -185,4 +193,10 @@ static int pack_string(grib_accessor* a, const char* val, size_t *len)
   return GRIB_SUCCESS;
 }
 
+static int pack_long(grib_accessor* a, const long* val, size_t *len) {
+	char sval[5]={0,};
+	size_t slen=4;
+	sprintf(sval,"%04d",(int)(*val));
+	return pack_string(a,sval,&slen);
+}
 
