@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-* 
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 /* 
@@ -61,6 +62,7 @@ static grib_accessor_class _grib_accessor_class_mars_param = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -85,6 +87,7 @@ static grib_accessor_class _grib_accessor_class_mars_param = {
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
     0,     /* unpack only ith value          */
+    0,     /* unpack a subarray         */
     0,             		/* clear          */
 };
 
@@ -96,6 +99,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -118,6 +122,7 @@ static void init_class(grib_accessor_class* c)
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
+	c->unpack_double_subarray	=	(*(c->super))->unpack_double_subarray;
 	c->clear	=	(*(c->super))->clear;
 }
 
@@ -133,6 +138,9 @@ static void init(grib_accessor* a,const long l, grib_arguments* c)
 }
 
 static int pack_string(grib_accessor* a, const char* val, size_t *len){
+#if 1
+	return GRIB_NOT_IMPLEMENTED;
+#else
 	grib_accessor_mars_param* self = (grib_accessor_mars_param*)a; 
 	long paramId=0;
 	long param=0;
@@ -147,17 +155,23 @@ static int pack_string(grib_accessor* a, const char* val, size_t *len){
 	paramId=table*1000+param;
 
 	return grib_set_long_internal(a->parent->h,self->paramId,paramId);
+#endif		
 }
 
 
 static int    unpack_string(grib_accessor* a, char* val, size_t *len)
 {   
 	grib_accessor_mars_param* self = (grib_accessor_mars_param*)a; 
-	long paramId=0;
 	long param=0;
 	long table=0;
 	int ret=0;
 
+#if 1
+	if(self->table!=NULL && (ret = grib_get_long_internal(a->parent->h,self->table,&table)) != GRIB_SUCCESS) return ret;
+	if(self->param!=NULL && (ret = grib_get_long_internal(a->parent->h,self->param,&param)) != GRIB_SUCCESS) return ret;
+#else
+	{
+	long paramId=0;
 	grib_get_long(a->parent->h,self->paramId,&paramId);
 
 	if (paramId==0 || (paramId < 4000 && paramId > 1000 )) {
@@ -172,6 +186,8 @@ static int    unpack_string(grib_accessor* a, char* val, size_t *len)
     	table=paramId/1000;
 		param=paramId-table*1000;
 	}
+	}
+#endif
 
 	/*if (table==200) table=128;*/
 	sprintf(val,"%ld.%ld",param,table);
