@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /**************************************
  * Enrico Fucile
@@ -77,6 +78,7 @@ static grib_accessor_class _grib_accessor_class_number_of_points_gaussian = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -113,6 +115,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -216,7 +219,7 @@ static int  unpack_long(grib_accessor* a, long* val, size_t *len){
     grib_get_long_array_internal(a->parent->h,self->pl,pl, &plsize);
 
     if (lon_last<0) lon_last+=360;
-    if (lon_first<0) lon_last+=360;
+    if (lon_first<0) lon_first+=360;
 
     d=fabs(lats[0]-lats[1]);
     if ( (fabs(lat_first-lats[0]) >= d ) ||
