@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /**********************************
  *  Enrico Fucile
@@ -69,6 +70,7 @@ static grib_accessor_class _grib_accessor_class_number_of_values = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -105,6 +107,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -151,15 +154,17 @@ static void init(grib_accessor* a,const long l, grib_arguments* c)
 
 static int  unpack_long(grib_accessor* a, long* val, size_t *len)
 {
-  int ret=GRIB_SUCCESS,i;
-  long bpv=0,npoints=0,bitmap_present=0;
-  size_t size=0;
-  long numberOfCodedValues=0;
-
   grib_accessor_number_of_values* self = (grib_accessor_number_of_values*)a;
+  int ret=GRIB_SUCCESS,i;
+  long npoints=0,bitmap_present=0;
+  size_t size=0;
 
+#if GRIB_WARNING
+  long numberOfCodedValues=0;
+  long bpv=0;
   if((ret = grib_get_long_internal(a->parent->h, self->bitsPerValue,&bpv)) != GRIB_SUCCESS)
     return ret;
+#endif
 
   if((ret = grib_get_long_internal(a->parent->h, self->numberOfPoints,&npoints)) != GRIB_SUCCESS)
       return ret;
@@ -182,18 +187,18 @@ static int  unpack_long(grib_accessor* a, long* val, size_t *len)
       *val=npoints;
   }
 
+#if GRIB_WARNING
   if ( bpv != 0 ) {
     if((ret = grib_get_long_internal(a->parent->h, self->numberOfCodedValues,&numberOfCodedValues)) != GRIB_SUCCESS)
       return ret;
-#if 0
      if (*val != numberOfCodedValues) {
        grib_context_log(a->parent->h->context,GRIB_LOG_WARNING,
        "number of values from data section (%ld) doesn't match geography (%ld)",numberOfCodedValues,*val);
        /* *val=numberOfCodedValues; */
   	}
-#endif
 			   
   }
+#endif
 
 
   return ret;
