@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /**************************************
  *  Enrico Fucile
@@ -19,7 +20,7 @@
    CLASS      = accessor
    SUPER      = grib_accessor_class_bytes
    IMPLEMENTS = init;update_size;resize; value_count
-   IMPLEMENTS = compare
+   IMPLEMENTS = compare;string_length;byte_count
    END_CLASS_DEF
 
  */
@@ -34,6 +35,8 @@ or edit "accessor.class" and rerun ./make_class.pl
 
 */
 
+static size_t string_length(grib_accessor*);
+static long byte_count(grib_accessor*);
 static long value_count(grib_accessor*);
 static void init(grib_accessor*,const long, grib_arguments* );
 static void init_class(grib_accessor_class*);
@@ -61,8 +64,9 @@ static grib_accessor_class _grib_accessor_class_padding = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    &string_length,              /* get length of string      */
     &value_count,                /* get number of values      */
-    0,                 /* get number of bytes      */
+    &byte_count,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
     0,            /* get native type               */
     0,                /* get sub_section                */
@@ -97,7 +101,6 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
-	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
 	c->get_native_type	=	(*(c->super))->get_native_type;
 	c->sub_section	=	(*(c->super))->sub_section;
@@ -153,4 +156,6 @@ static void resize(grib_accessor* a,size_t new_size)
 
 }
 
-static long value_count(grib_accessor* a){ return a->length;}
+static long value_count(grib_accessor* a){ return 1;}
+static long byte_count(grib_accessor* a){ return a->length;}
+static size_t string_length(grib_accessor* a){ return (size_t)a->length;}
