@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /**************************************
  *  Enrico Fucile
@@ -68,6 +69,7 @@ static grib_accessor_class _grib_accessor_class_select_step_template = {
     0,                    /* free mem                       */
     0,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     &value_count,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -104,6 +106,7 @@ static void init_class(grib_accessor_class* c)
 {
 	c->dump	=	(*(c->super))->dump;
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
 	c->get_native_type	=	(*(c->super))->get_native_type;
@@ -156,43 +159,87 @@ static int pack_long(grib_accessor* a, const long* val, size_t *len)
 
   grib_get_long(a->parent->h, self->productDefinitionTemplateNumber,&productDefinitionTemplateNumber);
 
-  if (self->instant) {
-	  switch (productDefinitionTemplateNumber) {
-		  case 8:
-			  productDefinitionTemplateNumberNew=0;
-			  break;
-		  case 11:
-			  productDefinitionTemplateNumberNew=1;
-			  break;
-		  case 0:
-		  case 1:
-			  productDefinitionTemplateNumberNew=productDefinitionTemplateNumber;
-			  break;
-		  default:
-			  productDefinitionTemplateNumberNew=0;
-			  break;
-	  }
+   if (self->instant) {
+      switch (productDefinitionTemplateNumber) {
+          case 8:
+              productDefinitionTemplateNumberNew=0;
+              break;
+          case 9:
+              productDefinitionTemplateNumberNew=5;
+              break;
+          case 10:
+              productDefinitionTemplateNumberNew=6;
+              break;
+          case 11:
+              productDefinitionTemplateNumberNew=1;
+              break;
+          case 12:
+              productDefinitionTemplateNumberNew=2;
+              break;
+          case 13:
+              productDefinitionTemplateNumberNew=3;
+              break;
+          case 14:
+              productDefinitionTemplateNumberNew=4;
+              break;
+          case 0:
+          case 1:
+          case 2:
+          case 3:
+          case 4:
+          case 5:
+          case 6:
+          case 7:
+          case 15:
+               productDefinitionTemplateNumberNew=productDefinitionTemplateNumber;
+              break;
+          default:
+              productDefinitionTemplateNumberNew=productDefinitionTemplateNumber;
+              break;
+      }
   } else {
-	  switch (productDefinitionTemplateNumber) {
-		  case 0:
-			  productDefinitionTemplateNumberNew=8;
-			  break;
-		  case 1:
-			  productDefinitionTemplateNumberNew=11;
-			  break;
-		  case 8:
-		  case 11:
-			  productDefinitionTemplateNumberNew=productDefinitionTemplateNumber;
-			  break;
-		  default:
-			  productDefinitionTemplateNumberNew=8;
-			  break;
-	  }
+      switch (productDefinitionTemplateNumber) {
+          case 0:
+              productDefinitionTemplateNumberNew=8;
+              break;
+          case 1:
+              productDefinitionTemplateNumberNew=11;
+              break;
+          case 2:
+              productDefinitionTemplateNumberNew=12;
+              break;
+          case 3:
+              productDefinitionTemplateNumberNew=13;
+              break;
+          case 4:
+              productDefinitionTemplateNumberNew=14;
+              break;
+          case 5:
+              productDefinitionTemplateNumberNew=9;
+              break;
+          case 6:
+              productDefinitionTemplateNumberNew=10;
+              break;
+          case 7:
+          case 8:
+          case 9:
+          case 10:
+          case 11:
+          case 12:
+          case 13:
+          case 14:
+              productDefinitionTemplateNumberNew=productDefinitionTemplateNumber;
+              break;
+          default:
+              productDefinitionTemplateNumberNew=productDefinitionTemplateNumber;
+              break;
+      }
 
   }
 
   if (productDefinitionTemplateNumber != productDefinitionTemplateNumberNew)
 	  grib_set_long(a->parent->h, self->productDefinitionTemplateNumber,productDefinitionTemplateNumberNew);
+
   return 0;
 }
 
