@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 /*
@@ -67,6 +68,7 @@ static grib_accessor_class _grib_accessor_class_step_in_units = {
     0,                    /* free mem                       */
     &dump,                       /* describes himself         */
     0,                /* get length of section     */
+    0,              /* get length of string      */
     0,                /* get number of values      */
     0,                 /* get number of bytes      */
     0,                /* get offset to bytes           */
@@ -91,6 +93,7 @@ static grib_accessor_class _grib_accessor_class_step_in_units = {
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
     0,     /* unpack only ith value          */
+    0,     /* unpack a subarray         */
     0,             		/* clear          */
 };
 
@@ -101,6 +104,7 @@ grib_accessor_class* grib_accessor_class_step_in_units = &_grib_accessor_class_s
 static void init_class(grib_accessor_class* c)
 {
 	c->next_offset	=	(*(c->super))->next_offset;
+	c->string_length	=	(*(c->super))->string_length;
 	c->value_count	=	(*(c->super))->value_count;
 	c->byte_count	=	(*(c->super))->byte_count;
 	c->byte_offset	=	(*(c->super))->byte_offset;
@@ -123,6 +127,7 @@ static void init_class(grib_accessor_class* c)
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
+	c->unpack_double_subarray	=	(*(c->super))->unpack_double_subarray;
 	c->clear	=	(*(c->super))->clear;
 }
 
@@ -130,103 +135,140 @@ static void init_class(grib_accessor_class* c)
 
 static void init(grib_accessor* a,const long l, grib_arguments* c)
 {
-  grib_accessor_step_in_units* self = (grib_accessor_step_in_units*)a;
-  int n = 0;
+	grib_accessor_step_in_units* self = (grib_accessor_step_in_units*)a;
+	int n = 0;
 
-  self->codedStep = grib_arguments_get_name(a->parent->h,c,n++);
-  self->codedUnits  = grib_arguments_get_name(a->parent->h,c,n++);
-  self->stepUnits  = grib_arguments_get_name(a->parent->h,c,n++);
-  self->indicatorOfUnitForTimeRange  = grib_arguments_get_name(a->parent->h,c,n++);
-  self->lengthOfTimeRange  = grib_arguments_get_name(a->parent->h,c,n++);
+	self->codedStep = grib_arguments_get_name(a->parent->h,c,n++);
+	self->codedUnits  = grib_arguments_get_name(a->parent->h,c,n++);
+	self->stepUnits  = grib_arguments_get_name(a->parent->h,c,n++);
+	self->indicatorOfUnitForTimeRange  = grib_arguments_get_name(a->parent->h,c,n++);
+	self->lengthOfTimeRange  = grib_arguments_get_name(a->parent->h,c,n++);
 }
 
 static void dump(grib_accessor* a, grib_dumper* dumper)
 {
-  grib_dump_double(dumper,a,NULL);
+	grib_dump_double(dumper,a,NULL);
 
 }
+
+/* Conversion of to seconds - Grib edition 2 table 4.4 */
+static int u2s2[] =  {
+  60,      /* (0)  minutes   */
+  3600,    /* (1)  hour      */
+  86400,   /* (2)  day       */
+  2592000, /* (3)  month     */
+  -1,      /* (4)  year      */
+  -1,      /* (5)  decade    */
+  -1,      /* (6)  30 years  */
+  -1,      /* (7)  century   */
+  -1,      /* (8)  RESERVED  */
+  -1,      /* (9)  RESERVED  */
+  10800,   /* (10) 3 hours   */
+  21600,   /* (11) 6 hours   */
+  43200,   /* (12) 12 hours  */
+  1        /* (13) seconds   */
+};
+
+/* Note: 'stepUnits' has a different table with extra entries e.g. 15 and 30 mins */
 static int u2s[] =  {
-  60,      /* (0) minutes */
-  3600,    /* (1) hour    */
-  86400,   /* (2) day     */
-  -1, -1, -1, -1, -1, -1,-1,
-  10800,   /* (10) 3 hours */
-  21600,   /* (11) 6 hours */
-  43200,   /* (12) 12 hours */
-  1        /* (13) seconds  */
+  60,      /* (0)  minutes    */
+  3600,    /* (1)  hour       */
+  86400,   /* (2)  day        */
+  2592000, /* (3)  month      */
+  -1,      /* (4)  year       */
+  -1,      /* (5)  decade     */
+  -1,      /* (6)  30 years   */
+  -1,      /* (7)  century    */
+  -1,      /* (8)             */
+  -1,      /* (9)             */
+  10800,   /* (10) 3 hours    */
+  21600,   /* (11) 6 hours    */
+  43200,   /* (12) 12 hours   */
+  1,       /* (13) seconds    */
+  900,     /* (14) 15 minutes */
+  1800     /* (15) 30 minutes */
 };
 
 static int unpack_long(grib_accessor* a, long* val, size_t *len)
 {
-  grib_accessor_step_in_units* self = (grib_accessor_step_in_units*)a;
-  int err = 0;
-  long codedStep,codedUnits,stepUnits;
-  grib_handle* h=a->parent->h;
+	grib_accessor_step_in_units* self = (grib_accessor_step_in_units*)a;
+	int err = 0;
+	long codedStep,codedUnits,stepUnits;
+	grib_handle* h=a->parent->h;
+	int factor=0;
+	long u2sf,u2sf_step_unit;
 
-  
-  if((err = grib_get_long_internal(h,self->codedUnits,&codedUnits))) return err;
-  if((err = grib_get_long_internal(h,self->stepUnits,&stepUnits))) return err;
-  if((err = grib_get_long_internal(h,self->codedStep,&codedStep))) return err;
 
-  if (stepUnits!=codedUnits) {
-    *val=codedStep*u2s[codedUnits];
-    if (*val%u2s[stepUnits]!=0) {
-      err = grib_set_long_internal(h,self->stepUnits,codedUnits);
-	  *val=codedStep;
-	  return err;
-    }
-    *val = *val / u2s[stepUnits];
-  } else *val=codedStep;
+	if((err = grib_get_long_internal(h,self->codedUnits,&codedUnits))) return err;
+	if((err = grib_get_long_internal(h,self->stepUnits,&stepUnits))) return err;
+	if((err = grib_get_long_internal(h,self->codedStep,&codedStep))) return err;
 
-  return GRIB_SUCCESS;
+	if (stepUnits!=codedUnits) {
+		*val=codedStep*u2s2[codedUnits];
+		if (*val<0) {
+			factor=60;
+			if (u2s2[codedUnits] % factor) return GRIB_DECODING_ERROR;
+			if (u2s[stepUnits] % factor) return GRIB_DECODING_ERROR;
+			u2sf=u2s2[codedUnits]/factor;
+			*val=codedStep*u2sf;
+			u2sf_step_unit=u2s[stepUnits]/factor;
+		} else {
+			u2sf_step_unit=u2s[stepUnits];
+		}
+
+		if (*val % u2sf_step_unit!=0) {
+			err = grib_set_long_internal(h,self->stepUnits,codedUnits);
+			*val=codedStep;
+			return err;
+		}
+		*val = *val / u2sf_step_unit;
+	} else *val=codedStep;
+
+	return GRIB_SUCCESS;
 }
 
 static int pack_long(grib_accessor* a, const long* val, size_t *len)
 {
-  grib_accessor_step_in_units* self = (grib_accessor_step_in_units*)a;
-  grib_handle* h=a->parent->h;
-  int err = 0;
-  long codedStep,codedUnits,stepUnits;
-  long oldStep;
-  char codedUnitsString[20]={0,};
-  char stepUnitsString[20]={0,};
-  size_t codedUnitsStringLength=sizeof(codedUnitsString);
-  size_t stepUnitsStringLength=sizeof(stepUnitsString);
-  
-  if((err = grib_get_long_internal(h,self->codedUnits,&codedUnits))) return err;
-  if((err = grib_get_long_internal(h,self->stepUnits,&stepUnits))) return err;
+	grib_accessor_step_in_units* self = (grib_accessor_step_in_units*)a;
+	grib_handle* h=a->parent->h;
+	int err = 0;
+	long codedStep,codedUnits,stepUnits;
+	long oldStep=0;
+	long indicatorOfUnitForTimeRange,lengthOfTimeRange;
 
-  unpack_long(a,&oldStep,len);
-  
-  if (stepUnits!=codedUnits) {
-    codedStep=*val * u2s[stepUnits];
-    if (codedStep%u2s[codedUnits]!=0) {
-      if((err = grib_get_string_internal(h,self->codedUnits,
-          codedUnitsString,&codedUnitsStringLength))) return err;
-      if((err = grib_get_string_internal(h,self->stepUnits,
-          stepUnitsString,&stepUnitsStringLength))) return err;
-      grib_context_log(h->context,GRIB_LOG_ERROR,
-                       "unable to convert %s=%ld %s in %s",
-                       a->name,*val,
-                       stepUnitsString,codedUnitsString);
-      return GRIB_WRONG_STEP_UNIT;
-    }
-    codedStep = codedStep / u2s[codedUnits];
-  } else codedStep=*val;
+	if((err = grib_get_long_internal(h,self->codedUnits,&codedUnits))) return err;
+	if((err = grib_get_long_internal(h,self->stepUnits,&stepUnits))) return err;
 
-  if (self->indicatorOfUnitForTimeRange) {
-	  long indicatorOfUnitForTimeRange,lengthOfTimeRange;
-	  if((err = grib_get_long_internal(h,
-		  self->indicatorOfUnitForTimeRange,&indicatorOfUnitForTimeRange))) return err;
-	  if((err = grib_get_long_internal(h,
-		  self->lengthOfTimeRange,&lengthOfTimeRange))) return err;
-	  
-	  if (codedUnits == indicatorOfUnitForTimeRange ) lengthOfTimeRange-=codedStep-oldStep;
-	  else lengthOfTimeRange-=codedStep * u2s[codedUnits]/u2s[indicatorOfUnitForTimeRange];
-	  lengthOfTimeRange = lengthOfTimeRange > 0 ? lengthOfTimeRange : 0 ;
-	  grib_set_long_internal(a->parent->h,self->lengthOfTimeRange,lengthOfTimeRange);
-  }
-  
-  return grib_set_long_internal(a->parent->h,self->codedStep,codedStep);
+	unpack_long(a,&oldStep,len);
+
+	if (stepUnits!=codedUnits) {
+		codedStep=*val * u2s[stepUnits];
+		if (codedStep%u2s2[codedUnits]!=0) {
+			codedUnits=stepUnits;
+			err = grib_set_long_internal(h,self->codedUnits,codedUnits);
+			if (err!=GRIB_SUCCESS) return err;
+			codedStep=*val;
+		}
+		else {
+			codedStep = codedStep / u2s2[codedUnits];
+		}
+	}
+	else {
+		codedStep = *val;
+	}
+
+	if (self->indicatorOfUnitForTimeRange) {
+		if((err = grib_get_long_internal(h,
+				self->indicatorOfUnitForTimeRange,&indicatorOfUnitForTimeRange))) return err;
+		if((err = grib_get_long_internal(h,
+				self->lengthOfTimeRange,&lengthOfTimeRange))) return err;
+		if (codedUnits == indicatorOfUnitForTimeRange )
+			lengthOfTimeRange-=codedStep-oldStep;
+		else lengthOfTimeRange-=codedStep * u2s2[codedUnits]/u2s2[indicatorOfUnitForTimeRange];
+		lengthOfTimeRange = lengthOfTimeRange > 0 ? lengthOfTimeRange : 0 ;
+		err = grib_set_long_internal(a->parent->h,self->lengthOfTimeRange,lengthOfTimeRange);
+		if (err!=GRIB_SUCCESS) return err;
+	}
+
+	return grib_set_long_internal(a->parent->h,self->codedStep,codedStep);
 }
-
