@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*! \file grib_api.h
   \brief grib_api C header file
@@ -25,7 +26,8 @@ extern "C" {
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/times.h>
+#include <sys/time.h>
+#include <math.h>
 
 #define GRIB_API_VERSION (GRIB_API_MAJOR_VERSION*10000+GRIB_API_MINOR_VERSION*100+GRIB_API_REVISION_VERSION)
 
@@ -717,6 +719,16 @@ int                  grib_get_offset(grib_handle* h, const char* key, size_t* of
 */
 int         grib_get_size(grib_handle* h, const char* key,size_t *size);
 
+/**
+*  Get the length of the string representation of the key, if several keys of the same name are present, the maximum length is returned
+*
+* @param h           : the handle to get the offset from
+* @param key         : the key to be searched
+* @param length        : the address of a size_t where the length will be set
+* @return            0 if OK, integer value on error
+*/
+int         grib_get_length(grib_handle* h, const char* key,size_t *length);
+
 /**
 *  Get a long value from a key, if several keys of the same name are present, the last one is returned
 *  @see  grib_set_long
@@ -1026,14 +1038,6 @@ typedef off_t    (*grib_data_seek_proc) (const grib_context* c, off_t offset, in
 */
 typedef int    (*grib_data_eof_proc) (const grib_context* c, void *stream);
 
-/**
-*  Retreive the context from a handle
-*
-* @param h           : the handle used to retreive the context from
-* @return            The handle's context, NULL it the handle is invalid
-*/
-grib_context*    grib_get_context       (grib_handle* h);
-
 /**
 *  Get the static default context
 *
@@ -1144,22 +1148,6 @@ void  grib_context_set_buffer_memory_proc(grib_context* c, grib_malloc_proc grib
        grib_free_proc gribfree,
        grib_realloc_proc gribrealloc);
 
-/**
-*  Sets the context search path for definition files
-*
-* @param c           : the context to be modified
-* @param path        : the search path to be set
-*/
-void   grib_context_set_path(grib_context* c, const char* path);
-
-/**
-*  Sets context dump mode
-*
-* @param c           : the context to be modified
-* @param mode        : the log mode to be set
-*/
-void    grib_context_set_dump_mode(grib_context* c, int mode);
-
 /**
 *  Sets the context printing procedure used for user interaction
 *
@@ -1273,6 +1261,7 @@ int grib_set_values(grib_handle* h,grib_values*  grib_values , size_t arg_count)
 grib_handle* grib_handle_new_from_partial_message_copy(grib_context* c, const void* data, size_t size);
 grib_handle* grib_handle_new_from_partial_message(grib_context* c,void* data, size_t buflen);
 int grib_is_missing(grib_handle* h, const char* key, int* err);
+int grib_is_defined(grib_handle* h, const char* key);
 int grib_set_missing(grib_handle* h, const char* key);
 int grib_get_gaussian_latitudes(long truncation,double* latitudes);
 
@@ -1285,8 +1274,15 @@ void grib_get_reduced_row(long pl,double lon_first,double lon_last,long* npoints
 
 /* read products */
 int wmo_read_any_from_file(FILE *f, void *buffer, size_t *len);
+int wmo_read_grib_from_file(FILE* f,void* buffer,size_t* len);
+int wmo_read_bufr_from_file(FILE* f,void* buffer,size_t* len);
+int wmo_read_gts_from_file(FILE* f,void* buffer,size_t* len);
 int wmo_read_any_from_stream(void *stream_data, long (*stream_proc )(void *, void *buffer, long len ), void *buffer, size_t *len);
-void *wmo_read_any_from_file_malloc(FILE *f, int *err);
+void *wmo_read_any_from_file_malloc(FILE *f,size_t *size,int *err);
+void *wmo_read_gts_from_file_malloc(FILE* f,size_t *size,int* err);
+void *wmo_read_bufr_from_file_malloc(FILE* f,size_t *size,int* err);
+grib_handle* grib_gts_handle_new_from_file ( grib_context* c, FILE* f,int *error );
+grib_handle* grib_bufr_handle_new_from_file ( grib_context* c, FILE* f,int *error );
 
 struct grib_points {
     grib_context* context;
@@ -1375,6 +1371,8 @@ typedef struct grib_util_grid_spec {
 #define GRIB_UTIL_PACKING_TYPE_JPEG    3
 #define GRIB_UTIL_PACKING_TYPE_GRID_COMPLEX    4
 #define GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE    5
+#define GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE_MATRIX    6
+#define GRIB_UTIL_PACKING_TYPE_GRID_SECOND_ORDER    7
 
 #define GRIB_UTIL_PACKING_SAME_AS_INPUT    	0
 #define GRIB_UTIL_PACKING_USE_PROVIDED  	1
@@ -1389,6 +1387,7 @@ typedef struct grib_util_packing_spec {
 	/* Packing options */
 	long packing_type;
 	long packing;      
+	long boustrophedonic;
 
 	long editionNumber; /* =0 for default value */
 
@@ -1398,6 +1397,8 @@ typedef struct grib_util_packing_spec {
 	long decimalScaleFactor;  
 
 	long computeLaplacianOperator;
+	int truncateLaplacian;
+	double laplacianOperator;
 
 	/* local definition */
 	long deleteLocalDefinition; /* default(=0) local definition is taken from the input field */
@@ -1541,5 +1542,7 @@ Error codes returned by the grib_api functions.
 #define GRIB_DIFFERENT_EDITION		-53
 /** Value is different */
 #define GRIB_VALUE_DIFFERENT		-54
+/** Invalid key value */
+#define GRIB_INVALID_KEY_VALUE		-55
 /*! @}*/
 #endif
