@@ -79,7 +79,7 @@ grib_action *grib_action_create_set_iarray(grib_context *context, const char *na
 grib_action *grib_action_create_noop(grib_context *context, const char *fname);
 
 /* action_class_write.c */
-grib_action *grib_action_create_write(grib_context *context, const char *name, int append);
+grib_action *grib_action_create_write(grib_context *context, const char *name, int append, int padtomultiple);
 
 /* action_class_print.c */
 grib_action *grib_action_create_print(grib_context *context, const char *name, char *outname);
@@ -108,6 +108,7 @@ int grib_unpack_string(grib_accessor *a, char *v, size_t *len);
 int grib_unpack_long(grib_accessor *a, long *v, size_t *len);
 long grib_accessor_get_native_type(grib_accessor *a);
 long grib_get_next_position_offset(grib_accessor *a);
+long grib_string_length(grib_accessor *a);
 long grib_byte_offset(grib_accessor *a);
 long grib_byte_count(grib_accessor *a);
 long grib_value_count(grib_accessor *a);
@@ -227,6 +228,8 @@ int grib_g1_step_apply_units(long *start, long *end, long *step_unit, long *P1,
 
 /* grib_accessor_class_g2step_range.c */
 
+/* grib_accessor_class_data_g22order_packing.c */
+
 /* grib_accessor_class_mars_step.c */
 
 /* grib_accessor_class_g1param.c */
@@ -247,6 +250,8 @@ int grib_g1_step_apply_units(long *start, long *end, long *step_unit, long *P1,
 
 /* grib_accessor_class_g2latlon.c */
 
+/* grib_accessor_class_g2lon.c */
+
 /* grib_accessor_class_global_gaussian.c */
 
 /* grib_accessor_class_gen.c */
@@ -286,6 +291,8 @@ int grib_g1_step_apply_units(long *start, long *end, long *step_unit, long *P1,
 /* grib_accessor_class_iterator.c */
 grib_iterator *grib_iterator_new(grib_handle *h, unsigned long flags, int *error);
 
+/* grib_accessor_class_message.c */
+
 /* grib_accessor_class_nearest.c */
 grib_nearest *grib_nearest_new(grib_handle *h, int *error);
 
@@ -294,6 +301,8 @@ grib_box *grib_box_new(grib_handle *h, int *error);
 
 /* grib_accessor_class_ksec1expver.c */
 
+/* grib_accessor_class_laplacian.c */
+
 /* grib_accessor_class_label.c */
 
 /* grib_accessor_class_long.c */
@@ -322,6 +331,8 @@ grib_box *grib_box_new(grib_handle *h, int *error);
 
 /* grib_accessor_class_signed.c */
 
+/* grib_accessor_class_signed_bits.c */
+
 /* grib_accessor_class_section.c */
 
 /* grib_accessor_class_step_in_units.c */
@@ -381,6 +392,8 @@ second_order_packed *grib_get_second_order_groups(grib_context *c, const unsigne
 
 /* grib_accessor_class_variable.c */
 
+/* grib_accessor_class_second_order_bits_per_value.c */
+
 /* grib_accessor_class_data_g2simple_packing.c */
 
 /* grib_accessor_class_data_g2simple_packing_with_preprocessing.c */
@@ -393,12 +406,20 @@ second_order_packed *grib_get_second_order_groups(grib_context *c, const unsigne
 
 /* grib_accessor_class_data_2order_packing_count.c */
 
+/* grib_accessor_class_data_g1second_order_row_by_row_packing.c */
+
+/* grib_accessor_class_data_g1second_order_constant_width_packing.c */
+
+/* grib_accessor_class_data_g1second_order_general_packing.c */
+
+/* grib_accessor_class_data_g1second_order_general_extended_packing.c */
+
 /* grib_accessor_class_g2grid.c */
 
-/* grib_accessor_class_data_g22order_packing.c */
-
 /* grib_accessor_class_data_apply_bitmap.c */
 
+/* grib_accessor_class_data_apply_boustrophedonic.c */
+
 /* grib_accessor_class_data_secondary_bitmap.c */
 
 /* grib_accessor_class_data_g1secondary_bitmap.c */
@@ -423,6 +444,12 @@ second_order_packed *grib_get_second_order_groups(grib_context *c, const unsigne
 
 /* grib_accessor_class_select_step_template.c */
 
+/* grib_accessor_class_local_definition.c */
+
+/* grib_accessor_class_g2_eps.c */
+
+/* grib_accessor_class_g2_mars_labeling.c */
+
 /* grib_accessor_class_md5.c */
 
 /* grib_jasper_encoding.c */
@@ -499,6 +526,12 @@ int grib_index_search(grib_index *index, grib_index_key *keys);
 
 /* grib_accessor_class_unsigned.c */
 
+/* grib_accessor_class_unsigned_bits.c */
+
+/* grib_accessor_class_spd.c */
+
+/* grib_accessor_class_sum.c */
+
 /* grib_accessor_class_vector.c */
 
 /* grib_accessor_class_long_vector.c */
@@ -691,6 +724,7 @@ int grib_file_pool_read(grib_context *c, FILE *fh);
 int grib_file_pool_write(FILE *fh);
 grib_file *grib_file_open(const char *filename, const char *mode, int *err);
 void grib_file_close(const char *filename, int *err);
+void grib_file_close_all(int *err);
 grib_file *grib_get_file(const char *filename, int *err);
 grib_file *grib_find_file(short id);
 grib_file *grib_file_new(grib_context *c, const char *name, int *err);
@@ -718,6 +752,8 @@ grib_handle *grib_handle_new_from_multi_message(grib_context *c, void **data, si
 grib_handle *grib_handle_new_from_file(grib_context *c, FILE *f, int *error);
 grib_handle *grib_handle_new_from_nc_file(grib_context *c, const char *file, int *error);
 grib_handle *grib_handle_headers_only_new_from_file(grib_context *c, FILE *f, int *error);
+grib_handle *grib_gts_handle_new_from_file(grib_context *c, FILE *f, int *error);
+grib_handle *grib_bufr_handle_new_from_file(grib_context *c, FILE *f, int *error);
 grib_multi_handle *grib_multi_handle_new(grib_context *c);
 int grib_multi_handle_delete(grib_multi_handle *h);
 int grib_multi_handle_append(grib_handle *h, int start_section, grib_multi_handle *mh);
@@ -757,14 +793,16 @@ int grib_hash_keys_get_size(grib_itrie *t);
 /* grib_io.c */
 off_t stdio_tell(void *data);
 int stdio_seek(void *data, off_t len);
-int stdio_read(void *data, void *buf, int len, int *err);
+size_t stdio_read(void *data, void *buf, size_t len, int *err);
 int wmo_read_any_from_file(FILE *f, void *buffer, size_t *len);
 int wmo_read_grib_from_file(FILE *f, void *buffer, size_t *len);
 int wmo_read_bufr_from_file(FILE *f, void *buffer, size_t *len);
+int wmo_read_gts_from_file(FILE *f, void *buffer, size_t *len);
 int wmo_read_any_from_stream(void *stream_data, long (*stream_proc )(void *, void *buffer, long len ), void *buffer, size_t *len);
-void *wmo_read_any_from_file_malloc(FILE *f, int *err);
-void *wmo_read_grib_from_file_malloc(FILE *f, int *err);
-void *wmo_read_bufr_from_file_malloc(FILE *f, int *err);
+void *wmo_read_gts_from_file_malloc(FILE *f, size_t *size, int *err);
+void *wmo_read_any_from_file_malloc(FILE *f, size_t *size, int *err);
+void *wmo_read_grib_from_file_malloc(FILE *f, size_t *size, int *err);
+void *wmo_read_bufr_from_file_malloc(FILE *f, size_t *size, int *err);
 int grib_read_any_headers_only_from_file_alloc(grib_context *ctx, FILE *f, void **buffer, size_t *length, off_t *offset);
 int grib_read_any_from_file_alloc(grib_context *ctx, FILE *f, void **buffer, size_t *length);
 int grib_read_any_headers_only_from_file(grib_context *ctx, FILE *f, void *buffer, size_t *len);
@@ -882,6 +920,7 @@ const char *grib_get_accessor_class_name(grib_handle *h, const char *name);
 int _grib_get_double_array_internal(grib_handle *h, grib_accessor *a, double *val, size_t buffer_len, size_t *decoded_length);
 int grib_get_double_array_internal(grib_handle *h, const char *name, double *val, size_t *length);
 int grib_get_double_array(grib_handle *h, const char *name, double *val, size_t *length);
+int grib_get_string_length(grib_handle *h, const char *name, size_t *size);
 int grib_get_size(grib_handle *h, const char *name, size_t *size);
 int grib_get_count(grib_handle *h, const char *name, size_t *size);
 int _grib_get_size(grib_handle *h, grib_accessor *a, size_t *size);
@@ -963,11 +1002,14 @@ grib_nearest *grib_nearest_factory(grib_handle *h, grib_arguments *args);
 
 /* grib_nearest_class_reduced.c */
 
+/* grib_nearest_class_latlon_reduced.c */
+
 /* grib_nearest_class_sh.c */
 
 /* grib_iterator_class_polar_stereographic.c */
 
 /* grib_iterator_class_lambert_azimuthal_equal_area.c */
+/* grib_iterator_class_lambert_conformal.c */
 
 /* grib_iterator.c */
 int grib_get_data(grib_handle *h, double *lats, double *lons, double *values, size_t *size);
@@ -1018,6 +1060,8 @@ grib_handle *grib_util_sections_copy(grib_handle *hfrom, grib_handle *hto, int w
 grib_string_list *grib_util_get_param_id(const char *mars_param);
 grib_string_list *grib_util_get_mars_param(const char *param_id);
 grib_handle *grib_util_set_spec(grib_handle *h, const grib_util_grid_spec *spec, const grib_util_packing_spec *packing_spec, int flags, const double *data_values, size_t data_values_count, int *err);
+int grib_moments(grib_handle *h, double east, double north, double west, double south, int order, double *moments, long *count);
+int parse_keyval_string(char* grib_tool, char* arg, int values_required, int default_type, grib_values values[], int* count);
 
 /* compile.c */
 void grib_compile_flags(grib_compiler *c, long flags);
@@ -1060,20 +1104,15 @@ const char *grib_binop_double_proc_name(grib_binop_double_proc proc);
 const char *grib_unop_long_proc_name(grib_unop_long_proc proc);
 const char *grib_unop_double_proc_name(grib_unop_double_proc proc);
 
-/* md5.c */
-void cvs_MD5Init(struct cvs_MD5Context *ctx);
-void cvs_MD5Update(struct cvs_MD5Context *ctx, unsigned char const *buf, unsigned len);
-void cvs_MD5Final(unsigned char digest[16 ], struct cvs_MD5Context *ctx);
-void cvs_MD5Transform(cvs_uint32 buf[4 ], const unsigned char inraw[64 ]);
-int main(int argc, char **argv);
-
 /* grib_bits_any_endian.c */
 unsigned long grib_decode_unsigned_long(const unsigned char *p, long *bitp, long nbits);
 int grib_encode_unsigned_long(unsigned char *p, unsigned long val, long *bitp, long nbits);
 int grib_encode_unsigned_longb(unsigned char *p, unsigned long val, long *bitp, long nb);
 
 /* grib_bits_any_endian_simple.c */
+int grib_decode_long_array(const unsigned char *p, long *bitp, long bitsPerValue, size_t n_vals, long *val);
 int grib_decode_double_array(const unsigned char *p, long *bitp, long bitsPerValue, double reference_value, double s, double d, size_t n_vals, double *val);
 int grib_decode_double_array_complex(const unsigned char *p, long *bitp, long nbits, double reference_value, double s, double *d, size_t size, double *val);
+int grib_encode_long_array(size_t n_vals, const long *val, long bits_per_value, unsigned char *p, long *off);
 int grib_encode_double_array(size_t n_vals, const double *val, long bits_per_value, double reference_value, double d, double divisor, unsigned char *p, long *off);
 int grib_encode_double_array_complex(size_t n_vals, double *val, long nbits, double reference_value, double *scal, double d, double divisor, unsigned char *p, long *bitp);
