@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 #include "grib_api_internal.h"
@@ -12,22 +13,90 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+grib_string_list grib_file_not_found;
+
+#if GRIB_PTHREADS
+static pthread_once_t once  = PTHREAD_ONCE_INIT;
+
+static pthread_mutex_t mutex_mem = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t mutex_c = PTHREAD_MUTEX_INITIALIZER;
+
+static void init() {
+	pthread_mutexattr_t attr;
+	pthread_mutexattr_init(&attr);
+	pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
+	pthread_mutex_init(&mutex_c,&attr);
+	pthread_mutex_init(&mutex_mem,&attr);
+	pthread_mutexattr_destroy(&attr);
+}
+
+#endif
+
+
 #if MANAGE_MEM
 
 #else
 static long cnt = 0;
 static long cntp = 0;
 
-static void  default_long_lasting_free(const grib_context* c, void* p)   {free(p);cntp--;}
-static void* default_long_lasting_malloc(const grib_context* c, size_t size) {cntp++;return malloc(size);}
-
-static void  default_buffer_free(const grib_context* c, void* p)   {free(p);cntp--;}
-static void* default_buffer_malloc(const grib_context* c, size_t size) {cntp++;return malloc(size);}
-static void* default_buffer_realloc(const grib_context* c, void* p, size_t size) {return realloc(p,size);}
-
-static void  default_free(const grib_context* c, void* p)   {free(p);cnt--;}
-static void* default_malloc(const grib_context* c, size_t size) {cnt++;return malloc(size);}
-static void* default_realloc(const grib_context* c, void* p, size_t size) {return realloc(p,size);}
+static void  default_long_lasting_free(const grib_context* c, void* p)   {
+	GRIB_PTHREAD_ONCE(&once,&init);
+	free(p);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cntp--;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+}
+static void* default_long_lasting_malloc(const grib_context* c, size_t size) {
+	void* ret;
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cntp++;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+	ret=malloc(size);
+	return ret;
+}
+static void  default_buffer_free(const grib_context* c, void* p)   {
+	GRIB_PTHREAD_ONCE(&once,&init);
+	free(p);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cntp--;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+}
+static void* default_buffer_malloc(const grib_context* c, size_t size) {
+	void* ret;
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cntp++;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+	ret=malloc(size);
+	return ret;
+}
+static void* default_buffer_realloc(const grib_context* c, void* p, size_t size) {
+	void* ret;
+	ret=realloc(p,size);
+	return ret;
+}
+static void  default_free(const grib_context* c, void* p)   {
+	GRIB_PTHREAD_ONCE(&once,&init);
+	free(p);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cnt--;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+}
+static void* default_malloc(const grib_context* c, size_t size) {
+	void* ret;
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cnt++;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+	ret=malloc(size);
+	return ret;
+}
+static void* default_realloc(const grib_context* c, void* p, size_t size) {
+	void* ret;
+	ret=realloc(p,size);
+	return ret;
+}
 #endif
 
 static size_t  default_read(const grib_context* c, void *ptr, size_t size, void *stream) {
@@ -76,21 +145,21 @@ size_t  grib_context_write(const grib_context* c,const void *ptr, size_t size, v
 static void  default_log(const grib_context* c, int level, const char* mess){
 	if (!c) c=grib_context_get_default();
 	if(level == GRIB_LOG_ERROR)   {
-		fprintf(stderr, "GRIB_API ERROR   :  %s\n", mess);
+		fprintf(c->log_stream, "GRIB_API ERROR   :  %s\n", mess);
 		/*Assert(1==0);*/
 	}
-	if(level == GRIB_LOG_FATAL)   fprintf(stderr, "GRIB_API ERROR   :  %s\n", mess);
-	if(level == GRIB_LOG_DEBUG && c->debug>0)   fprintf(stderr, "GRIB_API DEBUG   :  %s\n", mess);
-	if(level == GRIB_LOG_WARNING) fprintf(stderr, "GRIB_API WARNING :  %s\n", mess);
-	if(level == GRIB_LOG_INFO)    fprintf(stderr, "GRIB_API INFO    :  %s\n", mess);
+	if(level == GRIB_LOG_FATAL)   fprintf(c->log_stream, "GRIB_API ERROR   :  %s\n", mess);
+	if(level == GRIB_LOG_DEBUG && c->debug>0)   fprintf(c->log_stream, "GRIB_API DEBUG   :  %s\n", mess);
+	if(level == GRIB_LOG_WARNING) fprintf(c->log_stream, "GRIB_API WARNING :  %s\n", mess);
+	if(level == GRIB_LOG_INFO)    fprintf(c->log_stream, "GRIB_API INFO    :  %s\n", mess);
 
 	if(level == GRIB_LOG_FATAL) { Assert(0);}
 
 	if(getenv("GRIB_API_FAIL_IF_LOG_MESSAGE"))
 	{
 		long n = atol(getenv("GRIB_API_FAIL_IF_LOG_MESSAGE"));
-		if(n >= 1 && level == GRIB_LOG_ERROR) exit(1);
-		if(n >= 2 && level == GRIB_LOG_WARNING) exit(1);
+		if(n >= 1 && level == GRIB_LOG_ERROR) Assert(0);
+		if(n >= 2 && level == GRIB_LOG_WARNING) Assert(0);
 	}
 
 }
@@ -136,37 +205,40 @@ static  grib_context default_grib_context = {
 	0,                            /* write_on_fail              */
 	0,                            /* no_abort                   */
 	0,                            /* io_buffer_size             */
+	0,                            /* no_big_group_split         */
+	0,                            /* no_spd                     */
+	0,                            /* keep_matrix                */
 	0,                            /* grib_definition_files_path */
-	0,                            /* grib_templates_path        */
+	0,                            /* grib_samples_path          */
 	0,                            /* grib_concept_path          */
 	0,                            /* grib_reader                */
 	0,                            /* user data                  */
 	GRIB_REAL_MODE8,              /* real mode for fortran      */
 
 #if MANAGE_MEM
-	&grib_transient_free,                /* free_mem                   */
-	&grib_transient_malloc,              /* alloc_mem                  */
-	&grib_transient_realloc,              /* realloc_mem                  */
+	&grib_transient_free,         /* free_mem                   */
+	&grib_transient_malloc,       /* alloc_mem                  */
+	&grib_transient_realloc,      /* realloc_mem                */
 
-	&grib_permanent_free,   /* free_persistant_mem        */
-	&grib_permanent_malloc, /* alloc_persistant_mem       */
+	&grib_permanent_free,         /* free_persistant_mem        */
+	&grib_permanent_malloc,       /* alloc_persistant_mem       */
 
-	&grib_buffer_free,                /* buffer_free_mem                   */
-	&grib_buffer_malloc,              /* buffer_alloc_mem                  */
-	&grib_buffer_realloc,              /* buffer_realloc_mem                  */
+	&grib_buffer_free,            /* buffer_free_mem            */
+	&grib_buffer_malloc,          /* buffer_alloc_mem           */
+	&grib_buffer_realloc,         /* buffer_realloc_mem         */
 
 #else
 
-	&default_free,                /* free_mem                   */
-	&default_malloc,              /* alloc_mem                  */
-	&default_realloc,             /* realloc_mem                */
+	&default_free,                /* free_mem                  */
+	&default_malloc,              /* alloc_mem                 */
+	&default_realloc,             /* realloc_mem               */
 
-	&default_long_lasting_free,   /* free_persistant_mem        */
-	&default_long_lasting_malloc, /* alloc_persistant_mem       */
+	&default_long_lasting_free,   /* free_persistant_mem       */
+	&default_long_lasting_malloc, /* alloc_persistant_mem      */
 
-	&default_buffer_free,         /* free_buffer_mem            */
-	&default_buffer_malloc,        /* alloc_buffer_mem           */
-	&default_buffer_realloc,        /* realloc_buffer_mem           */
+	&default_buffer_free,         /* free_buffer_mem           */
+	&default_buffer_malloc,       /* alloc_buffer_mem          */
+	&default_buffer_realloc,      /* realloc_buffer_mem        */
 #endif
 
 	&default_read,                /* file read procedure        */
@@ -186,59 +258,81 @@ static  grib_context default_grib_context = {
 	0,                            /* handle_total_count         */
 	0,                            /* message_file_offset        */
 	0,                            /* no_fail_on_wrong_length    */
-	0,                            /* gts_header_on */
-	0,                            /* gribex_mode_on */
-	0,                            /* keys (grib_trie*) */
-	0,                            /* keys_count */
-	0,                            /* concepts_index */
-	0,                            /* concepts_count */
-	{0,},                         /* concepts */
-	0,                            /* def_files */
-	0,                            /* ieee_packing */
-	0,                            /* blacklist */
-    0                             /* classes */
+	0,                            /* gts_header_on              */
+	0,                            /* gribex_mode_on             */
+	0,                            /* large_constant_fields      */
+	0,                            /* keys (grib_trie*)          */
+	0,                            /* keys_count                 */
+	0,                            /* concepts_index             */
+	0,                            /* concepts_count             */
+	{0,},                         /* concepts                   */
+	0,                            /* def_files                  */
+	0,                            /* ieee_packing               */
+	0,                            /* blacklist                  */
+	0,                            /* log_stream                 */
+   0                             /* classes                    */
 #if GRIB_PTHREADS
-		,PTHREAD_MUTEX_INITIALIZER     /*mutex*/
+		,PTHREAD_MUTEX_INITIALIZER  /* mutex                     */
 #endif
 };
 
-#if GRIB_PTHREADS
-static pthread_once_t once  = PTHREAD_ONCE_INIT;
-static void init() {
-	pthread_mutexattr_t attr;
-	pthread_mutexattr_init(&attr);
-	pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
-	pthread_mutex_init(&(default_grib_context.mutex),&attr);
-	pthread_mutexattr_destroy(&attr);
-
-}
-#endif
 
 grib_context* grib_context_get_default(){
 	GRIB_PTHREAD_ONCE(&once,&init);
-	GRIB_MUTEX_LOCK(&(default_grib_context.mutex));
 
 	if(!default_grib_context.inited)
 	{
-		const char * write_on_fail = getenv("GRIB_API_WRITE_ON_FAIL");
-		const char * no_abort = getenv("GRIB_API_NO_ABORT");
-		const char * debug = getenv("GRIB_API_DEBUG");
-		const char *gribex=getenv("GRIB_GRIBEX_MODE_ON");
-		const char *ieee_packing=getenv("GRIB_IEEE_PACKING");
-		const char *io_buffer_size=getenv("GRIB_API_IO_BUFFER_SIZE");
+		const char * write_on_fail = NULL;
+		const char * large_constant_fields = NULL;
+		const char * no_abort = NULL;
+		const char * debug = NULL;
+		const char *gribex=NULL;
+		const char *ieee_packing=NULL;
+		const char *io_buffer_size=NULL;
+		const char *log_stream=NULL;
+		const char *no_big_group_split=NULL;
+		const char *no_spd=NULL;
+		const char *keep_matrix=NULL;
+
+		GRIB_MUTEX_LOCK(&mutex_c);
+
+		write_on_fail = getenv("GRIB_API_WRITE_ON_FAIL");
+		large_constant_fields = getenv("GRIB_API_LARGE_CONSTANT_FIELDS");
+		no_abort = getenv("GRIB_API_NO_ABORT");
+		debug = getenv("GRIB_API_DEBUG");
+		gribex=getenv("GRIB_GRIBEX_MODE_ON");
+		ieee_packing=getenv("GRIB_IEEE_PACKING");
+		io_buffer_size=getenv("GRIB_API_IO_BUFFER_SIZE");
+		log_stream=getenv("GRIB_API_LOG_STREAM");
+		no_big_group_split=getenv("GRIB_API_NO_BIG_GROUP_SPLIT");
+		no_spd=getenv("GRIB_API_NO_SPD");
+		keep_matrix=getenv("GRIB_API_KEEP_MATRIX");
 		default_grib_context.inited = 1;
 		default_grib_context.io_buffer_size = io_buffer_size ? atoi(io_buffer_size) : 0;
+		default_grib_context.no_big_group_split = no_big_group_split ? atoi(no_big_group_split) : 0;
+		default_grib_context.no_spd = no_spd ? atoi(no_spd) : 0;
+		default_grib_context.keep_matrix = keep_matrix ? atoi(keep_matrix) : 1;
 		default_grib_context.write_on_fail  = write_on_fail ? atoi(write_on_fail) : 0;
 		default_grib_context.no_abort  = no_abort ? atoi(no_abort) : 0;
 		default_grib_context.debug  = debug ? atoi(debug) : 0;
 		default_grib_context.gribex_mode_on=gribex ? atoi(gribex) : 0;
+		default_grib_context.large_constant_fields = large_constant_fields ? atoi(large_constant_fields) : 0;
 		default_grib_context.ieee_packing=ieee_packing ? atoi(ieee_packing) : 0;
-		default_grib_context.grib_templates_path = getenv("GRIB_SAMPLES_PATH");
-		if (!default_grib_context.grib_templates_path)
-			default_grib_context.grib_templates_path = getenv("GRIB_TEMPLATES_PATH");
+		default_grib_context.grib_samples_path = getenv("GRIB_SAMPLES_PATH");
+		default_grib_context.log_stream=stderr;
+		if (!log_stream) { 
+			default_grib_context.log_stream=stderr;
+		} else if (!strcmp(log_stream,"stderr") ) {
+			default_grib_context.log_stream=stderr;
+		} else if (!strcmp(log_stream,"stdout") ) {
+			default_grib_context.log_stream=stdout;
+		} 
+
+		if (!default_grib_context.grib_samples_path)
+			default_grib_context.grib_samples_path = getenv("GRIB_TEMPLATES_PATH");
 #ifdef GRIB_TEMPLATES_PATH
-		if(!default_grib_context.grib_templates_path)
-			default_grib_context.grib_templates_path = GRIB_TEMPLATES_PATH ;
+		if(!default_grib_context.grib_samples_path)
+			default_grib_context.grib_samples_path = GRIB_TEMPLATES_PATH ;
 #endif
 		default_grib_context.grib_definition_files_path = getenv("GRIB_DEFINITION_PATH");
 #ifdef GRIB_DEFINITION_PATH
@@ -253,11 +347,11 @@ grib_context* grib_context_get_default(){
 				&(default_grib_context.concepts_count));
 		default_grib_context.def_files=grib_trie_new(&(default_grib_context));
 		default_grib_context.classes=grib_trie_new(&(default_grib_context));
+
+		GRIB_MUTEX_UNLOCK(&mutex_c);
 	}
 
 
-	GRIB_MUTEX_UNLOCK(&(default_grib_context.mutex));
-
 	return &default_grib_context;
 }
 
@@ -272,6 +366,7 @@ grib_context* grib_context_new(grib_context* parent)
 
 	if (!parent) parent=grib_context_get_default();
 
+	GRIB_PTHREAD_ONCE(&once,&init);
 	GRIB_MUTEX_LOCK(&(parent->mutex));
 
 	c = (grib_context*)grib_context_malloc_clear_persistent(&default_grib_context,sizeof(grib_context));
@@ -295,10 +390,10 @@ grib_context* grib_context_new(grib_context* parent)
 	c->print               = default_grib_context.print    ;
 	c->user_data           = default_grib_context.user_data;
 	c->def_files           = default_grib_context.def_files;
-	
+
 #if GRIB_PTHREADS
 	pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
-	pthread_mutex_init(&(c->mutex),&attr);
+	pthread_mutex_init(&mutex_c,&attr);
 	pthread_mutexattr_destroy(&attr);
 #endif
 
@@ -306,9 +401,25 @@ grib_context* grib_context_new(grib_context* parent)
 	return c;
 }
 
+/* Hopefully big enough. Note: GRIB_DEFINITION_PATH can contain SEVERAL colon-separated sub-paths */
+#define DEF_PATH_MAXLEN 8192
+
+/* GRIB-235: Resolve path to expand symbolic links etc */
+static char* resolve_path(grib_context* c, char* path)
+{
+	char* result = NULL;
+	char resolved[DEF_PATH_MAXLEN+1];
+	if (!realpath(path, resolved)) {
+		result = grib_context_strdup(c, path); /* Failed to resolve. Use original path */
+	} else {
+		result = grib_context_strdup(c, resolved);
+	}
+	return result;
+}
+
 static int init_definition_files_dir(grib_context* c) {
 	int err=0;
-	char* path=NULL;
+	char path[DEF_PATH_MAXLEN];
 	char* p=NULL;
 	char* dir=NULL;
 	grib_string_list* next=NULL;
@@ -318,17 +429,21 @@ static int init_definition_files_dir(grib_context* c) {
 	if (c->grib_definition_files_dir) return 0;
 	if (!c->grib_definition_files_path) return GRIB_NO_DEFINITIONS;
 
-	path=c->grib_definition_files_path;
+	/* Note: strtok modifies its first argument so we copy */
+	strncpy(path, c->grib_definition_files_path, DEF_PATH_MAXLEN);
 
-	GRIB_MUTEX_LOCK(&(c->mutex));
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_c);
 
 	p=path;
 	while(*p!=':' && *p!='\0') p++;
 
 	if (*p != ':') {
+		/* No colon found so use it as is */
 		c->grib_definition_files_dir=(grib_string_list*)grib_context_malloc_clear_persistent(c,sizeof(grib_string_list));
-		c->grib_definition_files_dir->value=grib_context_strdup(c,path);
+		c->grib_definition_files_dir->value = resolve_path(c, path);
 	} else {
+		/* definitions path contains at least one colon so must consist of several paths */
 		dir=strtok(path,":");
 
 		while (dir != NULL) {
@@ -339,12 +454,12 @@ static int init_definition_files_dir(grib_context* c) {
 				c->grib_definition_files_dir=(grib_string_list*)grib_context_malloc_clear_persistent(c,sizeof(grib_string_list));
 				next=c->grib_definition_files_dir;
 			}
-			next->value=grib_context_strdup(c,dir);
+			next->value = resolve_path(c, dir);
 			dir=strtok(NULL,":");
 		}
 	}
 
-	GRIB_MUTEX_UNLOCK(&(c->mutex));
+	GRIB_MUTEX_UNLOCK(&mutex_c);
 
 	return err;
 }
@@ -357,16 +472,18 @@ char *grib_context_full_path(grib_context* c,const char* basename)
 	grib_string_list* fullpath=0;
 	if (!c) c=grib_context_get_default();
 
-	GRIB_MUTEX_LOCK(&(c->mutex));
+	GRIB_PTHREAD_ONCE(&once,&init);
 
 	if(*basename == '/' || *basename ==  '.') {
 		return (char*)basename;
 	} else {
-		if ((fullpath=grib_trie_get(c->def_files,basename))!=NULL) {
+		fullpath=grib_trie_get(c->def_files,basename);
+		if (fullpath!=NULL) {
 			return fullpath->value;
 		}
-		if (!c->grib_definition_files_dir)
+		if (!c->grib_definition_files_dir) {
 			err=init_definition_files_dir(c);
+		}
 
 		if (err != GRIB_SUCCESS) {
 			grib_context_log(c,GRIB_LOG_ERROR,
@@ -382,9 +499,10 @@ char *grib_context_full_path(grib_context* c,const char* basename)
 				fullpath=grib_context_malloc_clear_persistent(c,sizeof(grib_string_list));
 				Assert(fullpath);
 				fullpath->value=grib_context_strdup(c,full);
+				GRIB_MUTEX_LOCK(&mutex_c);
 				grib_trie_insert(c->def_files,basename,fullpath);
 				grib_context_log(c,GRIB_LOG_DEBUG,"Found def file %s",full);
-				GRIB_MUTEX_UNLOCK(&(c->mutex));
+				GRIB_MUTEX_UNLOCK(&mutex_c);
 				return fullpath->value;
 			}
 			dir=dir->next;
@@ -392,10 +510,11 @@ char *grib_context_full_path(grib_context* c,const char* basename)
 
 	}
 
+	GRIB_MUTEX_LOCK(&mutex_c);
 	/* Store missing files so we don't check for them again and again */
 	grib_trie_insert(c->def_files,basename,&grib_file_not_found);
 	/*grib_context_log(c,GRIB_LOG_ERROR,"Def file \"%s\" not found",basename);*/
-	GRIB_MUTEX_UNLOCK(&(c->mutex));
+	GRIB_MUTEX_UNLOCK(&mutex_c);
 	full[0]=0;
 	return NULL;
 }
@@ -452,7 +571,7 @@ void grib_context_reset(grib_context* c){
 
 void grib_context_delete( grib_context* c){
 	if (!c) c=grib_context_get_default();
-	
+
 	grib_hash_keys_delete( c->keys);
 	grib_trie_delete(c->def_files);
 
@@ -590,7 +709,8 @@ void grib_context_log(const grib_context *c,int level, const char* fmt, ...)
 	va_list list;
 
 	/* Save some CPU */
-	if( level == GRIB_LOG_DEBUG && c->debug<1 )
+	if( (level == GRIB_LOG_DEBUG && c->debug<1) || 
+			(level == GRIB_LOG_WARNING && c->debug<2) )
 		return;
 
 	va_start(list,fmt);
