@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 #include <ctype.h>
@@ -230,7 +231,7 @@ static void dump_string(grib_dumper* d,grib_accessor* a,const char* comment)
 #endif
 
   fprintf(self->dumper.out,"    p    = \"%s\";\n",value);
-  fprintf(self->dumper.out,"    size = strlen(p)+1;\n");
+  fprintf(self->dumper.out,"    size = strlen(p);\n");
   fprintf(self->dumper.out,"    GRIB_CHECK(grib_set_string(h,\"%s\",p,&size),%d);\n",a->name,0);
 
   if(err)
@@ -384,7 +385,7 @@ static void header(grib_dumper* d,grib_handle* h)
     ret=grib_get_long(h,"editionNumber",&edition);
   if (ret != GRIB_SUCCESS) {
     grib_context_log(h->context,GRIB_LOG_ERROR,"Unable to get edition number.");
-    exit(ret);
+    Assert(0);
   }
 
   fprintf(self->dumper.out,
