@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /**************************************
  *  Enrico Fucile
@@ -137,7 +138,11 @@ static void dump_long(grib_dumper* d,grib_accessor* a,const char* comment)
 {
   grib_dumper_default *self = (grib_dumper_default*)d;
   long value; size_t size = 1;
-  int err = grib_unpack_long(a,&value,&size);
+  long *values=NULL;
+  int err = 0;
+  int i;
+
+  size=grib_value_count(a);
 
   if ( (a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0)
     return;
@@ -149,6 +154,13 @@ static void dump_long(grib_dumper* d,grib_accessor* a,const char* comment)
     fprintf(self->dumper.out,"# type %s \n",a->creator->op);
   }
 
+  if (size>1) {
+    values=grib_context_malloc_clear(a->parent->h->context,sizeof(long)*size);
+    err=grib_unpack_long(a,values,&size);
+  } else {
+    err=grib_unpack_long(a,&value,&size);
+  }
+
   aliases(d,a);
   if(comment) {
     fprintf(self->dumper.out,"  ");
@@ -161,11 +173,23 @@ static void dump_long(grib_dumper* d,grib_accessor* a,const char* comment)
   } else
     fprintf(self->dumper.out,"  ");
 
-  if( ((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0) && grib_is_missing_internal(a) )
-    fprintf(self->dumper.out,"%s = MISSING;",a->name);
-  else
-    fprintf(self->dumper.out,"%s = %ld;",a->name,value);
-
+  if (size>1) {
+    int cols=19;
+    int count=0;
+    fprintf(self->dumper.out,"%s = { \t",a->name);
+    for (i=0;i<size;i++) {
+        if (count>cols) {fprintf(self->dumper.out,"\n\t\t\t\t");count=0;}
+        fprintf(self->dumper.out,"%ld ",values[i]);
+        count++;
+    }
+    fprintf(self->dumper.out,"}\n");
+    grib_context_free(a->parent->h->context,values);
+  } else {
+	  if( ((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0) && grib_is_missing_internal(a) )
+		fprintf(self->dumper.out,"%s = MISSING;",a->name);
+	  else
+		fprintf(self->dumper.out,"%s = %ld;",a->name,value);
+  }
 
   if(err) {
     fprintf(self->dumper.out,"  ");
@@ -277,9 +301,23 @@ static void dump_double(grib_dumper* d,grib_accessor* a,const char* comment)
 static void dump_string(grib_dumper* d,grib_accessor* a,const char* comment)
 {
   grib_dumper_default *self = (grib_dumper_default*)d;
-  char value[1024]; size_t size = sizeof(value);
-  int err = grib_unpack_string(a,value,&size);
-  char *p = value;
+  char *value=NULL; 
+  char *p = NULL;
+  size_t size = 0;
+  grib_context* c=NULL;
+  int err = grib_get_string_length(a->parent->h,a->name,&size);
+
+  c=a->parent->h->context;
+  if (size==0) return;
+
+  value=grib_context_malloc_clear(c,size);
+  if (!value) {
+  	grib_context_log(c,GRIB_LOG_FATAL,"unable to allocate %d bytes",(int)size);
+	return;
+  }
+
+  err = grib_unpack_string(a,value,&size);
+  p=value;
 
   if ( (a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0)
     return;
@@ -317,6 +355,7 @@ static void dump_string(grib_dumper* d,grib_accessor* a,const char* comment)
   }
 
   fprintf(self->dumper.out,"\n");
+  grib_context_free(c,value);
 }
 
 static void dump_bytes(grib_dumper* d,grib_accessor* a,const char* comment)
