@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /**************************************
  *  Enrico Fucile
@@ -136,8 +137,20 @@ int i;
 static void dump_long(grib_dumper* d,grib_accessor* a,const char* comment)
 {
   grib_dumper_wmo *self = (grib_dumper_wmo*)d;
-  long value; size_t size = 1;
-  int err = grib_unpack_long(a,&value,&size);
+  long value = 0; size_t size =0;
+  long *values=NULL;
+  int err = 0;
+  int i=0;
+
+  size=grib_value_count(a);
+
+  
+  if (size>1) {
+  	values=grib_context_malloc_clear(a->parent->h->context,sizeof(long)*size);
+	err=grib_unpack_long(a,values,&size);
+  } else {
+	err=grib_unpack_long(a,&value,&size);
+  }
 
   if( a->length == 0  &&
       (d->option_flags & GRIB_DUMP_FLAG_CODED) != 0)
@@ -149,21 +162,33 @@ static void dump_long(grib_dumper* d,grib_accessor* a,const char* comment)
 
   set_begin_end(d,a);
 
-  /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
 
   print_offset(self->dumper.out,self->begin,self->end);
 
   if ((d->option_flags & GRIB_DUMP_FLAG_TYPE) != 0)
         fprintf(self->dumper.out,"%s ",a->creator->op);
 
-  if( ((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0) && grib_is_missing_internal(a) )
-    fprintf(self->dumper.out,"%s = MISSING",a->name);
-  else
-    fprintf(self->dumper.out,"%s = %ld",a->name,value);
+  if (size>1) {
+	int cols=19;
+	int count=0;
+	fprintf(self->dumper.out,"%s = { \t",a->name);
+	for (i=0;i<size;i++) {
+		if (count>cols) {fprintf(self->dumper.out,"\n\t\t\t\t");count=0;}
+		fprintf(self->dumper.out,"%ld ",values[i]);
+		count++;
+	}
+	fprintf(self->dumper.out,"}\n");
+	grib_context_free(a->parent->h->context,values);
+  } else {
+	  if( ((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0) && grib_is_missing_internal(a) )
+		fprintf(self->dumper.out,"%s = MISSING",a->name);
+	  else
+		fprintf(self->dumper.out,"%s = %ld",a->name,value);
 
-  print_hexadecimal(self->dumper.out,d->option_flags,a);
+	  print_hexadecimal(self->dumper.out,d->option_flags,a);
 
-  if(comment) fprintf(self->dumper.out," [%s]",comment);
+	  if(comment) fprintf(self->dumper.out," [%s]",comment);
+  }
   if(err)
     fprintf(self->dumper.out," *** ERR=%d (%s)",err,grib_get_error_message(err));
 
@@ -179,10 +204,9 @@ static void dump_bits(grib_dumper* d,grib_accessor* a,const char* comment)
 {
   grib_dumper_wmo *self = (grib_dumper_wmo*)d;
   int i;
-  long value; size_t size = 1;
+  long value = 0; size_t size = 1;
   int err = grib_unpack_long(a,&value,&size);
 
-
   if( a->length == 0  &&
       (d->option_flags & GRIB_DUMP_FLAG_CODED) != 0)
     return;
@@ -254,9 +278,18 @@ static void dump_double(grib_dumper* d,grib_accessor* a,const char* comment)
 static void dump_string(grib_dumper* d,grib_accessor* a,const char* comment)
 {
   grib_dumper_wmo *self = (grib_dumper_wmo*)d;
-  char value[1024]; size_t size = sizeof(value);
-  int err = grib_unpack_string(a,value,&size);
-  char *p = value;
+  size_t size=0;
+  char *value=NULL;
+  char *p=NULL;
+  int err = grib_get_string_length(a->parent->h,a->name,&size);
+
+  value=grib_context_malloc_clear(a->parent->h->context,size);
+  if (!value) {
+  	grib_context_log(a->parent->h->context,GRIB_LOG_FATAL,"unable to allocate %d bytes",(int)size);
+	return;
+  }
+  err=grib_unpack_string(a,value,&size);
+  p=value;
 
   if( a->length == 0  &&
       (d->option_flags & GRIB_DUMP_FLAG_CODED) != 0)
@@ -280,6 +313,7 @@ static void dump_string(grib_dumper* d,grib_accessor* a,const char* comment)
     fprintf(self->dumper.out," *** ERR=%d (%s)",err,grib_get_error_message(err));
   aliases(d,a);
   fprintf(self->dumper.out,"\n");
+  if (value) grib_context_free(a->parent->h->context,value);
 }
 
 static void dump_bytes(grib_dumper* d,grib_accessor* a,const char* comment)
