@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include <stdarg.h>
 #include "grib_api_internal.h"
@@ -92,6 +93,27 @@
   }
   Assert(1==0);
 
+}
+
+ void grib_expression_compile(grib_expression* g,grib_compiler* f)
+{
+  grib_expression_class *c = g->cclass;
+    if(!c->compile)
+    {
+        fprintf(stderr, "NO COMPILE METHOD %s\n", c->name);
+        Assert(0);
+    }
+  while(c)
+  {
+    if(c->compile)
+    {
+      c->compile(g,f);
+      return;
+    }
+    c = c->super ? *(c->super) : NULL;
+  }
+  Assert(1==0);
+
 }
 
  void grib_expression_free(grib_context* ctx,grib_expression* g)
@@ -146,7 +168,7 @@ int grib_expression_set_value(grib_handle* h,grib_expression* g,grib_values* v)
         grib_context_log(h->context,GRIB_LOG_ERROR,
             "grib_expression_set_value: unable to evaluate %s as string",
             grib_expression_get_name( g));
-        exit(ret);
+        return ret;
       }
       Assert(v->string_value != buffer);
       Assert(v->string_value);
@@ -161,10 +183,11 @@ int grib_expression_set_value(grib_handle* h,grib_expression* g,grib_values* v)
 
 /*----------------------------------------*/
 
-grib_arguments *grib_arguments_new(grib_context* c,grib_expression* g)
+grib_arguments *grib_arguments_new(grib_context* c,grib_expression* g, grib_arguments* n)
 {
   grib_arguments* l = (grib_arguments*)grib_context_malloc_clear_persistent(c,sizeof(grib_arguments));
   l->expression = g;
+  l->next       = n;
   return l;
 }
 
