@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /***************************************************************************
@@ -14,6 +15,28 @@
  ***************************************************************************/
 #include "grib_api_internal.h"
 
+/* This is the internal version number of our definition files. We need to change it whenever an older engine */
+/* will not be able to work with the current definitions. See the key "internalVersion" in boot,def  */
+#define LATEST_VERSION  22
+
+/* #if GRIB_PTHREADS */
+#if 0
+static pthread_once_t once  = PTHREAD_ONCE_INIT;
+static pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t mutex2 = PTHREAD_MUTEX_INITIALIZER;
+
+static void init() {
+    pthread_mutexattr_t attr;
+    pthread_mutexattr_init(&attr);
+    pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init(&mutex1,&attr);
+    pthread_mutex_init(&mutex2,&attr);
+    pthread_mutexattr_destroy(&attr);
+
+}
+#endif
+
+
 static grib_handle* grib_handle_new_from_file_no_multi ( grib_context* c, FILE* f,int *error );
 static grib_handle* grib_handle_new_from_file_multi ( grib_context* c, FILE* f,int *error );
 static int grib2_get_next_section ( unsigned char* msgbegin,size_t msglen,unsigned char** secbegin,size_t* seclen,int* secnum,int* err );
@@ -140,17 +163,33 @@ grib_handle* grib_new_handle ( grib_context* c )
 	g = ( grib_handle* ) grib_context_malloc_clear ( c,sizeof ( grib_handle ) );
 
 
-	if ( g == NULL )
+	if ( g == NULL ) {
 		grib_context_log ( c,GRIB_LOG_ERROR,"grib_new_handle: cannot allocate handle" );
-	else
+	} else {
 		g->context = c;
+	}
 
 	grib_context_log ( c,GRIB_LOG_DEBUG,"grib_new_handle: allocated handle %p",g );
 
 	return g;
 }
 
+static void check_definitions_version(grib_handle* h)
+{
+	/* Check version of definition files is compatible with the engine */
+	int ret = 0;
+	long defs_file_version = 0;
+	if (grib_is_defined(h, "internalVersion")) {
+		ret = grib_get_long_internal(h, "internalVersion", &defs_file_version);
+		if (ret == GRIB_SUCCESS && (defs_file_version > LATEST_VERSION)) {
+			grib_context_log(h->context, GRIB_LOG_FATAL,
+					"Definition files version (%d) is greater than engine version (%d)!\n"
+					"These definition files are for a later version of the grib api engine.\n",
+					defs_file_version, LATEST_VERSION);
 
+		}
+	}
+}
 
 static grib_handle* grib_handle_create ( grib_handle  *gl, grib_context* c,void* data, size_t buflen )
 {
@@ -198,6 +237,7 @@ static grib_handle* grib_handle_create ( grib_handle  *gl, grib_context* c,void*
 	grib_section_adjust_sizes ( gl->root,0,0 );
 	grib_section_post_init ( gl->root );
 
+	check_definitions_version(gl);
 
 	return gl;
 
@@ -221,14 +261,42 @@ grib_handle* grib_handle_new_from_samples ( grib_context* c, const char* name )
 	   g = grib_internal_template(c,name);
 	   if(g) return g;
 	 */
+	if (c->debug==-1) {
+		printf("GRIB_API DEBUG: grib_handle_new_from_samples '%s'\n", name);
+	}
+
 	g=grib_external_template ( c,name );
 	if ( !g )
-		grib_context_log ( c,GRIB_LOG_ERROR,"Unable to locate sample file %s.tmpl\n                    in %s",name,c->grib_templates_path );
+		grib_context_log (c,GRIB_LOG_ERROR,"Unable to locate sample file %s.tmpl\n                    in %s",name,c->grib_samples_path);
 
 	return g;
 }
 
 
+int grib_write_message(grib_handle* h,const char* file,const char* mode) {
+	FILE* fh=0;
+	int err;
+	const void *buffer; size_t size;
+
+	fh=fopen(file,mode);
+	if (!fh) {
+		perror(file);
+		return GRIB_IO_PROBLEM;
+	}
+	err=grib_get_message(h,&buffer,&size);
+	if (err) return err;
+
+	if(fwrite(buffer,1,size,fh) != size) {
+		perror(file);
+		return GRIB_IO_PROBLEM;
+	}
+	if (fclose(fh) != 0) {
+		perror(file);
+		return GRIB_IO_PROBLEM;
+	}
+	return 0;
+}
+
 grib_handle* grib_handle_clone ( grib_handle* h )
 {
 	return grib_handle_new_from_message_copy ( h->context, h->buffer->data, h->buffer->ulength );
@@ -240,11 +308,14 @@ grib_handle* grib_handle_new_from_message_copy ( grib_context* c, const void* da
 	grib_handle *g = NULL;
 	void* copy =NULL;
 	if ( c == NULL ) c = grib_context_get_default();
+
+
 	c->handle_file_count=0;
 	c->handle_total_count=0;
 	copy = grib_context_malloc ( c,size );
-	if ( !copy )
+	if ( !copy ) {
 		return NULL;
+	}
 
 	memcpy ( copy,data,size );
 
@@ -289,9 +360,11 @@ grib_handle* grib_handle_new_from_partial_message ( grib_context* c,void* data,
 grib_handle* grib_handle_new_from_message ( grib_context* c,void* data, size_t buflen )
 {
 	grib_handle  *gl = NULL;
+	grib_handle  *h = NULL;
 	if ( c == NULL ) c = grib_context_get_default();
 	gl = grib_new_handle ( c );
-	return grib_handle_create ( gl,  c, data,  buflen );
+	h=grib_handle_create ( gl,  c, data,  buflen );
+	return h;
 }
 
 
@@ -324,13 +397,16 @@ grib_handle* grib_handle_new_from_file ( grib_context* c, FILE* f,int *error )
 	off_t offset=0;
 	grib_handle* h=0;
     if (!f) {*error=GRIB_IO_PROBLEM; return NULL;}
+
 	if ( c == NULL ) c = grib_context_get_default();
+
 	if ( ( offset=grib_context_tell ( c,f ) ) < 0 )
 	{
 		/*grib_context_log ( c, GRIB_LOG_ERROR, "grib_handle_new_from_file: cannot get offset" );*/
 		*error=GRIB_IO_PROBLEM;
 		return NULL; 
 	}
+
 	if ( offset == 0 ) c->handle_file_count=0;
 
 	if ( c->multi_support_on ) h=grib_handle_new_from_file_multi ( c,f,error );
@@ -476,6 +552,35 @@ static grib_handle* grib_handle_new_multi ( grib_context* c,unsigned char** data
 	return gl;
 }
 
+grib_handle* grib_handle_new_from_nc_file(grib_context* c,const char* file,int *error) {
+	FILE* fh=NULL;
+	char msg[4];
+	grib_handle* h=NULL;
+	size_t len=4;
+
+	fh=fopen(file,"r");
+	if (!fh) {
+		grib_context_log(c,(GRIB_LOG_ERROR)|(GRIB_LOG_PERROR),"unable to open %s",file);
+		perror(file);
+		return NULL;
+	}
+	if (fread(msg,1,3,fh)!=3) {
+		perror(file);
+		fclose(fh);
+		return NULL;
+	}
+	fclose(fh);
+	msg[3]='X';
+
+	h = grib_handle_new_from_message_copy ( c, msg, len );
+
+	if ( !h ) {
+	     *error = GRIB_DECODING_ERROR;
+	     grib_context_log ( c, GRIB_LOG_ERROR, "grib_handle_new_nc_from_file : cannot create handle \n" );
+	    return NULL;
+	 }
+	 return h;
+}
 
 static grib_handle* grib_handle_new_from_file_multi ( grib_context* c, FILE* f,int *error )
 {
@@ -500,7 +605,7 @@ static grib_handle* grib_handle_new_from_file_multi ( grib_context* c, FILE* f,i
 	if ( !gm->message )
 	{
 		gts_header_offset=grib_context_tell( c,f);
-		*error = grib_read_any_from_file_alloc ( c, f, &data, &olen );
+		data = wmo_read_grib_from_file_malloc ( f, &olen,error );
         end_msg_offset=grib_context_tell ( c,f );
         start_msg_offset=end_msg_offset-olen;
 		gm->message_length=olen;
@@ -646,6 +751,120 @@ static grib_handle* grib_handle_new_from_file_multi ( grib_context* c, FILE* f,i
 	return gl;
 }
 
+grib_handle* grib_handle_headers_only_new_from_file ( grib_context* c, FILE* f,int *error )
+{
+	void *data = NULL;
+	size_t olen = 0;
+	grib_handle  *gl = NULL;
+	off_t start_msg_offset=0;
+
+	if ( c == NULL ) c = grib_context_get_default();
+
+	*error = grib_read_any_headers_only_from_file_alloc ( c, f, &data, &olen ,&start_msg_offset);
+
+	if ( *error != GRIB_SUCCESS )
+	{
+		if ( data ) grib_context_free ( c,data );
+		if ( *error == GRIB_END_OF_FILE ) *error = GRIB_SUCCESS;
+		return NULL;
+	}
+
+	gl = grib_handle_new_from_partial_message ( c, data, olen );
+	
+	if ( !gl )
+	{
+		*error = GRIB_DECODING_ERROR;
+		grib_context_log ( gl->context, GRIB_LOG_ERROR, "grib_handle_new_from_file : cannot create handle \n" );
+		grib_context_free ( c,data );
+		return NULL;
+	}
+
+    gl->offset=start_msg_offset;
+	gl->buffer->property = GRIB_MY_BUFFER;
+	c->handle_file_count++;
+	c->handle_total_count++;
+
+	return gl;
+}
+
+grib_handle* grib_gts_handle_new_from_file ( grib_context* c, FILE* f,int *error )
+{
+	void *data = NULL;
+	size_t olen = 0;
+	grib_handle  *gl = NULL;
+	off_t end_msg_offset=0,start_msg_offset=0;
+
+	if ( c == NULL ) c = grib_context_get_default();
+
+	data = wmo_read_gts_from_file_malloc ( f, &olen,error );
+	end_msg_offset=grib_context_tell ( c,f );
+	start_msg_offset=end_msg_offset-olen;
+
+	if ( *error != GRIB_SUCCESS )
+	{
+		if ( data ) grib_context_free ( c,data );
+
+		if ( *error == GRIB_END_OF_FILE ) *error = GRIB_SUCCESS;
+		return NULL;
+	}
+
+	gl = grib_handle_new_from_message ( c, data, olen );
+	
+	if ( !gl )
+	{
+		*error = GRIB_DECODING_ERROR;
+		grib_context_log ( gl->context, GRIB_LOG_ERROR, "grib_handle_new_from_file : cannot create handle \n" );
+		grib_context_free ( c,data );
+		return NULL;
+	}
+
+    gl->offset=start_msg_offset;
+	gl->buffer->property = GRIB_MY_BUFFER;
+	c->handle_file_count++;
+	c->handle_total_count++;
+
+	return gl;
+}
+
+grib_handle* grib_bufr_handle_new_from_file ( grib_context* c, FILE* f,int *error )
+{
+	void *data = NULL;
+	size_t olen = 0;
+	grib_handle  *gl = NULL;
+	off_t end_msg_offset=0,start_msg_offset=0;
+
+	if ( c == NULL ) c = grib_context_get_default();
+
+	data = wmo_read_bufr_from_file_malloc ( f, &olen,error );
+	end_msg_offset=grib_context_tell ( c,f );
+	start_msg_offset=end_msg_offset-olen;
+
+	if ( *error != GRIB_SUCCESS )
+	{
+		if ( data ) grib_context_free ( c,data );
+
+		if ( *error == GRIB_END_OF_FILE ) *error = GRIB_SUCCESS;
+		return NULL;
+	}
+
+	gl = grib_handle_new_from_message ( c, data, olen );
+	
+	if ( !gl )
+	{
+		*error = GRIB_DECODING_ERROR;
+		grib_context_log ( gl->context, GRIB_LOG_ERROR, "grib_handle_new_from_file : cannot create handle \n" );
+		grib_context_free ( c,data );
+		return NULL;
+	}
+
+    gl->offset=start_msg_offset;
+	gl->buffer->property = GRIB_MY_BUFFER;
+	c->handle_file_count++;
+	c->handle_total_count++;
+
+	return gl;
+}
+
 static grib_handle* grib_handle_new_from_file_no_multi ( grib_context* c, FILE* f,int *error )
 {
 	void *data = NULL;
@@ -657,8 +876,7 @@ static grib_handle* grib_handle_new_from_file_no_multi ( grib_context* c, FILE*
 	int gtslen=0;
 
 	if ( c == NULL ) c = grib_context_get_default();
-
-	*error = grib_read_any_from_file_alloc ( c, f, &data, &olen );
+	data = wmo_read_grib_from_file_malloc ( f, &olen, error );
 	end_msg_offset=grib_context_tell ( c,f );
 	start_msg_offset=end_msg_offset-olen;
 
@@ -695,13 +913,14 @@ static grib_handle* grib_handle_new_from_file_no_multi ( grib_context* c, FILE*
 	if ( !gl )
 	{
 		*error = GRIB_DECODING_ERROR;
-		grib_context_log ( gl->context, GRIB_LOG_ERROR, "grib_handle_new_from_file : cannot create handle \n" );
+		grib_context_log ( c, GRIB_LOG_ERROR, "grib_handle_new_from_file : cannot create handle \n" );
 		grib_context_free ( c,data );
 		return NULL;
 	}
 
     gl->offset=start_msg_offset;
 	gl->buffer->property = GRIB_MY_BUFFER;
+
 	c->handle_file_count++;
 	c->handle_total_count++;
 
@@ -865,8 +1084,14 @@ int grib_get_message_copy ( grib_handle* h ,  void* message,size_t *len )
 
 int grib_get_message ( grib_handle* h,const void** msg,size_t* size )
 {
+	long totalLength=0;
+	int ret=0;
 	*msg  =  h->buffer->data;
 	*size =  h->buffer->ulength;
+
+	ret=grib_get_long(h,"totalLength",&totalLength);
+	if (!ret) *size=totalLength;
+
 	if ( h->context->gts_header_on && h->gts_header )
 	{
 		char strbuf[10];
@@ -926,6 +1151,8 @@ grib_handle *grib_handle_new ( grib_context* c )
 
 	h->header_mode=1;
 
+	check_definitions_version(h);
+
 	return h;
 }
 
@@ -1019,7 +1246,7 @@ static int grib2_has_next_section ( unsigned char* msgbegin,size_t msglen,unsign
 static void grib2_build_message ( grib_context* context,unsigned char* sections[],size_t sections_len[],void** data,size_t* len )
 {
 	int i=0;
-	char* end="7777";
+	char* theEnd="7777";
 	unsigned char* sec0;
 	unsigned char* p=0;
 	size_t msglen=0;
@@ -1046,7 +1273,7 @@ static void grib2_build_message ( grib_context* context,unsigned char* sections[
 		}
 	}
 
-	memcpy ( p,end,4 );
+	memcpy ( p,theEnd,4 );
 
 	grib_encode_unsigned_long ( *data,msglen,&bitp,64 );
 
@@ -1078,6 +1305,12 @@ void grib_gts_header_off ( grib_context* c )
 	c->gts_header_on=0;
 }
 
+int grib_get_gribex_mode ( grib_context* c)
+{
+	if ( !c ) c=grib_context_get_default();
+	return c->gribex_mode_on;
+}
+
 void grib_gribex_mode_on ( grib_context* c )
 {
 	if ( !c ) c=grib_context_get_default();
