@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /***************************************************************************
  *   Enrico Fucile  - 06.01.2009                                           *
@@ -103,13 +104,11 @@ unsigned long grib_ieee_to_long(double x) {
   }
 
   /* Overflow */
-  Assert(x <= ieee_table.vmax);
-/*
   if (x > ieee_table.vmax) {
-  printf("grib_ieee_to_long: x=%.20e > xmax=%.20e\n",x,ieee_table.vmax);
-  return 0;
-}
-*/
+    fprintf(stderr, "grib_ieee_to_long: Number is too large: x=%.20e > xmax=%.20e\n", x, ieee_table.vmax);
+    Assert(0);
+    return 0;
+  }
 
   binary_search(ieee_table.v, 254, x, &e);
 
@@ -149,7 +148,11 @@ double grib_ieeefloat_error(double x) {
   if (x < ieee_table.vmin) return ieee_table.vmin;
 
   /* Overflow */
-  Assert(x <= ieee_table.vmax);
+  if (x > ieee_table.vmax) {
+  		fprintf(stderr, "grib_ieeefloat_error: Number is too large: x=%.20e > xmax=%.20e\n", x, ieee_table.vmax);
+  		Assert(0);
+  		return 0;
+  }
 
   binary_search(ieee_table.v, 254, x, &e);
   
