@@ -1,17 +1,17 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /*
  *
  * Description: index
  *
- * Author: Enrico Fucile <enrico.fucile@ecmwf.int>
  *
  *
  */
@@ -24,6 +24,25 @@
 #define NULL_MARKER 0
 #define NOT_NULL_MARKER 255
 
+
+/* #if GRIB_PTHREADS */
+#if 0
+static pthread_once_t once  = PTHREAD_ONCE_INIT;
+static pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t mutex2 = PTHREAD_MUTEX_INITIALIZER;
+
+static void init() {
+    pthread_mutexattr_t attr;
+    pthread_mutexattr_init(&attr);
+    pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init(&mutex1,&attr);
+    pthread_mutex_init(&mutex2,&attr);
+    pthread_mutexattr_destroy(&attr);
+
+}
+#endif
+
+
 static const char* mars_keys =
     "mars.date,mars.time,mars.expver,mars.stream,mars.class,mars.type,"
 	"mars.step,mars.param,mars.levtype,mars.levelist,mars.number,mars.iteration,"
@@ -713,7 +732,13 @@ int grib_index_write(grib_index* index,const char* filename) {
 		return err;
 	}
 
-	fclose(fh);
+	if (fclose(fh) != 0) {
+		grib_context_log(index->context,(GRIB_LOG_ERROR)|(GRIB_LOG_PERROR),
+					"Unable to write in file %s",filename);
+		perror(filename);
+		return GRIB_IO_PROBLEM;
+	}
+
 	return err;
 }
 
@@ -860,13 +885,14 @@ int grib_index_add_file(grib_index* index,const char* filename) {
 	c=index->context;
 	
 	file=grib_file_open(filename,"r",&err);
+
 	if (!file || !file->handle) return err;
 
 	if (!index->files) {
 		grib_filesid++;
 		newfile=grib_context_malloc_clear(c,sizeof(grib_file));
 		newfile->id=grib_filesid;
-		newfile->name=file->name;
+		newfile->name=strdup(file->name);
 		index->files=newfile;
 	} else {
 		indfile=index->files;
@@ -992,6 +1018,7 @@ int grib_index_add_file(grib_index* index,const char* filename) {
 	}
 
 	grib_file_close(file->name,&err);
+
 	if (err) return err;
 	index->rewind=1;
 	return GRIB_SUCCESS;
@@ -1193,6 +1220,7 @@ int grib_index_select_string(grib_index* index,const char* skey,char* value) {
 grib_handle* grib_index_get_handle(grib_field* field,int *err) {
 	grib_handle* h=NULL;
 	grib_file_open(field->file->name,"r",err);
+
 	if (*err!=GRIB_SUCCESS) return NULL;
 
 	fseeko(field->file->handle,field->offset,SEEK_SET);
