@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /**************************************
  *  Enrico Fucile
@@ -41,6 +42,7 @@ long grib_get_binary_scale_fact(double max, double min, long bpval,int *ret)
   const long last = 127; /* Depends on edition, should be parameter */
 
   unsigned long maxint = grib_power(bpval,2) - 1;
+  double dmaxint=(double)maxint;
   
   *ret=0;
 
@@ -52,12 +54,12 @@ long grib_get_binary_scale_fact(double max, double min, long bpval,int *ret)
     return 0;
 
   /* range -= 1e-10; */
-  while( (range * zs)  <= (double)maxint) {
+  while( (range * zs)  <= dmaxint) {
     scale--;
     zs *= 2;
   }
 
-  while( (range * zs ) > (double)maxint) {
+  while( (range * zs ) > dmaxint) {
     scale++;
     zs /= 2;
   }
@@ -92,6 +94,7 @@ long grib_get_bits_per_value(double max, double min, long binary_scale_factor)
   const long last = 127; /* Depends on edition, should be parameter */
 
   unsigned long maxint = grib_power(binary_scale_factor,2) - 1;
+  double dmaxint=(double)maxint;
 
   if (maxint==0) maxint=1;
 
@@ -102,9 +105,9 @@ long grib_get_bits_per_value(double max, double min, long binary_scale_factor)
     return 0;
 
   /* range -= 1e-10; */
-  while( (range * zs)  <= (double)maxint) { scale--;zs *= 2;}
+  while( (range * zs)  <= dmaxint) { scale--;zs *= 2;}
 
-  while( (range * zs ) > (double)maxint) {scale++;zs /= 2;}
+  while( (range * zs ) > dmaxint) {scale++;zs /= 2;}
 
   while( (unsigned long)(range * zs + 0.5)  <= maxint) {
     scale--;
@@ -131,6 +134,7 @@ long grib_get_decimal_scale_fact(double max, double min, long bpval,long binary_
   const long last = 127; /* Depends on edition, should be parameter */
 
   unsigned long maxint = grib_power(bpval,2) - 1;
+  double dmaxint=(double)maxint;
 
   range*=grib_power(-binary_scale,2);
 
@@ -138,11 +142,11 @@ long grib_get_decimal_scale_fact(double max, double min, long bpval,long binary_
   if(range == 0)
     return 0;
 
-  while( (range * zs ) > (double)maxint) {
+  while( (range * zs ) > dmaxint) {
     scale--;
     zs /= 10;
   }
-  while( (range * zs)  <= (double)maxint) {
+  while( (range * zs)  <= dmaxint) {
     scale++;
     zs *= 10;
   }
