@@ -1,10 +1,11 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /***************************************************************************
@@ -101,6 +102,9 @@ static grib_handle* grib_sections_copy_internal(grib_handle* hfrom,grib_handle*
 
    h=grib_handle_new_from_message(hfrom->context,buffer,totalLength);
 
+   /*to allow free of buffer*/
+   h->buffer->property = GRIB_MY_BUFFER;
+
    switch (edition) {
    	case 1:
 		if (sections[1] && sections[2]) break;
@@ -305,24 +309,22 @@ static grib_trie* init_list(const char* name) {
 	return 0;
 }
 
-static void print_values(const grib_util_grid_spec* spec,const double* data_values,size_t data_values_count,const grib_values *values,int count) {
+static void print_values(grib_context* c,const grib_util_grid_spec* spec,const double* data_values,size_t data_values_count,const grib_values *values,int count) {
 		int i;
-		printf("\n");
-		printf("XXXXXX => grib_set_values: setting %d values \n",count);
+		printf("GRIB_API DEBUG grib_util grib_set_values: setting %d values \n",count);
 
 		for(i = 0; i < count ; i++)
 		{
-			printf("%s = ",values[i].name);
 			switch(values[i].type)
 			{
-				case GRIB_TYPE_LONG: printf(" %ld;",(long)values[i].long_value); break;
-				case GRIB_TYPE_DOUBLE: printf(" %g;",values[i].double_value); break;
-				case GRIB_TYPE_STRING: printf(" \"%s\";",values[i].string_value); break;
+				case GRIB_TYPE_LONG: printf("GRIB_API DEBUG grib_util: => %s =  %ld;\n"
+						,values[i].name,(long)values[i].long_value); break;
+				case GRIB_TYPE_DOUBLE: printf("GRIB_API DEBUG grib_util: => %s = %.16e;\n"
+						,values[i].name,values[i].double_value); break;
+				case GRIB_TYPE_STRING: printf("GRIB_API DEBUG grib_util: => %s = \"%s\";\n"
+						,values[i].name,values[i].string_value); break;
 			}
-			printf("\n");
 		}
-		printf("\n");
-
 
 		if(spec->bitmapPresent) {
 			int missing = 0;
@@ -342,8 +344,6 @@ static void print_values(const grib_util_grid_spec* spec,const double* data_valu
 
 			}
 
-			printf("MISSING VALUES : %f %d out of %d (min = %f)\n", spec->missingValue, missing, data_values_count, min);
-
 		}
 }
 
@@ -366,7 +366,7 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 #define COPY_SPEC_LONG(x)     do { Assert(count<1024); values[count].name = #x; values[count].type = GRIB_TYPE_LONG;   values[count].long_value = spec->x; count++; } while(0)
 #define COPY_SPEC_DOUBLE(x)   do { Assert(count<1024); values[count].name = #x; values[count].type = GRIB_TYPE_DOUBLE; values[count].double_value = spec->x; count++; } while(0)
 
-	grib_values  values[1024];
+	grib_values  values[1024]={{0,},};
 	size_t       count = 0;
 	int i;
 	long editionNumber;
@@ -380,26 +380,44 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 	long input_decimal_scale_factor=0;
 	size_t len=100;
 	size_t input_grid_type_len=100;
-	long P;
+	double laplacianOperator;
+	int packingTypeIsSet=0;
+	int setSecondOrder=0;
+    size_t slen=17;
 
 	static grib_util_packing_spec default_packing_spec = {0, };
 
-	int debug = 1;
-
 	if(!packing_spec) {
 		packing_spec = &default_packing_spec;
 	}
 
+	/* Get edition number from input handle */
+	if((*err = grib_get_long(h,"edition",&editionNumber)) != 0)
+	{
+		grib_context* c=grib_context_get_default();
+		if (c->write_on_fail) 
+			grib_write_message(h,"error.grib","w");
+		return NULL;
+	}
+
+	if (packing_spec->deleteLocalDefinition) {
+		/* TODO: not working for grib2 */
+		SET_LONG_VALUE("deleteLocalDefinition",1);
+	}
+
 	len=100;
 	grib_get_string(h,"packingType",input_packing_type,&len);
 	grib_get_long(h,"bitsPerValue",&input_bits_per_value);
 	grib_get_long(h,"decimalScaleFactor",&input_decimal_scale_factor);
-	printf("XXXXXX => input_packing_type = %s\n",input_packing_type);
-	printf("XXXXXX => input_bits_per_value = %ld\n",input_bits_per_value);
-	printf("XXXXXX => input_decimal_scale_factor = %ld\n",input_decimal_scale_factor);
+	if (h->context->debug==-1) {
+		printf("GRIB_API DEBUG grib_util: input_packing_type = %s\n",input_packing_type);
+		printf("GRIB_API DEBUG grib_util: input_bits_per_value = %ld\n",input_bits_per_value);
+		printf("GRIB_API DEBUG grib_util: input_decimal_scale_factor = %ld\n",input_decimal_scale_factor);
+	}
 
 	if (flags & GRIB_UTIL_SET_SPEC_FLAGS_ONLY_PACKING) {
-		if (packing_spec->packing == GRIB_UTIL_PACKING_USE_PROVIDED) {
+		if (packing_spec->packing == GRIB_UTIL_PACKING_USE_PROVIDED && 
+			strcmp(input_packing_type,"grid_simple_matrix")) {
 			switch (packing_spec->packing_type) {
 				case GRIB_UTIL_PACKING_TYPE_SPECTRAL_COMPLEX:
 					if (strcmp(input_packing_type,"spectral_complex") && !strcmp(input_packing_type,"spectral_simple"))
@@ -410,9 +428,12 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 						SET_STRING_VALUE("packingType","spectral_simple");
 					break;
 				case GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE:
-					if (strcmp(input_packing_type,"grid_simple") && !strcmp(input_packing_type,"grid_complex"))
+					if (strcmp(input_packing_type,"grid_simple") && !strcmp(input_packing_type,"grid_complex") )
 						SET_STRING_VALUE("packingType","grid_simple");
 					break;
+				case GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE_MATRIX:
+					SET_STRING_VALUE("packingType","grid_simple_matrix");
+					break;
 				case GRIB_UTIL_PACKING_TYPE_GRID_COMPLEX:
 					if (strcmp(input_packing_type,"grid_complex") && !strcmp(input_packing_type,"grid_simple"))
 						SET_STRING_VALUE("packingType","grid_complex");
@@ -421,10 +442,16 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 					if (strcmp(input_packing_type,"grid_jpeg") && !strcmp(input_packing_type,"grid_simple"))
 						SET_STRING_VALUE("packingType","grid_jpeg");
 					break;
+				case GRIB_UTIL_PACKING_TYPE_GRID_SECOND_ORDER:
+						/* we delay the set of grid_second_order because we don't want
+							to do it on a field with bitsPerValue=0 */
+						setSecondOrder=1;
+					break;
 				default :
 					printf("invalid packing_spec->packing_type = %ld\n",(long)packing_spec->packing_type);
 					abort();
 			}
+			packingTypeIsSet=1;
 		}
 		switch(packing_spec->accuracy) {
 
@@ -449,15 +476,15 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 				abort();
 		}
 
-		if (packing_spec->deleteLocalDefinition) {
-			/* TODO: not working for grib2 */
-			SET_LONG_VALUE("deleteLocalDefinition",1);
+		/*nothing to be changed*/
+		if (count==0) {
+			*err = -1;
+			return h;
 		}
 
-		/*nothing to be changed*/
-		if (count==0) return h;
+		if (h->context->debug==-1) 
+			print_values(h->context,spec,data_values,data_values_count,values,count);
 
-		if (debug) print_values(spec,data_values,data_values_count,values,count);
 		if((*err = grib_set_values(h,values,count)) != 0)
 		{
 			fprintf(stderr,"GRIB_UTIL_SET_SPEC: Cannot set values  %s\n",grib_get_error_message(*err));
@@ -467,27 +494,76 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 					fprintf(stderr," %s %s\n",values[i].name,grib_get_error_message(values[i].error));
 			goto cleanup;
 		}
-		if (debug) printf("XXXXXX => grib_set_double_array \n");
-		{
+		if (h->context->debug==-1) {
 			int j=0;
-			for (j=0;j<20;j++) printf("XXXXXX => %g\n",data_values[j]);
-			printf("XXXXXX => data_values_count=%d \n",(int)data_values_count);
+		    printf("GRIB_API DEBUG grib_util: grib_set_double_array\n");
+			for (j=0;j<20;j++) printf("GRIB_API DEBUG grib_util %g\n",data_values[j]);
+			printf("GRIB_API DEBUG grib_util: data_values_count=%d \n",(int)data_values_count);
 		}
+
 		if((*err = grib_set_double_array(h,"values",data_values,data_values_count)) != 0)
 		{
 			goto cleanup;
 		}
-		if (debug) printf("XXXXXX <= grib_set_double_array \n");
+		if (h->context->debug==-1) 
+			printf("GRIB_API DEBUG grib_util: done grib_set_double_array \n");
+
+
+		/* convert to second_order if not constant field */
+        if (setSecondOrder ) {
+                size_t slen=17;
+                int ii=0;
+                int constant=1;
+				double missingValue=0;
+                double value=missingValue;
+
+				grib_get_double(h,"missingValue",&missingValue);
+                for (ii=0;ii<data_values_count;ii++) {
+                    if (data_values[ii]!=missingValue) {
+                        if (value==missingValue) {
+                            value=data_values[ii];
+                        } else {
+                            if (value!=data_values[ii]) {
+                                constant=0;
+                                break;
+                            }
+                        }
+                    }
+                }
+                if (!constant) {
+					if (editionNumber == 1 ) {
+						long numberOfGroups;
+						grib_handle* htmp=grib_handle_clone(h);
+
+						slen=17;
+						grib_set_string(htmp,"packingType","grid_second_order",&slen);
+						grib_get_long(htmp,"numberOfGroups",&numberOfGroups);
+						/* GRIBEX is not able to decode overflown numberOfGroups with SPD */
+						if (numberOfGroups>65534 && h->context->no_spd ) {
+							slen=24;
+							grib_set_string(h,"packingType","grid_second_order_no_SPD",&slen);
+							grib_handle_delete(htmp);
+						} else {
+							grib_handle_delete(h);
+							h=htmp;
+						}
+					} else {
+						slen=17;
+						grib_set_string(h,"packingType","grid_second_order",&slen);
+						grib_set_double_array(h,"values",data_values,data_values_count);
+					}
+				} else {
+					if (h->context->gribex_mode_on) {
+						h->context->gribex_mode_on=0;
+						grib_set_double_array(h,"values",data_values,data_values_count);
+						h->context->gribex_mode_on=1;
+					}
+				}
+        }
 
 		return h;
 	}
 
-	/* Get edition number from input handle */
-	if((*err = grib_get_long(h,"editionNumber",&editionNumber)) != 0)
-	{
-		return NULL;
-	}
-
 
 	switch(spec->grid_type) {
 
@@ -547,13 +623,11 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 	}
 
 	/* TODO: recycle tmp handle */
-	if (debug) printf("XXXXXX => grib_handle_new_from_template %s\n", name);
 	tmp = grib_handle_new_from_samples(NULL, name);
 	if(!tmp) {
 		*err = GRIB_INVALID_FILE;
 		return NULL;
 	}
-	if (debug) printf("XXXXXX <= grib_handle_new_from_template %s\n", name);
 
 	/* Set grid  */
 	switch(spec->grid_type) {
@@ -581,18 +655,6 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 			COPY_SPEC_DOUBLE(latitudeOfFirstGridPointInDegrees);
 			COPY_SPEC_DOUBLE(latitudeOfLastGridPointInDegrees);
 
-			if(!fabs((spec->latitudeOfFirstGridPointInDegrees - spec->latitudeOfLastGridPointInDegrees)/spec->jDirectionIncrementInDegrees) == spec->Nj)
-			{
-				printf("XXXXXXXXXXXXXXXX %g %ld\n",
-						(spec->latitudeOfFirstGridPointInDegrees - spec->latitudeOfLastGridPointInDegrees)/spec->jDirectionIncrementInDegrees,
-						spec->Nj);
-			}
-
-			/*
-			   Assert((spec->latitudeOfFirstGridPointInDegrees - spec->latitudeOfLastGridPointInDegrees)/spec->jDirectionIncrementInDegrees == spec->Nj));
-			   Assert((spec->longitudeOfLastGridPointInDegrees - spec->longitudeOfFirstGridPointInDegrees)/spec->iDirectionIncrementInDegrees == spec->Ni));
-			 */
-
 			break;
 
 		case GRIB_UTIL_GRID_SPEC_REGULAR_GG:
@@ -638,6 +700,7 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 			if (spec->missingValue) COPY_SPEC_DOUBLE(missingValue);
 
 			COPY_SPEC_DOUBLE(longitudeOfFirstGridPointInDegrees);
+			COPY_SPEC_DOUBLE(latitudeOfFirstGridPointInDegrees);
 			COPY_SPEC_LONG(Ni);
 			COPY_SPEC_LONG(Nj);
 
@@ -657,11 +720,9 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 			if (spec->missingValue) COPY_SPEC_DOUBLE(missingValue);
 			SET_LONG_VALUE("ijDirectionIncrementGiven", 0);
 
-			/* TODO: add Assert */
 			COPY_SPEC_LONG(Nj);
 			COPY_SPEC_LONG(N);
 
-			/* TODO: Compute here ... */
 			COPY_SPEC_DOUBLE(longitudeOfFirstGridPointInDegrees);
 			COPY_SPEC_DOUBLE(longitudeOfLastGridPointInDegrees);
 
@@ -680,16 +741,24 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 			if(packing_spec->packing_type == GRIB_UTIL_PACKING_TYPE_SPECTRAL_COMPLEX)
 			{
 				SET_STRING_VALUE("packingType", "spectral_complex");
+				packingTypeIsSet=1;
 				SET_LONG_VALUE("JS", 20);
 				SET_LONG_VALUE("KS", 20);
 				SET_LONG_VALUE("MS", 20);
-
-				if ((!(*err) && strcmp(input_grid_type,"sh")) || packing_spec->computeLaplacianOperator )
+				if (packing_spec->packing == GRIB_UTIL_PACKING_USE_PROVIDED && editionNumber==2 ) {
 					SET_LONG_VALUE("computeLaplacianOperator", 1);
-				else {
+				} else if ((!(*err) && strcmp(input_grid_type,"sh")) || packing_spec->computeLaplacianOperator ) {
+					SET_LONG_VALUE("computeLaplacianOperator", 1);
+					if (packing_spec->truncateLaplacian) 
+						SET_LONG_VALUE("truncateLaplacian",1);
+				} else {
 					SET_LONG_VALUE("computeLaplacianOperator", 0);
-					*err=grib_get_long(h,"P",&P);
-					SET_LONG_VALUE("P", P);
+					*err=grib_get_double(h,"laplacianOperator",&laplacianOperator);
+					if (packing_spec->truncateLaplacian) SET_LONG_VALUE("truncateLaplacian",1);
+					SET_DOUBLE_VALUE("laplacianOperator", packing_spec->laplacianOperator);
+					if (laplacianOperator) {
+						SET_DOUBLE_VALUE("laplacianOperator", laplacianOperator);
+					}
 				}
 
 			}
@@ -710,11 +779,10 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 	}
 
 	/* process packing options */
-	if(packing_spec->editionNumber) {
-		SET_LONG_VALUE("editionNumber", packing_spec->editionNumber);
-	}
+	if (!packingTypeIsSet && 
+		packing_spec->packing == GRIB_UTIL_PACKING_USE_PROVIDED && 
+		strcmp(input_packing_type,"grid_simple_matrix")) {
 
-	if (packing_spec->packing == GRIB_UTIL_PACKING_USE_PROVIDED) {
 		switch (packing_spec->packing_type) {
 			case GRIB_UTIL_PACKING_TYPE_SPECTRAL_COMPLEX:
 				if (strcmp(input_packing_type,"spectral_complex") && !strcmp(input_packing_type,"spectral_simple"))
@@ -726,7 +794,7 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 				break;
 			case GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE:
 				if (strcmp(input_packing_type,"grid_simple") && !strcmp(input_packing_type,"grid_complex"))
-					SET_STRING_VALUE("packingType","grid_simple");
+						SET_STRING_VALUE("packingType","grid_simple");
 				break;
 			case GRIB_UTIL_PACKING_TYPE_GRID_COMPLEX:
 				if (strcmp(input_packing_type,"grid_complex") && !strcmp(input_packing_type,"grid_simple"))
@@ -736,12 +804,38 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 				if (strcmp(input_packing_type,"grid_jpeg") && !strcmp(input_packing_type,"grid_simple"))
 					SET_STRING_VALUE("packingType","grid_jpeg");
 				break;
+			case GRIB_UTIL_PACKING_TYPE_GRID_SECOND_ORDER:
+						/* we delay the set of grid_second_order because we don't want
+							to do it on a field with bitsPerValue=0 */
+						setSecondOrder=1;
+				break;
 			default :
 				printf("invalid packing_spec->packing_type = %ld\n",(long)packing_spec->packing_type);
 				abort();
 		}
+	} 
+	if (!strcmp(input_packing_type,"grid_simple_matrix")) {
+		long numberOfDirections,numberOfFrequencies;
+		int ret;
+		if (h->context->keep_matrix) {
+			SET_STRING_VALUE("packingType","grid_simple_matrix");
+			ret=grib_get_long(h,"numberOfDirections",&numberOfDirections);
+			if (!ret) {
+				ret=grib_get_long(h,"numberOfDirections",&numberOfDirections);
+				SET_LONG_VALUE("NC1",numberOfDirections);
+				ret=grib_get_long(h,"numberOfFrequencies",&numberOfFrequencies);
+				SET_LONG_VALUE("NC2",numberOfFrequencies);
+				SET_LONG_VALUE("physicalFlag1",1);
+				SET_LONG_VALUE("physicalFlag2",2);
+				SET_LONG_VALUE("NR",1);
+				SET_LONG_VALUE("NC",1);
+			}
+		} else {
+			SET_STRING_VALUE("packingType","grid_simple");
+		}
 	}
 
+
 	switch(packing_spec->accuracy) {
 
 		case GRIB_UTIL_ACCURACY_SAME_BITS_PER_VALUES_AS_INPUT:
@@ -768,8 +862,6 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 			SET_LONG_VALUE("decimalScaleFactor", packing_spec->decimalScaleFactor);
 			break;
 
-
-
 		default:
 			printf("invalid packing_spec->accuracy = %ld\n",(long)packing_spec->accuracy);
 			abort();
@@ -781,22 +873,19 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 			values[count++] = packing_spec->extra_settings[i];
 		}
 	}
-	if (packing_spec->deleteLocalDefinition) {
-		/* TODO: not working for grib2 */
-		SET_LONG_VALUE("deleteLocalDefinition",1);
-	}
-
 	/* grib_write_message(h,"input.grib","w"); */
 	/* grib_write_message(tmp,"geo.grib","w"); */
-	if (debug) printf("XXXXXX => grib_utils_sections_copy %s\n", name);
 	if((outh = grib_util_sections_copy(h, tmp, GRIB_SECTION_PRODUCT | GRIB_SECTION_LOCAL,err)) == NULL)
 	{
 		goto cleanup;
 	}
-	Assert(*err == 0);
-	if (debug) printf("XXXXXX <= grib_utils_sections_copy %s\n", name);
 
-	if (debug) print_values(spec,data_values,data_values_count,values,count);
+	grib_handle_delete(tmp);
+	Assert(*err == 0);
+
+	if (h->context->debug==-1)
+		print_values(h->context,spec,data_values,data_values_count,values,count);
+
 	if((*err = grib_set_values(outh,values,count)) != 0)
 	{
 		fprintf(stderr,"SET_GRID_DATA_DESCRIPTION: Cannot set values  %s\n",grib_get_error_message(*err));
@@ -807,19 +896,103 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 		goto cleanup;
 	}
 	if (spec->pl_size!=0 && spec->grid_type==GRIB_UTIL_GRID_SPEC_REDUCED_GG) {
-		grib_set_long_array(outh,"pl",spec->pl,spec->pl_size);
+		*err=grib_set_long_array(outh,"pl",spec->pl,spec->pl_size);
+		if (*err) {
+			fprintf(stderr,"SET_GRID_DATA_DESCRIPTION: Cannot set pl  %s\n",grib_get_error_message(*err));
+			goto cleanup;
+		}
 	}
 
-	if (debug) printf("XXXXXX <= grib_set_values \n");
-
-	if (debug) printf("XXXXXX => grib_set_double_array \n");
 	if((*err = grib_set_double_array(outh,"values",data_values,data_values_count)) != 0)
 	{
+		FILE* ferror;
+		long i,count;
+		grib_context* c=grib_context_get_default();
+
+		ferror=fopen("error.data","w");
+		count=0;
+		fprintf(ferror,"# data_values_count=%ld\n",(long)data_values_count);
+		fprintf(ferror,"set values={ ");
+		for (i=0;i<data_values_count-1;i++) {
+			fprintf(ferror,"%g, ",data_values[i]);
+			if (count>10) {fprintf(ferror,"\n");count=0;}
+			count++;
+		}
+		fprintf(ferror,"%g }",data_values[data_values_count-1]);
+		fclose(ferror);
+		if (c->write_on_fail) 
+			grib_write_message(outh,"error.grib","w");
 		goto cleanup;
 	}
-	if (debug) printf("XXXXXX <= grib_set_double_array \n");
-	fflush(stdout);
 	/* grib_write_message(outh,"h.grib","w"); */
+	/* if the field is empty GRIBEX is packing as simple*/
+	/*
+	if (!strcmp(input_packing_type,"grid_simple_matrix")) {
+		long numberOfValues;
+		grib_get_long(outh,"numberOfValues",&numberOfValues);
+		if (numberOfValues==0)  {
+			slen=11;
+			grib_set_string(outh,"packingType","grid_simple",&slen);
+		}
+	}
+	*/
+
+		/* convert to second_order if not constant field */
+        if (setSecondOrder ) {
+                int ii=0;
+                int constant=1;
+				double missingValue=0;
+                double value=missingValue;
+
+				grib_get_double(outh,"missingValue",&missingValue);
+                for (ii=0;ii<data_values_count;ii++) {
+                    if (data_values[ii]!=missingValue) {
+                        if (value==missingValue) {
+                            value=data_values[ii];
+                        } else {
+                            if (value!=data_values[ii]) {
+                                constant=0;
+                                break;
+                            }
+                        }
+                    }
+                }
+                if (!constant) {
+					if (editionNumber == 1 ) {
+						long numberOfGroups;
+						grib_handle* htmp=grib_handle_clone(outh);
+
+						slen=17;
+						grib_set_string(htmp,"packingType","grid_second_order",&slen);
+						grib_get_long(htmp,"numberOfGroups",&numberOfGroups);
+						/* GRIBEX is not able to decode overflown numberOfGroups with SPD */
+						if (numberOfGroups>65534 && outh->context->no_spd) {
+							slen=24;
+							grib_set_string(outh,"packingType","grid_second_order_no_SPD",&slen);
+							grib_handle_delete(htmp);
+						} else {
+							grib_handle_delete(outh);
+							outh=htmp;
+						}
+					} else {
+						slen=17;
+						grib_set_string(outh,"packingType","grid_second_order",&slen);
+						grib_set_double_array(outh,"values",data_values,data_values_count);
+					}
+				} else {
+					if (outh->context->gribex_mode_on) {
+						outh->context->gribex_mode_on=0;
+						grib_set_double_array(outh,"values",data_values,data_values_count);
+						outh->context->gribex_mode_on=1;
+					}
+				}
+        }
+
+	if(packing_spec->editionNumber && packing_spec->editionNumber!=editionNumber) 
+		grib_set_long(outh,"edition", packing_spec->editionNumber);
+
+	if (h->context->debug==-1) 
+		printf("GRIB_API DEBUG: grib_util_set_spec end\n");
 
 	return outh;
 
@@ -827,3 +1000,238 @@ cleanup:
 	if(outh)  grib_handle_delete(outh);
 	return NULL;
 }
+
+
+int grib_moments(grib_handle* h,double east,double north,double west,double south,int order,double* moments,long *count) {
+	grib_iterator* iter=NULL;
+	int ret=0,i,j,l;
+	size_t n=0,numberOfPoints=0;
+	double *lat,*lon,*values;
+	double vlat,vlon,val;
+	double dx,dy,ddx,ddy;
+	double mass,centroidX,centroidY;
+	double missingValue;
+	grib_context* c=grib_context_get_default();
+
+	ret=grib_get_size(h,"values",&n);
+	if (ret) return ret;
+
+	lat=grib_context_malloc_clear(c,sizeof(double)*n);
+	lon=grib_context_malloc_clear(c,sizeof(double)*n);
+	values=grib_context_malloc_clear(c,sizeof(double)*n);
+
+	iter=grib_iterator_new(h,0,&ret);
+	numberOfPoints=0;
+	while(grib_iterator_next(iter,&vlat,&vlon,&val)) {
+		if (vlon>=east && vlon<=west && vlat>=south && vlat<=north) {
+			lat[numberOfPoints]=vlat;
+			lon[numberOfPoints]=vlon;
+			values[numberOfPoints]=val;
+			numberOfPoints++;
+		}
+	}
+	grib_iterator_delete(iter);
+
+	ret=grib_get_double(h,"missingValue",&missingValue);
+
+	centroidX=0;
+	centroidY=0;
+	mass=0;
+	*count=0;
+	for (i=0;i<numberOfPoints;i++) {
+		if (values[i]!=missingValue) {
+			centroidX+=lon[i]*values[i];
+			centroidY+=lat[i]*values[i];
+			mass+=values[i];
+			(*count)++;
+		}
+	}
+	centroidX/=mass;
+	centroidY/=mass;
+	mass/=*count;
+
+	for (j=0;j<order*order;j++) moments[j]=0;
+
+	for (i=0;i<numberOfPoints;i++) {
+		if (values[i]!=missingValue) {
+			dx=(lon[i]-centroidX);
+			dy=(lat[i]-centroidY);
+			ddx=1; 
+			for (j=0;j<order;j++) {
+				ddy=1;
+				for (l=0;l<order;l++) {
+					moments[j*order+l]+=ddx*ddy*values[i];
+					ddy*=dy;
+				}
+				ddx*=dx;
+			}
+		}
+	}
+	for (j=0;j<order;j++) {
+		for (l=0;l<order;l++) {
+			if (j+l>1) {
+				moments[j*order+l]=pow(fabs(moments[j*order+l]),1.0/(j+l)) / *count ;
+			} else {
+				moments[j*order+l]/=*count ;
+			}
+		}
+	}
+
+	grib_context_free(c,lat);
+	grib_context_free(c,lon);
+	grib_context_free(c,values);
+
+	return ret;
+}
+
+/* Helper function for 'parse_keyval_string' */
+static void set_value(grib_values* value,char* str,int equal)
+{
+  char *p=0,*q=0,*s=0;
+  char buf[1000]={0,};
+  grib_context* c=grib_context_get_default();
+
+  value->equal=equal;
+  q=str;
+
+  while (*q != '/' && *q!=0 ) q++;
+  if (*q=='/') {
+    s=grib_context_strdup(c,q+1);
+    value->next=grib_context_malloc_clear(c,sizeof(grib_values));
+    value->next->type=value->type;
+    value->next->name=grib_context_strdup(c,value->name);
+    set_value(value->next,s,equal);
+    grib_context_free(c,s);
+  }
+
+  memcpy(buf,str,q-str);
+
+  switch (value->type) {
+    case GRIB_TYPE_DOUBLE:
+      value->double_value = strtod(buf,&p);
+      if(*p != 0) value->has_value=1;
+      else if (!strcmp(str,"missing") ||
+               !strcmp(str,"MISSING") ||
+               !strcmp(str,"Missing")  ) {
+        value->type=GRIB_TYPE_MISSING;
+        value->has_value=1;
+      }
+    break;
+    case GRIB_TYPE_LONG:
+      value->long_value = strtol(buf,&p,10);
+      if(*p != 0) value->has_value=1;
+      else if (!strcmp(buf,"missing") ||
+                !strcmp(buf,"MISSING") ||
+                !strcmp(buf,"Missing")  ) {
+        value->type=GRIB_TYPE_MISSING;
+        value->has_value=1;
+      }
+    break;
+    case GRIB_TYPE_STRING:
+      if ( !strcmp(buf,"missing") ||
+            !strcmp(buf,"MISSING") ||
+            !strcmp(buf,"Missing")  ) {
+            value->type=GRIB_TYPE_MISSING;
+            value->has_value=1;
+      } else {
+        value->string_value = grib_context_strdup(c,buf);
+        value->has_value=1;
+      }
+      break;
+    case GRIB_TYPE_UNDEFINED:
+      value->long_value = strtol(buf,&p,10);
+      if(*p == 0) {
+        value->type = GRIB_TYPE_LONG;
+        value->has_value=1;
+      } else {
+        value->double_value = strtod(buf,&p);
+        if(*p == 0) {
+            value->type = GRIB_TYPE_DOUBLE;
+            value->has_value=1;
+        } else if ( !strcmp(buf,"missing") ||
+                    !strcmp(buf,"MISSING") ||
+                    !strcmp(buf,"Missing")  ) {
+            value->type=GRIB_TYPE_MISSING;
+            value->has_value=1;
+        } else {
+          value->string_value = grib_context_strdup(c,buf);
+          value->type         = GRIB_TYPE_STRING;
+          value->has_value=1;
+        }
+      }
+      break;
+   }
+}
+
+/*
+ 'grib_tool'        Optional tool name which is printed on error. Can be NULL
+ 'arg'              The string to be parsed e.g. key1=value1,key2!=value2 etc
+ 'values_required'  If true then each key must have a value after it
+ 'default_type'     The default type e.g. GRIB_TYPE_UNDEFINED or GRIB_TYPE_DOUBLE
+ 'values'           The array we populate and return
+ 'count'            The number of elements
+*/
+int parse_keyval_string(char* grib_tool, char* arg, int values_required, int default_type, grib_values values[], int* count)
+{
+	char* p;
+	int i=0;
+	if (arg == NULL) {
+		*count=0;
+		return GRIB_SUCCESS;
+	}
+	p=strtok(arg,",");
+	while (p != NULL) {
+		values[i].name=(char*)calloc(1,strlen(p)+1);
+		strcpy((char*)values[i].name,p);
+		p=strtok(NULL,",");
+		i++;
+		if (i > *count) return GRIB_ARRAY_TOO_SMALL;
+	}
+	*count=i;
+
+	for (i=0;i<*count;i++) {
+		int equal=1;
+		char* value=NULL;
+		if (values_required) {
+			/* Can be either k=v or k!=v */
+			p=(char*)values[i].name;
+			while( *p != '=' && *p != '!' && *p != '\0') p++;
+			if (*p == '=') {
+				*p='\0';
+				p++;
+				value=p;
+				equal=1;
+			} else if (*p == '!' && *(++p) == '=' ) {
+				*p='\0';
+				*(p-1)='\0';
+				p++;
+				value=p;
+				equal=0;
+			} else {
+				return GRIB_INVALID_ARGUMENT;
+			}
+		}
+		p=(char*)values[i].name;
+		while(*p != ':' && *p != '\0') p++;
+		if ( *p == ':' ) {
+			values[i].type = grib_type_to_int(*(p+1));
+			if (*(p+1)=='n') values[i].type = GRIB_NAMESPACE;
+			*p='\0';
+			p++;
+		} else {
+			values[i].type = default_type;
+		}
+		if (values_required) {
+			if (strlen(value)==0) {
+				if (grib_tool)
+					printf("%s error: no value provided for key \"%s\"\n", grib_tool, values[i].name);
+				else
+					printf("Error: no value provided for key \"%s\"\n", values[i].name);
+				exit(GRIB_INVALID_ARGUMENT);
+			}
+			set_value(&values[i],value,equal);
+		}
+	}
+	return GRIB_SUCCESS;
+}
+
