@@ -1,18 +1,24 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
-
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 /***************************************************************************
  * Jean Baptiste Filippi - 01.11.2005
  * Enrico Fucile                                                           *
  ***************************************************************************/
 #include "grib_api_internal.h"
 
+GRIB_INLINE static int grib_inline_strcmp(const char* a,const char* b) {
+  if (*a != *b) return 1;
+    while((*a!=0 && *b!=0) &&  *(a) == *(b) ) {a++;b++;}
+	  return (*a==0 && *b==0) ? 0 : 1;
+}
+
 #define SWAP(a,b) temp=(a);(a)=(b);(b)=temp;
 
 int grib_set_expression(grib_handle* h, const char* name,grib_expression* e) {
@@ -55,6 +61,9 @@ int grib_set_long_internal(grib_handle* h, const char* name, long val) {
 
   a = grib_find_accessor(h, name);
 
+  if (h->context->debug==-1) 
+	  printf("GRIB_API DEBUG grib_set_long %s=%ld\n",name,(long)val);
+
   if(a){
     ret = grib_pack_long(a, &val, &l);
     if(ret == GRIB_SUCCESS){
@@ -78,7 +87,7 @@ int grib_set_long(grib_handle* h, const char* name, long val) {
   a = grib_find_accessor(h, name);
 
   if (h->context->debug==-1) 
-	  printf("GRIB_API DEBUG: grib_set_long %s=%ld\n",name,(long)val);
+	  printf("GRIB_API DEBUG grib_set_long %s=%ld\n",name,(long)val);
 
   if(a){
     if(a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY)
@@ -94,25 +103,27 @@ int grib_set_long(grib_handle* h, const char* name, long val) {
 }
 
 int grib_set_double_internal(grib_handle* h, const char* name, double val) {
-  grib_context* c=h->context;
   int ret = GRIB_SUCCESS;
   grib_accessor* a =NULL;
   size_t l = 1;
 
   a = grib_find_accessor(h, name);
 
+  if (h->context->debug==-1) 
+	  printf("GRIB_API DEBUG grib_set_double %s=%g\n",name,val);
+
   if(a){
     ret = grib_pack_double(a, &val, &l);
     if(ret == GRIB_SUCCESS){
       return grib_dependency_notify_change(a);
     }
 
-    grib_context_log(c,GRIB_LOG_ERROR,"unable to set %s=%g as double (%s)",
+    grib_context_log(h->context,GRIB_LOG_ERROR,"unable to set %s=%g as double (%s)",
                      name,val,grib_get_error_message(ret));
     return ret;
   }
 
-  grib_context_log(c,GRIB_LOG_ERROR,"unable to find accessor %s",name);
+  grib_context_log(h->context,GRIB_LOG_ERROR,"unable to find accessor %s",name);
   return GRIB_NOT_FOUND;
 }
 
@@ -180,7 +191,7 @@ int grib_copy_namespace(grib_handle* dest, const char* name, grib_handle* src) {
 				continue;
 			}
 
-			if (a->flags && GRIB_ACCESSOR_FLAG_READ_ONLY) {
+			if (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) {
 				key_err->err=GRIB_SUCCESS;
 				key_err=key_err->next;
 				continue;
@@ -289,7 +300,7 @@ int grib_set_double(grib_handle* h, const char* name, double val) {
   a = grib_find_accessor(h, name);
 
   if (h->context->debug==-1) 
-	  printf("GRIB_API DEBUG: grib_set_double %s=%g\n",name,val);
+	  printf("GRIB_API DEBUG grib_set_double %s=%g\n",name,val);
 
   if(a){
 
@@ -308,34 +319,49 @@ int grib_set_double(grib_handle* h, const char* name, double val) {
 int grib_set_string_internal(grib_handle* h, const char* name,
                              const char* val, size_t *length)
 {
-  grib_context* c=h->context;
   int ret = GRIB_SUCCESS;
   grib_accessor* a =NULL;
 
   a = grib_find_accessor(h, name);
 
+  if (h->context->debug==-1) 
+	  printf("GRIB_API DEBUG grib_set_string %s=%s\n",name,val);
+
   if(a){
     ret = grib_pack_string(a, val, length);
     if(ret == GRIB_SUCCESS){
       return grib_dependency_notify_change(a);
     }
 
-    grib_context_log(c,GRIB_LOG_ERROR,"unable to set %s=%s as string (%s)",
+    grib_context_log(h->context,GRIB_LOG_ERROR,"unable to set %s=%s as string (%s)",
                      name,val,grib_get_error_message(ret));
     return ret;
   }
 
-  grib_context_log(c,GRIB_LOG_ERROR,"unable to find accessor %s",name);
+  grib_context_log(h->context,GRIB_LOG_ERROR,"unable to find accessor %s",name);
   return GRIB_NOT_FOUND;
 }
 
 int grib_set_string(grib_handle* h, const char* name, const char* val, size_t *length)
 {
   int ret=0;
-  grib_accessor* a = grib_find_accessor(h, name);
+  grib_accessor* a;
+
+#if 1
+  /*second order doesn't have a proper representation for constant fields
+  the best is not to do the change of packing type if bitsPerValue=0
+  */
+  if (!grib_inline_strcmp(name,"packingType") && !grib_inline_strcmp(val,"grid_second_order")) {
+  	long bitsPerValue=0;
+	grib_get_long(h,"bitsPerValue",&bitsPerValue);
+	if (bitsPerValue==0) return 0;
+  }
+#endif
+
+  a = grib_find_accessor(h, name);
 
   if (h->context->debug==-1) 
-	  printf("GRIB_API DEBUG: grib_set_string %s=%s\n",name,val);
+	  printf("GRIB_API DEBUG grib_set_string %s=%s\n",name,val);
 
   if(a)
   {
@@ -353,7 +379,6 @@ int grib_set_string(grib_handle* h, const char* name, const char* val, size_t *l
 
 int grib_set_bytes_internal(grib_handle* h, const char* name, const unsigned char* val, size_t *length)
 {
-  grib_context* c=h->context;
   int ret = GRIB_SUCCESS;
   grib_accessor* a =NULL;
 
@@ -365,12 +390,12 @@ int grib_set_bytes_internal(grib_handle* h, const char* name, const unsigned cha
       return grib_dependency_notify_change(a);
     }
 
-    grib_context_log(c,GRIB_LOG_ERROR,"unable to set %s=%ld as bytes (%s)",
+    grib_context_log(h->context,GRIB_LOG_ERROR,"unable to set %s=%ld as bytes (%s)",
                      name,val,grib_get_error_message(ret));
     return ret;
   }
 
-  grib_context_log(c,GRIB_LOG_ERROR,"unable to find accessor %s",name);
+  grib_context_log(h->context,GRIB_LOG_ERROR,"unable to find accessor %s",name);
   return GRIB_NOT_FOUND;
 }
 
@@ -478,6 +503,13 @@ int grib_is_missing(grib_handle* h, const char* name,int* err)
   }
 }
 
+/* Return true if the given key exists (is defined) in our grib message */
+int grib_is_defined(grib_handle* h, const char* name)
+{
+	grib_accessor* a = grib_find_accessor(h, name);
+	return (a ? 1 : 0);
+}
+
 int grib_set_flag(grib_handle *h,const char* name,unsigned long flag) {
   grib_accessor* a=grib_find_accessor(h,name);
 
@@ -526,7 +558,9 @@ static int _grib_set_double_array(grib_handle* h, const char* name,
 {
   size_t encoded = 0;
   grib_accessor* a = grib_find_accessor(h, name);
-  int err = a ?_grib_set_double_array_internal(h,a,val,length,&encoded,check) : GRIB_NOT_FOUND ;
+  int err=0;
+
+  err = a ?_grib_set_double_array_internal(h,a,val,length,&encoded,check) : GRIB_NOT_FOUND ;
 
   if(err == GRIB_SUCCESS && length > encoded)
     err = GRIB_ARRAY_TOO_SMALL;
@@ -539,7 +573,17 @@ static int _grib_set_double_array(grib_handle* h, const char* name,
 
 int grib_set_double_array_internal(grib_handle* h, const char* name, const double* val, size_t length)
 {
-  int ret=_grib_set_double_array(h,name,val,length,0);
+  int ret=0;
+
+  if (h->context->debug==-1) 
+	  printf("GRIB_API DEBUG grib_set_double_array %ld values\n",(long)length);
+
+  if (length==0) {
+	  grib_accessor* a = grib_find_accessor(h, name);
+	  ret=grib_pack_double(a, val , &length);
+  } else {
+	  ret=_grib_set_double_array(h,name,val,length,0);
+  }
 
   if (ret!=GRIB_SUCCESS)
     grib_context_log(h->context,GRIB_LOG_ERROR,"unable to set double array %s (%s)",
@@ -549,6 +593,58 @@ int grib_set_double_array_internal(grib_handle* h, const char* name, const doubl
 
 int grib_set_double_array(grib_handle* h, const char* name, const double* val, size_t length)
 {
+  double v=val[0];
+  int constant,i;
+
+  if (h->context->debug==-1) 
+	  printf("GRIB_API DEBUG grib_set_double_array %ld values\n",(long)length);
+
+  if (length==0) {
+	  grib_accessor* a = grib_find_accessor(h, name);
+	  return grib_pack_double(a, val , &length);
+  }
+
+  /*second order doesn't have a proper representation for constant fields
+  the best is not to do the change of packing type if the field is constant
+  */
+  if (!strcmp(name,"values") || !strcmp(name,"codedValues")) {
+	  double missingValue;
+	  int ret=0;
+
+	  ret=grib_get_double(h,"missingValue",&missingValue);
+	  if (ret) missingValue=9999;
+
+	  v=missingValue;
+	  constant=1;
+	  for (i=0;i<length;i++) {
+		if (val[i]!=missingValue) {
+			if (v==missingValue) {
+				v=val[i];
+			} else if (v!=val[i]) {
+				constant=0;
+				break;
+			}
+		}
+	  }
+	  if (constant) {
+		char packingType[50]={0,};
+		size_t slen=50;
+
+		grib_get_string(h,"packingType",packingType,&slen);
+		if (!strcmp(packingType,"grid_second_order") || 
+			!strcmp(packingType,"grid_second_order_no_SPD") ||
+			!strcmp(packingType,"grid_second_order_SPD1") ||
+			!strcmp(packingType,"grid_second_order_SPD2") ||
+			!strcmp(packingType,"grid_second_order_SPD3") 
+			) {
+			slen=11; /*length of 'grid_simple' */
+			if (h->context->debug == -1) 
+				printf("GRIB_API DEBUG grib_set_double_array forcing grid_simple\n");
+			grib_set_string(h,"packingType","grid_simple",&slen);
+		}
+	  }
+  }
+
   return _grib_set_double_array(h,name,val,length,1);
 }
 
@@ -618,7 +714,6 @@ int grib_get_long_internal(grib_handle* h, const char* name, long* val)
     grib_context_log(h->context,GRIB_LOG_ERROR,
                      "unable to get %s as long (%s)",
                      name, grib_get_error_message(ret));
-	Assert(0);
   }
 
   return ret;
@@ -837,6 +932,23 @@ int grib_get_double_array(grib_handle* h, const char* name, double* val, size_t
 }
 
 
+int grib_get_string_length(grib_handle* h, const char* name,size_t* size)
+{
+  grib_accessor* a = grib_find_accessor(h, name);
+  size_t s=0;
+  if(!a) return GRIB_NOT_FOUND;
+
+  *size = 0;
+  while(a) {
+    s = grib_string_length(a);
+	if (s > *size) *size=s;
+    a = a->same;
+  }
+  (*size)+=1;
+
+  return GRIB_SUCCESS;
+}
+
 int grib_get_size(grib_handle* h, const char* name,size_t* size)
 {
   grib_accessor* a = grib_find_accessor(h, name);
@@ -849,6 +961,7 @@ int grib_get_size(grib_handle* h, const char* name,size_t* size)
   }
   return GRIB_SUCCESS;
 }
+
 int grib_get_count(grib_handle* h, const char* name,size_t* size)
 {
   grib_accessor* a = grib_find_accessor(h, name);
@@ -968,6 +1081,7 @@ static int grib_get_key_value(grib_handle* h,grib_key_value_list* kv) {
       kv->error=ret;
       break;
     case GRIB_TYPE_STRING:
+  	  grib_get_string_length(h,kv->name,&size);
       kv->string_value=grib_context_malloc_clear(h->context,size*sizeof(char));
       ret=grib_get_string(h,kv->name,kv->string_value,&size);
       kv->error=ret;
@@ -1083,7 +1197,7 @@ int grib_get_values(grib_handle* h,grib_values* args,size_t count) {
 
 int grib_set_values(grib_handle* h,grib_values* args,size_t count)
 {
-  int i;
+  int i, error=0;
   int err=0;
   size_t len;
   int more = 1;
@@ -1100,16 +1214,13 @@ int grib_set_values(grib_handle* h,grib_values* args,size_t count)
   while(more)
   {
     more = 0;
-
     for(i = 0; i < count ; i++)
     {
       if(args[i].error != GRIB_NOT_FOUND)
         continue;
 
-
       switch(args[i].type)
       {
-        int error=0;
         case GRIB_TYPE_LONG:
           error = grib_set_long(h,args[i].name,args[i].long_value);
           args[i].error=error;
