@@ -1,14 +1,18 @@
 #!/bin/sh
-# Copyright 2005-2007 ECMWF
-# 
-# Licensed under the GNU Lesser General Public License which
-# incorporates the terms and conditions of version 3 of the GNU
-# General Public License.
-# See LICENSE and gpl-3.0.txt for details.
+# Copyright 2005-2013 ECMWF.
+#
+# This software is licensed under the terms of the Apache Licence Version 2.0
+# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+#
+# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+#
 
 
 . ./include.sh
 
+REDIRECT=/dev/null
+
 cat > rules<<EOF
 edition=2;
 EOF
@@ -38,7 +42,7 @@ tmpdata=grib_api.grib
 
 rm -f $tmpdata || true
 
-${tools_dir}grib_convert ${data_dir}/mf.rules ${data_dir}/lfpw.grib1 $tmpdata 2> /dev/null > /dev/null
+${tools_dir}grib_convert ${data_dir}/mf.rules ${data_dir}/lfpw.grib1 $tmpdata 2> $REDIRECT > $REDIRECT
 ${tools_dir}grib_compare -f ${data_dir}/lfpw.grib2 $tmpdata
 
 rm -f $tmpdata || true
