@@ -1,14 +1,18 @@
 #!/bin/sh
-# Copyright 2005-2007 ECMWF
-# 
-# Licensed under the GNU Lesser General Public License which
-# incorporates the terms and conditions of version 3 of the GNU
-# General Public License.
-# See LICENSE and gpl-3.0.txt for details.
+# Copyright 2005-2013 ECMWF.
+#
+# This software is licensed under the terms of the Apache Licence Version 2.0
+# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+#
+# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+#
 
 
 . ./include.sh
 
+REDIRECT=/dev/null
+
 files="constant_field\
  reduced_gaussian_pressure_level_constant \
  reduced_latlon_surface_constant \
@@ -33,7 +37,7 @@ file=${data_dir}/$f
 
 rm -f ${file}.grib1_ || true
 
-${tools_dir}grib_set -s editionNumber=1 ${file}.grib2 ${file}.grib1_ 2> /dev/null > /dev/null
+${tools_dir}grib_set -s editionNumber=1 ${file}.grib2 ${file}.grib1_ 2> $REDIRECT > $REDIRECT
 
 
 grib1Statistics=`${tools_dir}grib_get -fp numberOfValues,numberOfPoints,max,min,average,numberOfMissing ${file}.grib1_` 
