@@ -1,10 +1,12 @@
 #!/bin/sh
-# Copyright 2005-2007 ECMWF
+# Copyright 2005-2013 ECMWF.
+#
+# This software is licensed under the terms of the Apache Licence Version 2.0
+# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+# 
+# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
 #
-# Licensed under the GNU Lesser General Public License which
-# incorporates the terms and conditions of version 3 of the GNU
-# General Public License.
-# See LICENSE and gpl-3.0.txt for details.
 
 . ./include.sh
 
@@ -37,19 +39,18 @@ ${tools_dir}grib_filter -o $outsimple w.filter $infile
 ${tools_dir}grib_filter r.filter $outsimple > $outsimple.txt 
 diff $outsimple.txt ${data_dir}/ieee_test.good
 
-GRIB_IEEE_PACKING=32
-export GRIB_IEEE_PACKING
-${tools_dir}grib_filter -o $out32 w.filter $infile 
+# Disabled for now. See GRIB-2 and GRIB-80
+#GRIB_IEEE_PACKING=32
+#export GRIB_IEEE_PACKING
+#${tools_dir}grib_filter -o $out32 w.filter $infile 
+#${tools_dir}grib_filter r.filter $out32 > $out32.txt 
+#diff $out32.txt ${data_dir}/ieee_test.good
 
-${tools_dir}grib_filter r.filter $out32 > $out32.txt 
-diff $out32.txt ${data_dir}/ieee_test.good
-
-GRIB_IEEE_PACKING=64
-export GRIB_IEEE_PACKING
-${tools_dir}grib_filter -o $out64 w.filter $infile 
-
-${tools_dir}grib_filter r.filter $out64 > $out64.txt 
-diff $out64.txt ${data_dir}/ieee_test.good
+#GRIB_IEEE_PACKING=64
+#export GRIB_IEEE_PACKING
+#${tools_dir}grib_filter -o $out64 w.filter $infile 
+#${tools_dir}grib_filter r.filter $out64 > $out64.txt 
+#diff $out64.txt ${data_dir}/ieee_test.good
 
 rm -f $outsimple $out32 $out64 $out32.txt $out64.txt
 rm -f ${data_dir}/$outsimple.txt ${data_dir}/$out32.txt ${data_dir}/$out64.txt 
@@ -60,15 +61,15 @@ diff $shdata.txt $shdata.good
 
 GRIB_IEEE_PACKING=32
 export GRIB_IEEE_PACKING
-${tools_dir}grib_copy -r $shdata ${shdata}_32
-${tools_dir}grib_filter r.filter ${shdata}_32 > ${shdata}_32.txt
-diff ${shdata}_32.txt ${shdata}_32.good
+#${tools_dir}grib_copy -r $shdata ${shdata}_32
+#${tools_dir}grib_filter r.filter ${shdata}_32 > ${shdata}_32.txt
+#diff ${shdata}_32.txt ${shdata}_32.good
 
 GRIB_IEEE_PACKING=64
 export GRIB_IEEE_PACKING
-${tools_dir}grib_copy -r $shdata ${shdata}_64
-${tools_dir}grib_filter r.filter ${shdata}_64 > ${shdata}_64.txt
-diff ${shdata}_64.txt ${shdata}.good
+#${tools_dir}grib_copy -r $shdata ${shdata}_64
+#${tools_dir}grib_filter r.filter ${shdata}_64 > ${shdata}_64.txt
+#diff ${shdata}_64.txt ${shdata}.good
 
 rm -f r.filter ${shdata}_64.txt ${shdata}_32.txt 
 
