@@ -1,14 +1,17 @@
 #!/bin/sh
-# Copyright 2005-2007 ECMWF
-# 
-# Licensed under the GNU Lesser General Public License which
-# incorporates the terms and conditions of version 3 of the GNU
-# General Public License.
-# See LICENSE and gpl-3.0.txt for details.
+# Copyright 2005-2013 ECMWF.
+#
+# This software is licensed under the terms of the Apache Licence Version 2.0
+# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+#
+# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+#
 
 
 . ./include.sh
 
+REDIRECT=/dev/null
 
 infile="${data_dir}/missing.grib2"
 outfile="${data_dir}/missing_new.grib2"
@@ -29,7 +32,7 @@ scaledValueOfSecondFixedSurface=`${tools_dir}grib_get -w count=1 -p scaledValueO
 
 set +e
 
-${tools_dir}grib_set -s centre=missing $infile $outfile 2>/dev/null > /dev/null
+${tools_dir}grib_set -s centre=missing $infile $outfile 2> $REDIRECT > $REDIRECT
 
 [ $? -ne 0 ]
 
