@@ -1,20 +1,24 @@
 #!/bin/sh
-# Copyright 2005-2007 ECMWF
-# 
-# Licensed under the GNU Lesser General Public License which
-# incorporates the terms and conditions of version 3 of the GNU
-# General Public License.
-# See LICENSE and gpl-3.0.txt for details.
+# Copyright 2005-2013 ECMWF.
+#
+# This software is licensed under the terms of the Apache Licence Version 2.0
+# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+#
+# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+#
 
 
 . ./include.sh
 
+REDIRECT=/dev/null
+
 infile=${data_dir}/regular_gaussian_surface.grib1
 outfile=${data_dir}/set.grib1
 
 rm -f $outfile || true
 
-${tools_dir}grib_set -v -p levtype,centre,levtype:l,centre:l -s levtype=pl,centre=80 $infile $outfile >/dev/null
+${tools_dir}grib_set -v -p levtype,centre,levtype:l,centre:l -s levtype=pl,centre=80 $infile $outfile >$REDIRECT
 
 levtype=`${tools_dir}grib_get -p levtype $outfile`
 [ $levtype = "pl" ]
@@ -33,7 +37,7 @@ outfile=${data_dir}/set.grib2
 
 rm -f $outfile || true
 
-${tools_dir}grib_set -v -p levtype:l,centre:s  -s levtype:l=100,centre:s=cnmc $infile $outfile >/dev/null
+${tools_dir}grib_set -v -p levtype:l,centre:s  -s levtype:l=100,centre:s=cnmc $infile $outfile >$REDIRECT
 
 levtype=`${tools_dir}grib_get -p levtype $outfile`
 [ $levtype = "pl" ]
@@ -49,7 +53,7 @@ centre=`${tools_dir}grib_get -p centre:l $outfile`
 
 set +e
 
-${tools_dir}grib_set -p levtype $infile $outfile 2>/dev/null >/dev/null
+${tools_dir}grib_set -p levtype $infile $outfile 2> $REDIRECT > $REDIRECT
 
 [ $? -ne 0 ]
 
