@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-* 
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "config.h"
 
@@ -60,9 +61,7 @@ static void potential_vorticity_level(grib_handle*,const parameter*,double,doubl
 TODO: 
 - Shape of the earth
 - Levels
-
- */
-
+*/
 
 #include "tigge_check.h"
 
@@ -100,7 +99,6 @@ void warn(const char* name,int a)
 	}
 }
 
-
 void save(grib_handle* h, const char *name,FILE* f)
 {
 	size_t size;
@@ -122,7 +120,6 @@ void save(grib_handle* h, const char *name,FILE* f)
 	}
 }
 
-
 long get(grib_handle *h,const char* what)
 {
 	int e; long val;
@@ -149,7 +146,7 @@ double dget(grib_handle *h,const char* what)
 
 int missing(grib_handle *h,const char* what)
 {
-  int err=0;
+	int err=0;
 	return grib_is_missing(h,what,&err);
 }
 
@@ -173,7 +170,6 @@ int le(grib_handle *h,const char* what,long value)
 	return get(h,what) <= value;
 }
 
-
 void gaussian_grid(grib_handle* h)
 {
 	long n = get(h,"numberOfParallelsBetweenAPoleAndTheEquator");
@@ -224,10 +220,10 @@ void gaussian_grid(grib_handle* h)
 	{
 		double  ee = 360.0 - 360.0/(4.0*n);
 		CHECK(west == 0.0);
-		
+
 		if(ee != east)
 			printf("east %g %g %g\n",east,ee,ee-east);
-			
+
 		CHECK(east == ee); 
 		CHECK(fabs(east-ee) < 1e-10);
 	}
@@ -240,7 +236,7 @@ void gaussian_grid(grib_handle* h)
 		double dwest  = dget(h,"longitudeOfFirstGridPointInDegrees");
 		double deast  = dget(h,"longitudeOfLastGridPointInDegrees");
 		double dwe = dget(h,"iDirectionIncrementInDegrees");
-/*printf("parallel=%ld east=%ld west=%ld we=%ld\n",parallel,east,west,we);*/
+		/*printf("parallel=%ld east=%ld west=%ld we=%ld\n",parallel,east,west,we);*/
 
 		CHECK(parallel == (east-west)/we + 1);
 		CHECK(fabs((deast-dwest)/dwe + 1 - parallel) < 1e-10);
@@ -314,7 +310,6 @@ void gaussian_grid(grib_handle* h)
 	CHECK(eq(h,"resolutionAndComponentFlags7",0));
 	CHECK(eq(h,"resolutionAndComponentFlags8",0));
 
-
 }
 
 static void check_range(grib_handle* h,const parameter* p,double min,double max)
@@ -348,29 +343,29 @@ static void point_in_time(grib_handle* h,const parameter* p,double min,double ma
 {
 	switch(get(h,"typeOfProcessedData"))
 	{
-		case 2: /* Analysis and forecast products */
-			CHECK(eq(h,"productDefinitionTemplateNumber",0));
-			break;
+	case 2: /* Analysis and forecast products */
+		CHECK(eq(h,"productDefinitionTemplateNumber",0));
+		break;
 
-		case 3: /* Control forecast products */
-			CHECK(eq(h,"productDefinitionTemplateNumber",1));
-			CHECK(eq(h,"perturbationNumber",0));
-			CHECK(ne(h,"numberOfForecastsInEnsemble",0));
-			break;
+	case 3: /* Control forecast products */
+		CHECK(eq(h,"productDefinitionTemplateNumber",1));
+		CHECK(eq(h,"perturbationNumber",0));
+		CHECK(ne(h,"numberOfForecastsInEnsemble",0));
+		break;
 
-		case 4: /* Perturbed forecast products */
-			CHECK(eq(h,"productDefinitionTemplateNumber",1));
-			if (is_lam) {
-				CHECK(le(h,"perturbationNumber",get(h,"numberOfForecastsInEnsemble")));
-			} else { 
-				CHECK(le(h,"perturbationNumber",get(h,"numberOfForecastsInEnsemble")-1));
-			}
-			break;
+	case 4: /* Perturbed forecast products */
+		CHECK(eq(h,"productDefinitionTemplateNumber",1));
+		if (is_lam) {
+			CHECK(le(h,"perturbationNumber",get(h,"numberOfForecastsInEnsemble")));
+		} else {
+			CHECK(le(h,"perturbationNumber",get(h,"numberOfForecastsInEnsemble")-1));
+		}
+		break;
 
-		default:
-			printf("Unsupported typeOfProcessedData %ld\n",get(h,"typeOfProcessedData"));
-			CHECK(0);
-			break;
+	default:
+		printf("Unsupported typeOfProcessedData %ld\n",get(h,"typeOfProcessedData"));
+		CHECK(0);
+		break;
 	}
 
 	if(get(h,"indicatorOfUnitOfTimeRange") == 11 || is_lam ) /*  six hours */
@@ -384,7 +379,6 @@ static void point_in_time(grib_handle* h,const parameter* p,double min,double ma
 		CHECK((get(h,"forecastTime") % 6) == 0);  /* Every six hours */
 	}
 
-
 	check_range(h,p,min,max);
 
 }
@@ -395,20 +389,20 @@ static void pressure_level(grib_handle* h,const parameter* p,double min,double m
 
 	switch(level)
 	{
-		case 1000:
-		case  200:
-		case  250:
-		case  300:
-		case  500:
-		case  700:
-		case  850:
-		case  925:
-		case  50:
-			break;
-		default:
-			printf("%s, field %d [%s]: invalid pressure level %ld\n",file,field,param,level);
-			error++;
-			break;
+	case 1000:
+	case  200:
+	case  250:
+	case  300:
+	case  500:
+	case  700:
+	case  850:
+	case  925:
+	case  50:
+		break;
+	default:
+		printf("%s, field %d [%s]: invalid pressure level %ld\n",file,field,param,level);
+		error++;
+		break;
 	}
 }
 
@@ -418,12 +412,12 @@ static void potential_vorticity_level(grib_handle* h,const parameter* p,double m
 
 	switch(level)
 	{
-		case 2:
-			break;
-		default:
-			printf("%s, field %d [%s]: invalid potential vorticity level %ld\n",file,field,param,level);
-			error++;
-			break;
+	case 2:
+		break;
+	default:
+		printf("%s, field %d [%s]: invalid potential vorticity level %ld\n",file,field,param,level);
+		error++;
+		break;
 	}
 
 }
@@ -434,54 +428,68 @@ static void potential_temperature_level(grib_handle* h,const parameter* p,double
 
 	switch(level)
 	{
-		case 320:
-			break;
-		default:
-			printf("%s, field %d [%s]: invalid potential temperature level %ld\n",file,field,param,level);
-			error++;
-			break;
+	case 320:
+		break;
+	default:
+		printf("%s, field %d [%s]: invalid potential temperature level %ld\n",file,field,param,level);
+		error++;
+		break;
 	}
 }
 
-
-
 static void statistical_process(grib_handle* h,const parameter* p,double min,double max)
 {
 	switch(get(h,"typeOfProcessedData"))
 	{
-		case 2: /* Analysis and forecast products */
-			CHECK(eq(h,"productDefinitionTemplateNumber",8));
-			break;
+	case 2: /* Analysis and forecast products */
+		CHECK(eq(h,"productDefinitionTemplateNumber",8));
+		break;
 
-		case 3: /* Control forecast products */
-			CHECK(eq(h,"productDefinitionTemplateNumber",11));
-			break;
+	case 3: /* Control forecast products */
+		CHECK(eq(h,"productDefinitionTemplateNumber",11));
+		break;
 
-		case 4: /* Perturbed forecast products */
-			CHECK(eq(h,"productDefinitionTemplateNumber",11));
-			break;
+	case 4: /* Perturbed forecast products */
+		CHECK(eq(h,"productDefinitionTemplateNumber",11));
+		break;
 
-		default:
-			printf("Unsupported typeOfProcessedData %ld\n",get(h,"typeOfProcessedData"));
-			error++;
-			return;
-			break;
+	default:
+		printf("Unsupported typeOfProcessedData %ld\n",get(h,"typeOfProcessedData"));
+		error++;
+		return;
+		break;
 	}
 
-	if(get(h,"indicatorOfUnitOfTimeRange") == 11 || is_lam ) /*  six hours */
+	if (!is_lam)
 	{
-		/* Six hourly is OK */
-		;
+		if(get(h,"indicatorOfUnitOfTimeRange") == 11) /*  six hours */
+		{
+			/* Six hourly is OK */
+			;
+		}
+		else
+		{
+			CHECK(eq(h,"indicatorOfUnitOfTimeRange",1));/* Hours */
+			CHECK((get(h,"forecastTime") % 6) == 0);  /* Every six hours */
+		}
 	}
 	else
 	{
-		CHECK(eq(h,"indicatorOfUnitOfTimeRange",1));/* Hours */
-		CHECK((get(h,"forecastTime") % 6) == 0);  /* Every six hours */
+		if(get(h,"indicatorOfUnitOfTimeRange") == 10 ) /*  three hours */
+		{
+			/* Three hourly is OK */
+			;
+		}
+		else
+		{
+			CHECK(eq(h,"indicatorOfUnitOfTimeRange",1));/* Hours */
+			CHECK((get(h,"forecastTime") % 3) == 0);  /* Every three hours */
+		}
 	}
 
-	CHECK(eq(h,"numberOfTimeRangeSpecificationsDescribingTheTimeIntervalsUsedToCalculateTheStatisticallyProcessedField",1));
-	CHECK(eq(h,"totalNumberOfDataValuesMissingInStatisticalProcess",0));
-	CHECK(eq(h,"typeOfTimeIncrementBetweenSuccessiveFieldsUsedInTheStatisticalProcessing",2));
+	CHECK(eq(h,"numberOfTimeRange",1));
+	CHECK(eq(h,"numberOfMissingInStatisticalProcess",0));
+	CHECK(eq(h,"typeOfTimeIncrement",2));
 	/*CHECK(eq(h,"indicatorOfUnitOfTimeForTheIncrementBetweenTheSuccessiveFieldsUsed",255));*/
 
 	CHECK(eq(h,"timeIncrementBetweenSuccessiveFields",0));
@@ -490,22 +498,33 @@ static void statistical_process(grib_handle* h,const parameter* p,double min,dou
 		CHECK(eq(h,"minuteOfEndOfOverallTimeInterval",0));
 		CHECK(eq(h,"secondOfEndOfOverallTimeInterval",0));
 
-		if (!is_lam) CHECK((get(h,"endStep") % 6) == 0);  /* Every six hours */
+		if (!is_lam)
+		{
+			CHECK((get(h,"endStep") % 6) == 0); /* Every six hours */
+		}
+		else
+		{
+			CHECK((get(h,"endStep") % 3) == 0);  /* Every three hours */
+		}
 		CHECK(get(h,"endStep") <= 24*30);  
 	}
 
 
-	if(get(h,"indicatorOfUnitOfTimeForTimeRangeOverWhichStatisticalProcessingIsDone") == 11)
+	if(get(h,"indicatorOfUnitForTimeRange") == 11)
 	{
 		/* Six hourly is OK */
-		CHECK(get(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone")*6 + get(h,"startStep") == get(h,"endStep"));
+		CHECK(get(h,"lengthOfTimeRange")*6 + get(h,"startStep") == get(h,"endStep"));
+	}
+	else if(get(h,"indicatorOfUnitForTimeRange") == 10)
+	{
+		/* Three hourly is OK */
+		CHECK(get(h,"lengthOfTimeRange")*3 + get(h,"startStep") == get(h,"endStep"));
 	}
 	else
 	{
-		CHECK(eq(h,"indicatorOfUnitOfTimeForTimeRangeOverWhichStatisticalProcessingIsDone",1)); /* Hours */
-		CHECK(get(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone") + get(h,"startStep") == get(h,"endStep"));
+		CHECK(eq(h,"indicatorOfUnitForTimeRange",1)); /* Hours */
+		CHECK(get(h,"lengthOfTimeRange") + get(h,"startStep") == get(h,"endStep"));
 	}
-
 }
 
 static void has_bitmap(grib_handle* h,const parameter* p,double min,double max)
@@ -518,10 +537,10 @@ static void six_hourly(grib_handle* h,const parameter* p,double min,double max)
 {
 	statistical_process(h,p,min,max);
 
-	if(get(h,"indicatorOfUnitOfTimeForTimeRangeOverWhichStatisticalProcessingIsDone") == 11)
-		CHECK(eq(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone",1));
+	if(get(h,"indicatorOfUnitForTimeRange") == 11)
+		CHECK(eq(h,"lengthOfTimeRange",1));
 	else
-		CHECK(eq(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone",6));
+		CHECK(eq(h,"lengthOfTimeRange",6));
 
 	CHECK(get(h,"endStep") == get(h,"startStep") + 6);
 	check_range(h,p,min,max);
@@ -531,10 +550,10 @@ static void three_hourly(grib_handle* h,const parameter* p,double min,double max
 {
 	statistical_process(h,p,min,max);
 
-	if(get(h,"indicatorOfUnitOfTimeForTimeRangeOverWhichStatisticalProcessingIsDone") == 11)
-		CHECK(eq(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone",1));
+	if(get(h,"indicatorOfUnitForTimeRange") == 11)
+		CHECK(eq(h,"lengthOfTimeRange",1));
 	else
-		CHECK(eq(h,"lengthOfTheTimeRangeOverWhichStatisticalProcessingIsDone",3));
+		CHECK(eq(h,"lengthOfTimeRange",3));
 
 	CHECK(get(h,"endStep") == get(h,"startStep") + 3);
 	check_range(h,p,min,max);
@@ -569,7 +588,7 @@ static void predefined_level(grib_handle* h,const parameter* p,double min,double
 {
 	CHECK(ne(h,"typeOfFirstFixedSurface",255));
 	CHECK(missing(h,"scaleFactorOfFirstFixedSurface"));
-	CHECK(missing(h,"scaledValueOfFirstFixedSurface"));
+ 	CHECK(missing(h,"scaledValueOfFirstFixedSurface"));
 
 	CHECK(eq(h,"typeOfSecondFixedSurface",255));
 	CHECK(missing(h,"scaleFactorOfSecondFixedSurface"));
@@ -598,7 +617,6 @@ static void given_thickness(grib_handle* h,const parameter* p,double min,double
 	CHECK(!missing(h,"scaleFactorOfSecondFixedSurface"));
 }
 
-
 void latlon_grid(grib_handle* h)
 {
 	long data_points = get(h,"numberOfDataPoints");
@@ -639,8 +657,6 @@ void latlon_grid(grib_handle* h)
 		CHECK(eq(h,"basicAngleOfTheInitialProductionDomain",0));
 	}
 
-
-
 	CHECK(meridian*parallel == data_points);
 
 	CHECK(eq(h,"resolutionAndComponentFlags1",0));
@@ -652,8 +668,8 @@ void latlon_grid(grib_handle* h)
 	CHECK(eq(h,"iDirectionIncrementGiven",1));
 	CHECK(eq(h,"jDirectionIncrementGiven",1));
 
-	CHECK(eq(h,"numberOfOctetsForOptionalListOfNumbersDefiningNumberOfPoints",0));
-	CHECK(eq(h,"interpretationOfListOfNumbersDefiningNumberOfPoints",0));
+	CHECK(eq(h,"numberOfOctectsForNumberOfPoints",0));
+	CHECK(eq(h,"interpretationOfNumberOfPoints",0));
 
 	if(get(h,"iScansNegatively"))
 	{
@@ -715,9 +731,6 @@ void latlon_grid(grib_handle* h)
 
 #define X(x) printf("%s=%ld ",#x,get(h,#x))
 
-/* 
- */
-
 void check_parameter(grib_handle* h,double min,double max)
 {
 	int i;
@@ -770,8 +783,6 @@ void check_parameter(grib_handle* h,double min,double max)
 		error++;
 	}
 }
-/* 
- */
 
 void verify(grib_handle* h)
 {
@@ -804,12 +815,16 @@ void verify(grib_handle* h)
 
 	CHECK(eq(h,"productionStatusOfProcessedData",4)||eq(h,"productionStatusOfProcessedData",5)); /*  TIGGE Operational */
 
-
-	if (!is_lam) CHECK((get(h,"step") % 6) == 0);
+	if (!is_lam){
+		CHECK((get(h,"step") % 6) == 0);
+	}
+	else
+	{
+		CHECK((get(h,"step") % 3) == 0);
+	}
 	CHECK(ge(h,"startStep",0));
 	CHECK(le(h,"endStep",30*24));
 
-
 	/* 2 = analysis or forecast , 3 = control forecast, 4 = perturbed forecast */
 	CHECK(eq(h,"typeOfProcessedData",2)||eq(h,"typeOfProcessedData",3)||eq(h,"typeOfProcessedData",4)); 
 
@@ -820,26 +835,25 @@ void verify(grib_handle* h)
 
 	CHECK(eq(h,"sourceOfGridDefinition",0)); /* Specified in Code table 3.1 */
 
-
 	switch(get(h,"gridDefinitionTemplateNumber"))
 	{
-		case 0:
-			latlon_grid(h);
-			break;
+	case 0:
+	case 1: /*rotated latlon*/
+		latlon_grid(h);
+		break;
 
-		case 40:
-			gaussian_grid(h);
-			break;
+	case 40:
+		gaussian_grid(h);
+		break;
 
 
-		default:
-			printf("%s, field %d [%s]: Unsupported gridDefinitionTemplateNumber %ld\n",
-					file,field,param,
-					get(h,"gridDefinitionTemplateNumber"));
-			error++;
-			return;
-			break;
-
+	default:
+		printf("%s, field %d [%s]: Unsupported gridDefinitionTemplateNumber %ld\n",
+				file,field,param,
+				get(h,"gridDefinitionTemplateNumber"));
+		error++;
+		return;
+		break;
 	}
 
 	/* If there is no bitmap, this should be true */
@@ -848,12 +862,12 @@ void verify(grib_handle* h)
 	if(eq(h,"bitMapIndicator",255))
 		CHECK(get(h,"numberOfValues") == get(h,"numberOfDataPoints"));
 	else
-		CHECK(get(h,"numberOfValues") < get(h,"numberOfDataPoints"));
+		CHECK(get(h,"numberOfValues") <= get(h,"numberOfDataPoints"));
 
 	/* Check values */
 	CHECK(eq(h,"typeOfOriginalFieldValues",0)); /* Floating point */
 
-	if(valueflg)
+	if (valueflg)
 	{
 
 		size_t count, n;
@@ -885,7 +899,6 @@ void verify(grib_handle* h)
 			free(values);
 			error++;
 			return;
-
 		}
 
 		if(n != count)
@@ -907,7 +920,7 @@ void verify(grib_handle* h)
 				if((min == missing) || ((values[i] != missing) && (min>values[i]))) min = values[i];
 				if((max == missing) || ((values[i] != missing) && (max<values[i]))) max = values[i];
 			}	
-			
+
 		}
 		else
 		{
@@ -918,19 +931,12 @@ void verify(grib_handle* h)
 				if(max<values[i]) max = values[i];
 			}
 		}
-
-
 		free(values);
-
-
 	}
-		check_parameter(h,min,max);
+	check_parameter(h,min,max);
 
 }
 
-/* 
- */
-
 void validate(const char* path)
 {
 	FILE *f = fopen(path,"r");
@@ -941,7 +947,6 @@ void validate(const char* path)
 	file  = path;
 	field = 0;
 
-
 	if(!f) {
 		printf("%s: %s\n",path,strerror(errno));
 		error++;
@@ -1025,39 +1030,39 @@ int main(int argc,const char** argv)
 		{
 			switch(argv[i][1])
 			{
-				case 'w':
-					warnflg++;
-					break;
+			case 'w':
+				warnflg++;
+				break;
 
-				case 'z':
-					zeroflg++;
-					break;
+			case 'z':
+				zeroflg++;
+				break;
 
-				case 'v':
-					valueflg++;
-					break;
+			case 'v':
+				valueflg++;
+				break;
 
-				case 'g':
-					if(++i == argc) usage();
-					good = argv[i];
-					fgood = fopen(good,"w");
-					if(!fgood) { perror(good); exit(1); }
-					break;
+			case 'g':
+				if(++i == argc) usage();
+				good = argv[i];
+				fgood = fopen(good,"w");
+				if(!fgood) { perror(good); exit(1); }
+				break;
 
-				case 'b':
-					if(++i == argc) usage();
-					bad = argv[i];
-					fbad = fopen(bad,"w");
-					if(!fbad) { perror(bad); exit(1); } 
-					break;
+			case 'b':
+				if(++i == argc) usage();
+				bad = argv[i];
+				fbad = fopen(bad,"w");
+				if(!fbad) { perror(bad); exit(1); }
+				break;
 
-				case 'l':
-				    is_lam=1;
-					break;
+			case 'l':
+				is_lam=1;
+				break;
 
-				default:
-					usage();
-					break;
+			default:
+				usage();
+				break;
 			}
 		}		
 		else
