@@ -1,18 +1,16 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /*
  * C Implementation: grib_convert
  *
- * Author: Enrico Fucile <enrico.fucile@ecmwf.int>
- *
- *
  */
 
 #include "grib_tools.h"
@@ -28,6 +26,7 @@ grib_option grib_options[]={
   {"I",0,0,1,0,0},
   {"g",0,0,0,1,0},
   {"G",0,0,0,1,0},
+  {"T",0,0,0,1,0},
   {"V",0,0,0,1,0},
   {"7",0,0,0,1,0},
   {"v",0,0,0,1,0}
@@ -69,7 +68,7 @@ int grib_tool_init(grib_runtime_options* options) {
       while(e)  {
         Assert(set < (sizeof(options->set_values)/sizeof(options->set_values[0])));
         options->set_values[set].name = e->name;
-        GRIB_CHECK(grib_expression_set_value(NULL,e->value,&(options->set_values[set])),NULL);
+        GRIB_CHECK_NOLINE(grib_expression_set_value(NULL,e->value,&(options->set_values[set])),NULL);
         set++;
         e = e->next;
       }
@@ -98,7 +97,11 @@ int grib_tool_init(grib_runtime_options* options) {
   return 0;
 }
 
-int grib_tool_new_file_action(grib_runtime_options* options) {
+int grib_tool_new_filename_action(grib_runtime_options* options,const char* file) {
+   return 0;
+}
+
+int grib_tool_new_file_action(grib_runtime_options* options,grib_tools_file* file) {
   return 0;
 }
 
@@ -114,7 +117,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h) {
   while(r) {
     if(r->condition)  {
       long ok = 0;
-      GRIB_CHECK(grib_expression_evaluate_long(h,r->condition,&ok),NULL);
+      GRIB_CHECK_NOLINE(grib_expression_evaluate_long(h,r->condition,&ok),NULL);
       if(ok) {
         grib_rule_entry* e = r->entries;
         while(e) {
@@ -143,11 +146,12 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h) {
 
   if( err != GRIB_SUCCESS) {
       for(i = 0; i < options->set_values_count ; i++) {
-        if(options->set_values[i].error != GRIB_SUCCESS &&
-           options->set_values[i].error != GRIB_CONCEPT_NO_MATCH && options->fail)
+        if(options->set_values[i].error != GRIB_SUCCESS && options->fail)
            grib_context_log(h->context,GRIB_LOG_ERROR,"unable to set key \"%s\" (%s)",
               options->set_values[i].name,grib_get_error_message( options->set_values[i].error));
       }
+	  grib_context_log(h->context,GRIB_LOG_ERROR,"error processing message %d\n",options->infile->handle_count);
+	  if (options->fail) exit(err);
   }
 
   grib_get_double(h,"scaleValuesBy",&scale);
@@ -159,7 +163,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h) {
     size_t size;
     int i;
 
-    GRIB_CHECK(grib_get_size(h,"values",&size),0);
+    GRIB_CHECK_NOLINE(grib_get_size(h,"values",&size),0);
 
     v    = (double*)calloc(size,sizeof(double));
     if(!v) {
@@ -167,7 +171,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h) {
       exit(1);
     }
 
-    GRIB_CHECK(grib_get_double_array(h,"values",v,&size),0);
+    GRIB_CHECK_NOLINE(grib_get_double_array(h,"values",v,&size),0);
 
     if(options->verbose)
     {
@@ -177,7 +181,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h) {
     for(i = 0; i< size; i++)
       v[i] =  v[i] * scale + offset;
 
-    GRIB_CHECK(grib_set_double_array(h,"values",v,size),0);
+    GRIB_CHECK_NOLINE(grib_set_double_array(h,"values",v,size),0);
     free(v);
   }
 
