@@ -1,23 +1,21 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*
  * C Implementation: grib_copy
  *
- * Author: Enrico Fucile <enrico.fucile@ecmwf.int>
- *
- *
  */
 
 #include "grib_tools.h"
 
-char* grib_tool_description="Build index from a set of files";
+char* grib_tool_description="Build an index file for a set of input grib files.";
 char* grib_tool_name="grib_index_build";
 char* grib_tool_usage="[options] grib_file grib_file ... ";
 grib_index* idx=NULL;
@@ -30,9 +28,13 @@ grib_option grib_options[]={
 	{"o:",0,0,1,1,"gribidx"},
     {"k:",0,0,0,1,0},
     {"V",0,0,0,1,0},
-    {"M",0,0,0,1,0}
+    {"T:",0,0,0,1,0},
+    {"M",0,0,0,1,0},
+    {"N",0,0,0,1,0}
 };
 
+int compress_index;
+
 int grib_options_count=sizeof(grib_options)/sizeof(grib_option);
 
 int main(int argc, char *argv[]) {
@@ -48,6 +50,10 @@ int grib_tool_init(grib_runtime_options* options) {
 
   int ret=0;
   grib_context* c=grib_context_get_default();
+
+  if (grib_options_on("N")) compress_index=0;
+  else compress_index=1;
+
   if (grib_options_on("k:")) 
       keys=grib_options_get_option("k:");
   else
@@ -91,7 +97,11 @@ int grib_tool_finalise_action(grib_runtime_options* options) {
 	grib_index_key* keys;
 	grib_string_list* values;
 	int first;
-	grib_index_compress(idx);
+
+
+	if (compress_index) {
+		grib_index_compress(idx);
+	}
 	printf("--- %s: keys included in the index file %s:\n",
 			grib_tool_name,options->outfile->name);
 	printf("--- "); 
