@@ -1,18 +1,16 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*
  * C Implementation: grib_ls
  *
- * Author: Enrico Fucile <enrico.fucile@ecmwf.int>
- *
- *
  */
 #include "grib_tools.h"
 
@@ -23,7 +21,7 @@ grib_option grib_options[]={
     {"F:",0,0,1,1,"%g"},
     {"P:",0,0,0,1,0},
     {"w:",0,0,0,1,0},
-	{"j",0,"json output",0,1,0},
+	{"j",0,"json output\n",0,1,0},
     {"B:",0,0,0,1,0},
     {"l:",0,0,0,1,0},
 	{"s:",0,0,0,1,0},
@@ -37,7 +35,7 @@ grib_option grib_options[]={
     {"H",0,0,1,0,0},
     {"g",0,0,0,1,0},
     {"P",0,0,1,0,0},
-    {"G",0,0,0,1,0},
+    {"T:",0,0,0,1,0},
     {"7",0,0,0,1,0},
     {"v",0,0,1,0,0},
     {"x",0,0,0,1,0}
@@ -65,7 +63,7 @@ int grib_tool_before_getopt(grib_runtime_options* options) {
 }
 
 int grib_tool_init(grib_runtime_options* options) {
-  char  *end = NULL, *end1=NULL;
+  char  *theEnd = NULL, *end1=NULL;
   size_t size=4;
   int ret=0;
   double min=0,max=0;
@@ -78,12 +76,12 @@ int grib_tool_init(grib_runtime_options* options) {
   
   if (options->latlon) {
 	  
-    lat = strtod(options->latlon,&end);
-    if (*end != ',') {
+    lat = strtod(options->latlon,&theEnd);
+    if (*theEnd != ',') {
       printf("ERROR: wrong latitude value\n");
       exit(1);
     }
-    lon= strtod(++end,&end1);
+    lon= strtod(++theEnd,&end1);
     
     mode=GRIB_NEAREST_SAME_POINT | GRIB_NEAREST_SAME_GRID;
 
