@@ -1,21 +1,17 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*
  * C Implementation: grib_options
  *
- * Author: Enrico Fucile <enrico.fucile@ecmwf.int>
- *
- *
  */
-#include <unistd.h>
-#include <string.h>
 #include "grib_tools.h"
 
 char* names[]={"parameter","vertical","geography","data","mars","local"};
@@ -28,7 +24,7 @@ grib_options_help grib_options_help_list[] ={
      "\n\t\tAll the keys in this list are skipped in the comparison. Bit-by-bit compare on.\n"},
   {"B:","order by directive",
      "\n\t\tOrder by. The output will be ordered according the order by directive."
-     "\n\t\tOrder by example: \"step asc, centre desc\" (step ascending and centre discending)\n"},
+     "\n\t\tOrder by example: \"step asc, centre desc\" (step ascending and centre descending)\n"},
   {"c:","key[:l/d/s/n],key[:l/d/s/n],...",
    "\n\t\tOnly the listed keys or namespaces (:n) are compared. The optional letter after the colon is used "
    "\n\t\tto force the type in the comparison: l->integer, d->float, s->string, n->namespace."
@@ -55,7 +51,7 @@ grib_options_help grib_options_help_list[] ={
   {"o:","output_grib_file",
    "\n\t\tOutput grib is written to ouput_grib_file."
    "\n\t\tIf an ouput grib file is required and -o is not used, the"
-   "ouput grib is written to filtered.out\n"},
+   " ouput grib is written to filtered.out\n"},
   {"p:","key[:{s/d/l}],key[:{s/d/l}],...",
    "\n\t\tDeclaration of keys to print."
    "\n\t\tFor each key a string (key:s) or a double (key:d) or a long (key:l)"
@@ -83,7 +79,7 @@ grib_options_help grib_options_help_list[] ={
   {"C",0,"C code mode. A C code program generating the grib message is dumped.\n"},
   {"D",0,"Debug mode.\n"},
   {"H",0,"Print octet content in hexadecimal format.\n"},
-  {"M",0,"Multi-grib support off. Turn off support for multiple fields in single grib message\n"},
+  {"M",0,"Multi-field support off. Turn off support for multiple fields in single grib message.\n"},
   {"O",0,"Octet mode. WMO documentation style dump.\n"},
   {"P:","key[:{s/d/l}],key[:{s/d/l}],...",
    "\n\t\tAs -p adding the declared keys to the default list.\n"},
@@ -93,16 +89,16 @@ grib_options_help grib_options_help_list[] ={
 "\n\t\tall=relative_error will compare all the floating point keys using relative_error. Default all=0.\n"},
   {"S",0,"Strict. Only grib messages matching all the constraints are copied to"
    "\n\t\tthe output file\n"},
+  {"T:","T | B","Message type. T->GTS, B->BUFR. The input file is interpreted according the message type.\n"},
   {"V",0,"Version.\n"},
   {"W:","width","\n\t\tMinimum width of each column in output. Default is 10.\n"},
-  {"x",0,"Fast parsing option, only headers are loaded.\n"}
+  {"x",0,"Fast parsing option, only headers are loaded.\n"},
+  {"k:","key1,key2,...","\n\t\tSpecify a list of keys to index on. By default the input files are indexed on the MARS keys.\n"}
 
 };
 
 int grib_options_help_count=sizeof(grib_options_help_list)/sizeof(grib_options_help);
 
-static void set_value(grib_values* value,char* str,int equal);
-
 char* grib_options_get_option(const char* id) {
   int i=0;
 
@@ -145,7 +141,7 @@ int grib_get_runtime_options(int argc,char** argv,grib_runtime_options* options)
 
 
   while ((c = getopt (argc, argv, optstr)) != -1) {
-    if (c == '?') usage(argv[0]);
+    if (c == '?') usage();
     i=0;
     while ( i < grib_options_count && grib_options[i].id[0] != c) i++;
     grib_options[i].on=1;
@@ -171,7 +167,13 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
   if (grib_options_on("B:"))
     options->orderby=grib_options_get_option("B:");
 
-  if (grib_options_on("x")) options->headers_only=1;
+  if (grib_options_on("x")) options->mode=MODE_HEADERS_ONLY;
+  if (grib_options_on("T:")) {
+	char* x=grib_options_get_option("T:");
+	if ( *x == 'T' ) options->mode=MODE_GTS;
+	else if ( *x == 'B' ) options->mode=MODE_BUFR;
+	else options->mode=MODE_GRIB;
+  }
 
   if (grib_options_on("F:"))
     options->format=grib_options_get_option("F:");
@@ -199,7 +201,7 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
   if (grib_options_on("G")) grib_gribex_mode_on(context);
 
   nfiles=argc-optind;
-  if ( nfiles < (1+has_output+has_input_extra) && !options->infile ) usage(argv[0]);
+  if ( nfiles < (1+has_output+has_input_extra) && !options->infile ) usage();
 
   if (has_input_extra) {
     options->infile_extra=(grib_tools_file*)calloc(1,sizeof(grib_tools_file));
@@ -255,9 +257,10 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
   }
   
   options->requested_print_keys_count=MAX_KEYS;
-  GRIB_CHECK_NOLINE(grib_options_get_keys(argv[0],karg,0,GRIB_TYPE_UNDEFINED,
-      options->requested_print_keys,&(options->requested_print_keys_count)),0);
-
+  ret = parse_keyval_string(grib_tool_name,karg,0,GRIB_TYPE_UNDEFINED,
+      options->requested_print_keys,&(options->requested_print_keys_count));
+  if (ret == GRIB_INVALID_ARGUMENT) usage();
+  GRIB_CHECK_NOLINE(ret,0);
 
   options->strict=grib_options_on("S");
 
@@ -276,26 +279,30 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
   if (grib_options_on("s:")) {
     sarg=grib_options_get_option("s:");
     options->set_values_count=MAX_KEYS;
-    ret=grib_options_get_keys(argv[0],sarg,1,GRIB_TYPE_UNDEFINED,options->set_values,&(options->set_values_count));
+    ret=parse_keyval_string(grib_tool_name,sarg,1,GRIB_TYPE_UNDEFINED,options->set_values,&(options->set_values_count));
+    if (ret == GRIB_INVALID_ARGUMENT) usage();
   }
 
   if (grib_options_on("R:")) {
     sarg=grib_options_get_option("R:");
     options->tolerance_count=MAX_KEYS;
-    ret=grib_options_get_keys(argv[0],sarg,1,GRIB_TYPE_DOUBLE,options->tolerance,&(options->tolerance_count));
+    ret=parse_keyval_string(grib_tool_name,sarg,1,GRIB_TYPE_DOUBLE,options->tolerance,&(options->tolerance_count));
+    if (ret == GRIB_INVALID_ARGUMENT) usage();
   }
 
   if (grib_options_on("b:")) {
     barg=grib_options_get_option("b:");
     options->set_values_count=MAX_KEYS;
-    ret=grib_options_get_keys(argv[0],barg,0,GRIB_TYPE_STRING,options->set_values,&(options->set_values_count));
+    ret=parse_keyval_string(grib_tool_name,barg,0,GRIB_TYPE_STRING,options->set_values,&(options->set_values_count));
+    if (ret == GRIB_INVALID_ARGUMENT) usage();
   }
 
   if (grib_options_on("c:")) {
     sarg=grib_options_get_option("c:");
     options->compare_count=MAX_KEYS;
-    ret=grib_options_get_keys(argv[0],sarg,0,GRIB_TYPE_UNDEFINED,options->compare,
+    ret=parse_keyval_string(grib_tool_name,sarg,0,GRIB_TYPE_UNDEFINED,options->compare,
                 &(options->compare_count));
+    if (ret == GRIB_INVALID_ARGUMENT) usage();
   }
   if (grib_options_on("e")) {
     for (i=0;i<names_count; i++) {
@@ -308,146 +315,13 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
   warg=grib_options_get_option("w:");
 
   options->constraints_count=MAX_KEYS;
-  ret=grib_options_get_keys(argv[0],warg,1,GRIB_TYPE_STRING,
+  ret=parse_keyval_string(grib_tool_name,warg,1,GRIB_TYPE_UNDEFINED,
       options->constraints,&(options->constraints_count));
+  if (ret == GRIB_INVALID_ARGUMENT) usage();
 
   return GRIB_SUCCESS;
 }
 
-int grib_options_get_keys(char* progname,char* arg,int values_required,int default_type,grib_values values[],int* count) {
-   char* p;
-   int i=0;
-   if (arg == NULL) {
-     *count=0;
-     return GRIB_SUCCESS;
-   }
-   p=strtok(arg,",");
-   while (p != NULL) {
-     values[i].name=(char*)calloc(1,strlen(p)+1);
-     strcpy((char*)values[i].name,p);
-     p=strtok(NULL,",");
-     i++;
-     if (i > *count) return GRIB_ARRAY_TOO_SMALL;
-   }
-   *count=i;
-
-   for (i=0;i<*count;i++) {
-     int equal=1;
-     char* value=NULL;
-     if (values_required) {
-       p=(char*)values[i].name;
-       while( *p != '=' && *p != '!' && *p != '\0') p++;
-       if (*p == '=') {
-         *p='\0';
-         p++;
-         value=p;
-         equal=1;
-       } else if (*p == '!' && *(++p) == '=' ) {
-         *p='\0';
-         *(p-1)='\0';
-         p++;
-         value=p;
-         equal=0;
-       } else usage( progname );
-     }
-     p=(char*)values[i].name;
-     while(*p != ':' && *p != '\0') p++;
-     if ( *p == ':' ) {
-       values[i].type = grib_type_to_int(*(p+1));
-       if (*(p+1)=='n') values[i].type = GRIB_NAMESPACE;
-       *p='\0';
-       p++;
-     } else {
-       values[i].type = default_type;
-     }
-     if (values_required) {
-      if (strlen(value)==0) {
-        printf("%s error: no value provided for key \"%s\"\n",grib_tool_name,values[i].name);
-        exit(GRIB_INVALID_ARGUMENT);
-      }
-      set_value(&values[i],value,equal);
-     }
-   }
-   return GRIB_SUCCESS;
-}
-
-static void set_value(grib_values* value,char* str,int equal) {
-  char *p=0,*q=0,*s=0;
-  char buf[1000]={0,};
-  grib_context* c=grib_context_get_default();
-  
-  value->equal=equal;
-  q=str;
-  
-  while (*q != '/' && *q!=0 ) q++;
-  if (*q=='/') {
-    s=grib_context_strdup(c,q+1);
-    value->next=grib_context_malloc_clear(c,sizeof(grib_values));
-    value->next->type=value->type;
-    value->next->name=grib_context_strdup(c,value->name);
-    set_value(value->next,s,equal);
-    grib_context_free(c,s);
-  }
-
-  memcpy(buf,str,q-str);
-  
-  switch (value->type) {
-    case GRIB_TYPE_DOUBLE:
-      value->double_value = strtod(buf,&p);
-      if(*p != 0) value->has_value=1;
-      else if (!strcmp(str,"missing") ||
-               !strcmp(str,"MISSING") ||
-               !strcmp(str,"Missing")  ) {
-        value->type=GRIB_TYPE_MISSING;
-        value->has_value=1;
-      }
-    break;
-    case GRIB_TYPE_LONG:
-      value->long_value = strtol(buf,&p,10);
-      if(*p != 0) value->has_value=1;
-      else if (!strcmp(buf,"missing") ||
-                !strcmp(buf,"MISSING") ||
-                !strcmp(buf,"Missing")  ) {
-        value->type=GRIB_TYPE_MISSING;
-        value->has_value=1;
-      }
-    break;
-    case GRIB_TYPE_STRING:
-      if ( !strcmp(buf,"missing") ||
-            !strcmp(buf,"MISSING") ||
-            !strcmp(buf,"Missing")  ) {
-            value->type=GRIB_TYPE_MISSING;
-            value->has_value=1;
-      } else {
-        value->string_value = grib_context_strdup(c,buf);
-        value->has_value=1;
-      }
-      break;
-    case GRIB_TYPE_UNDEFINED:
-      value->long_value = strtol(buf,&p,10);
-      if(*p == 0) {
-        value->type = GRIB_TYPE_LONG;
-        value->has_value=1;
-      } else {
-        value->double_value = strtod(buf,&p);
-        if(*p == 0) {
-            value->type = GRIB_TYPE_DOUBLE;
-            value->has_value=1;
-        } else if ( !strcmp(buf,"missing") ||
-                    !strcmp(buf,"MISSING") ||
-                    !strcmp(buf,"Missing")  ) {
-            value->type=GRIB_TYPE_MISSING;
-            value->has_value=1;
-        } else {
-          value->string_value = grib_context_strdup(c,buf);
-          value->type         = GRIB_TYPE_STRING;
-          value->has_value=1;
-        }
-      }
-      break;
-   }
-}
-
 char* grib_options_get_help(char* id) {
    int i=0;
    char msg[]="ERROR: help not found for option ";
