@@ -1,28 +1,20 @@
-/**
-* Copyright 2005-2007 ECMWF
-* 
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*
  * C Implementation: grib_debug
  *
  * Description: 
  *
- *
- * Author: Enrico Fucile <enrico.fucile@ecmwf.int>
- *
- *
  */
  
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-
 #include "grib_api_internal.h"
 void usage(const char* p)
 {
@@ -55,7 +47,7 @@ void try(const char* txt,double v,double ref,long bscale,long dscale)
 int main(int argc, char *argv[])
 {
 	extern char *optarg;
-	extern int optind, opterr, optopt;
+	extern int optind;
 	double min = 0,max = 100;
 	long bits = 16;
 	int g2 = 0;
