@@ -1,18 +1,16 @@
-/**
- * Copyright 2005-2007 ECMWF
+/*
+ * Copyright 2005-2013 ECMWF.
  *
- * Licensed under the GNU Lesser General Public License which
- * incorporates the terms and conditions of version 3 of the GNU
- * General Public License.
- * See LICENSE and gpl-3.0.txt for details.
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
 /*
  * C Implementation: grib_tools
  *
- * Author: Enrico Fucile <enrico.fucile@ecmwf.int>
- *
- *
  */
 
 #include "grib_tools.h"
@@ -35,6 +33,8 @@ static int grib_tool_index(grib_runtime_options* options);
 static int process(grib_context* c,grib_runtime_options* options,const char* path);
 static int scan(grib_context* c,grib_runtime_options* options,const char* dir);
 
+FILE* dump_file;
+
 grib_runtime_options options={
 		0,         /* verbose       */
 		0,         /* fail          */
@@ -95,7 +95,8 @@ grib_runtime_options options={
         0,         /* index2  */
         0,         /* context  */
         0,         /* stop  */
-        0,         /* headers_only  */
+        0,         /* mode  */
+        0,         /* skip_all  */
         {{0,},}    /* grib_values tolerance[MAX_KEYS] */
 
 	};
@@ -119,9 +120,13 @@ static int is_index_file(const char* filename) {
 	return ret;
 }
 
-static grib_handle* grib_handle_new_from_file_x(grib_context* c,FILE* f,int headers_only,int *err) {
-	if (headers_only) 
+static grib_handle* grib_handle_new_from_file_x(grib_context* c,FILE* f,int mode,int *err) {
+	if (mode==MODE_HEADERS_ONLY) 
 		return grib_handle_headers_only_new_from_file(c,f,err);
+	if (mode==MODE_GTS)
+		return grib_gts_handle_new_from_file(c,f,err);
+	if (mode==MODE_BUFR)
+		return grib_bufr_handle_new_from_file(c,f,err);
 
 	return grib_handle_new_from_file(c,f,err);
 }
@@ -204,8 +209,8 @@ static int grib_tool_with_orderby(grib_runtime_options* options) {
 	}
 
 	options->handle_count=0;
-	while((h = grib_fieldset_next_handle(set,&err))
-			!= NULL || err != GRIB_SUCCESS ) {
+	while(!options->skip_all && ((h = grib_fieldset_next_handle(set,&err))
+			!= NULL || err != GRIB_SUCCESS )) {
 		options->handle_count++;
 		options->error=err;
 
@@ -290,8 +295,8 @@ static int grib_tool_without_orderby(grib_runtime_options* options) {
 		grib_tool_new_file_action(options,infile);
 
 
-		while((h = grib_handle_new_from_file_x(c,infile->file,options->headers_only,&err))
-				!= NULL || err != GRIB_SUCCESS ) {
+		while(!options->skip_all && ((h = grib_handle_new_from_file_x(c,infile->file,options->mode,&err))
+				!= NULL || err != GRIB_SUCCESS ) ) {
 			infile->handle_count++;
 			options->handle_count++;
 			options->error=err;
@@ -540,6 +545,7 @@ static void grib_print_header(grib_runtime_options* options,grib_handle* h) {
 
 	size_t strlenkey=0;
 	int width;
+	int written_to_dump = 0; /* boolean */
 	if (!options->print_keys || options->handle_count!=1)
 		return;
 
@@ -555,7 +561,9 @@ static void grib_print_header(grib_runtime_options* options,grib_handle* h) {
 				options->default_print_width+2 : strlenkey+2;
             if (options->default_print_width < 0)
 				width=strlenkey+1;
+			
 			fprintf(dump_file,"%-*s",(int)width,options->print_keys[j].name);
+			written_to_dump = 1;
 		}
 		if (options->latlon) {
 			if (options->latlon_mode==4) {
@@ -563,12 +571,18 @@ static void grib_print_header(grib_runtime_options* options,grib_handle* h) {
 				fprintf(dump_file," value2 ");
 				fprintf(dump_file," value3 ");
 				fprintf(dump_file," value4 ");
-			} else fprintf(dump_file," value ");
+			} else {
+				fprintf(dump_file," value ");
+			}
+			written_to_dump = 1;
 		}
 		if (options->index_on) {
 			fprintf(dump_file,"        value(%d) ",(int)options->index);
+			written_to_dump = 1;
+		}
+		if (written_to_dump) {
+			fprintf(dump_file,"\n");
 		}
-		fprintf(dump_file,"\n");
 	}
 }
 
@@ -612,7 +626,7 @@ static void grib_tools_set_print_keys(grib_runtime_options* options, grib_handle
 
       grib_keys_iterator_delete(kiter);
       if (options->print_keys_count==0 && options->latlon == 0 ) {
-        printf("ERROR: name space \"%s\" does not contain any key\n",ns);
+        printf("ERROR: namespace \"%s\" does not contain any key\n",ns);
         exit(1);
       }
 
@@ -621,75 +635,77 @@ static void grib_tools_set_print_keys(grib_runtime_options* options, grib_handle
 }
 
 static int to_skip(grib_handle* h,grib_values* v,int *err) {
-  double dvalue=0;
-  int ret=0;
-  long lvalue=0;
-  char value[MAX_STRING_LEN]={0,};
-  size_t len=MAX_STRING_LEN;
-  *err=0;
-  
-  switch (v->type) {
-    case GRIB_TYPE_STRING:
-      *err=grib_get_string( h,v->name,value,&len);
-      ret = v->equal ? grib_inline_strcmp(value,v->string_value) : !grib_inline_strcmp(value,v->string_value);
-      break;
-    case GRIB_TYPE_DOUBLE:
-      *err=grib_get_double( h,v->name,&dvalue);
-      ret = v->equal ? (dvalue != v->double_value) : (dvalue == v->double_value);
-      break;
-    case GRIB_TYPE_LONG:
-      *err=grib_get_long( h,v->name,&lvalue);
-      ret = v->equal ? (lvalue != v->long_value) : (lvalue == v->long_value);
-      break;
-    case GRIB_TYPE_MISSING:
-      lvalue=grib_is_missing( h,v->name,err);
-      ret = (lvalue == v->equal) ? 0 : 1;
-      break;
+	double dvalue=0;
+	int ret=0;
+	long lvalue=0;
+	char value[MAX_STRING_LEN]={0,};
+	size_t len=MAX_STRING_LEN;
+	*err=0;
+
+	switch (v->type) {
+	case GRIB_TYPE_STRING:
+		*err=grib_get_string( h,v->name,value,&len);
+		ret = v->equal ? grib_inline_strcmp(value,v->string_value) : !grib_inline_strcmp(value,v->string_value);
+		break;
+	case GRIB_TYPE_DOUBLE:
+		*err=grib_get_double( h,v->name,&dvalue);
+		ret = v->equal ? (dvalue != v->double_value) : (dvalue == v->double_value);
+		break;
+	case GRIB_TYPE_LONG:
+		*err=grib_get_long( h,v->name,&lvalue);
+		ret = v->equal ? (lvalue != v->long_value) : (lvalue == v->long_value);
+		break;
+	case GRIB_TYPE_MISSING:
+		lvalue=grib_is_missing( h,v->name,err);
+		ret = (lvalue == v->equal) ? 0 : 1;
+		break;
 	default:
-	  fprintf(dump_file,"invalid type for %s\n",v->name);
-	  exit(1);
+		fprintf(dump_file,"invalid type for %s\n",v->name);
+		exit(1);
 
-  }
+	}
 
-  return ret;
+	return ret;
 }
 
 void grib_skip_check(grib_runtime_options* options,grib_handle* h) {
 	int i,ret=0;
-    grib_values* v=NULL;
-    for (i=0;i < options->constraints_count ;i++) {
-        v=&(options->constraints[i]);
-        if (v->equal) {
-          options->skip=1;
-          while (v) {
-            if (!to_skip(h,v,&ret)) {
-              options->skip=0;
-              break;
-            }
-            if (ret != GRIB_SUCCESS && options->fail) {
-                grib_context_log(h->context,GRIB_LOG_ERROR,"unable to get \"%s\" (%s)",
-                        v->name,grib_get_error_message(ret));
-                exit(ret);
-            }
-            v=v->next;
-          }
-        } else {
-          options->skip=0;
-          while (v) {
-            if (to_skip(h,v,&ret)) {
-              options->skip=1;
-              break;
-            }
-            if (ret != GRIB_SUCCESS && options->fail) {
-              grib_context_log(h->context,GRIB_LOG_ERROR,"unable to get \"%s\" (%s)",
-                               v->name,grib_get_error_message(ret));
-              exit(ret);
-            }
-            v=v->next;
-          }
-        }
-        if (options->skip==1)
-          break;
+	grib_values* v=NULL;
+	for (i=0;i < options->constraints_count ;i++) {
+		v=&(options->constraints[i]);
+		if (v->equal) {
+			options->skip=1;
+			while (v) {
+				if (!to_skip(h,v,&ret)) {
+					if (!strcmp(v->name,"count") && !v->next)
+						options->skip_all=1;
+					options->skip=0;
+					break;
+				}
+				if (ret != GRIB_SUCCESS && options->fail) {
+					grib_context_log(h->context,GRIB_LOG_ERROR,"unable to get \"%s\" (%s)",
+							v->name,grib_get_error_message(ret));
+					exit(ret);
+				}
+				v=v->next;
+			}
+		} else {
+			options->skip=0;
+			while (v) {
+				if (to_skip(h,v,&ret)) {
+					options->skip=1;
+					break;
+				}
+				if (ret != GRIB_SUCCESS && options->fail) {
+					grib_context_log(h->context,GRIB_LOG_ERROR,"unable to get \"%s\" (%s)",
+							v->name,grib_get_error_message(ret));
+					exit(ret);
+				}
+				v=v->next;
+			}
+		}
+		if (options->skip==1)
+			break;
 	}
 
 	if (!options->skip) {
@@ -702,11 +718,12 @@ void grib_skip_check(grib_runtime_options* options,grib_handle* h) {
 void grib_print_key_values(grib_runtime_options* options,grib_handle* h) {
 	int i=0;
 	int ret=0,width=0;
-	size_t strlenvalue=0;
+	int strlenvalue=0;
 	double dvalue=0;
 	long lvalue=0;
 	char value[MAX_STRING_LEN];
 	char* notfound="not_found";
+	int written_to_dump = 0; /* boolean */
 
 	if (!options->verbose) return;
 
@@ -717,25 +734,25 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h) {
 		if (grib_is_missing(h,options->print_keys[i].name,&ret) && ret==GRIB_SUCCESS)
 			sprintf(value,"MISSING");
 		else if ( ret == GRIB_SUCCESS ) {
-          if (options->print_keys[i].type == GRIB_TYPE_UNDEFINED)
-            grib_get_native_type(h,options->print_keys[i].name,&(options->print_keys[i].type));
+			if (options->print_keys[i].type == GRIB_TYPE_UNDEFINED)
+				grib_get_native_type(h,options->print_keys[i].name,&(options->print_keys[i].type));
 			switch (options->print_keys[i].type) {
-				case GRIB_TYPE_STRING:
-					ret=grib_get_string( h,options->print_keys[i].name,value,&len);
-					break;
-				case GRIB_TYPE_DOUBLE:
-					ret=grib_get_double( h,options->print_keys[i].name,&dvalue);
-					sprintf(value,options->format,dvalue);
-					break;
-				case GRIB_TYPE_LONG:
-					ret=grib_get_long( h,options->print_keys[i].name,&lvalue);
-					sprintf(value,"%d",(int)lvalue);
-					break;
-				default:
-					fprintf(dump_file,"invalid format option for %s\n",options->print_keys[i].name);
-					exit(1);
+			case GRIB_TYPE_STRING:
+				ret=grib_get_string( h,options->print_keys[i].name,value,&len);
+				break;
+			case GRIB_TYPE_DOUBLE:
+				ret=grib_get_double( h,options->print_keys[i].name,&dvalue);
+				sprintf(value,options->format,dvalue);
+				break;
+			case GRIB_TYPE_LONG:
+				ret=grib_get_long( h,options->print_keys[i].name,&lvalue);
+				sprintf(value,"%d",(int)lvalue);
+				break;
+			default:
+				fprintf(dump_file,"invalid format option for %s\n",options->print_keys[i].name);
+				exit(1);
 			}
-        }
+		}
 		if (ret != GRIB_SUCCESS) {
 			if (options->fail) GRIB_CHECK_NOLINE(ret,options->print_keys[i].name);
 			if (ret == GRIB_NOT_FOUND) strcpy(value,notfound);
@@ -745,35 +762,37 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h) {
 			}
 		}
 
-        strlenvalue = strlen(value);
+		strlenvalue = strlen(value);
 
-        width = strlenvalue < options->default_print_width ?
-                    options->default_print_width + 2 :
-                    strlenvalue + 2;
+		width = strlenvalue < options->default_print_width ?
+				options->default_print_width + 2 :
+				strlenvalue + 2;
 
-        if (options->default_print_width < 0) width = strlenvalue + 1;
-
-        if (options->print_keys_count==i+1 && options->latlon==0) width--;
+		if (options->default_print_width < 0) width = strlenvalue + 1;
 
+		if (options->print_keys_count==i+1 && options->latlon==0) width--;
 
 		fprintf(dump_file,"%-*s",(int)width,value);
+		written_to_dump = 1;
 	}
 
 	if (options->latlon) {
 
 		if (options->latlon_mode==4){
 			int i=0;
-            for (i=0;i<4;i++) {
-              fprintf(dump_file,options->format,options->values[i]);
-              fprintf(dump_file," ");
-            }
+			for (i=0;i<4;i++) {
+				fprintf(dump_file,options->format,options->values[i]);
+				fprintf(dump_file," ");
+			}
+			written_to_dump=1;
 		} else if (options->latlon_mode==1) {
 			sprintf(value,options->format,options->values[options->latlon_idx]);
 			strlenvalue = strlen(value);
 			width = strlenvalue < options->default_print_width ?
-                    options->default_print_width + 2 :
+					options->default_print_width + 2 :
 					strlenvalue + 2;
 			fprintf(dump_file,"%-*s",(int)width,value);
+			written_to_dump=1;
 		}
 	}
 	if (options->index_on) {
@@ -795,16 +814,18 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h) {
 			v=values[options->index];
 			grib_context_free(h->context,values);		
 		}
-		
+
 		sprintf(value,options->format,v);
 		strlenvalue = strlen(value);
 		width = strlenvalue < options->default_print_width ?
-                    options->default_print_width + 2 :
+				options->default_print_width + 2 :
 				strlenvalue + 2;
 		fprintf(dump_file,"%-*s",(int)width,value);
-		
+		written_to_dump=1;
+	}
+	if (written_to_dump) {
+		fprintf(dump_file,"\n");
 	}
-	fprintf(dump_file,"\n");
 
 }
 
