@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2007 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2013 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 #include "grib_api_internal.h"
 #include <stdio.h>
@@ -32,6 +33,11 @@
 #define MAX_FAILED  1024
 #define MAX_CONSTRAINT_VALUES 500
 
+#define MODE_GRIB 	      0
+#define MODE_HEADERS_ONLY 1
+#define MODE_GTS          2
+#define MODE_BUFR         3
+
 typedef union grib_typed_value {
   long*        long_value;
   double*      double_value;
@@ -144,7 +150,8 @@ typedef struct grib_runtime_options {
   grib_index* index2;
   grib_context* context;
   int stop;
-  int headers_only;
+  int mode;
+  int skip_all;
   grib_values tolerance[MAX_KEYS];
 } grib_runtime_options;
 
@@ -154,14 +161,13 @@ extern char* grib_tool_name;
 extern char* grib_tool_description;
 extern char* grib_tool_usage;
 
-FILE* dump_file;
+extern FILE* dump_file;
 
 char* grib_options_get_option(const char* id);
 int grib_options_on(const char* id);
 int grib_options_get(int argc,char** argv);
 int grib_options_get_values(int argc,char** argv,int values_required,int default_type, grib_values values[],int* n,int* optind);
 int grib_tool(int argc, char **argv);
-int grib_options_get_keys(char* progname,char* arg,int values_required,int default_type,grib_values values[],int* count);
 char* grib_options_get_help(char* id);
 char* grib_options_get_args(char* id);
 int grib_options_command_line(const char* id);
